#!/bin/sh
# 
#  *****************************************************************
#  *                                                               *
#  *    Copyright (c) Compaq Computer Corporation, 2000            *
#  *                                                               *
#  *   All Rights Reserved.  Unpublished rights  reserved  under   *
#  *   the copyright laws of the United States.                    *
#  *                                                               *
#  *   The software contained on this media  is  proprietary  to   *
#  *   and  embodies  the  confidential  technology  of  Compaq    *
#  *   Computer Corporation.  Possession, use,  duplication  or    *
#  *   dissemination of the software and media is authorized only  *
#  *   pursuant to a valid written license from Compaq Computer    *
#  *   Corporation.                                                *
#  *                                                               *
#  *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
#  *   by the U.S. Government is subject to restrictions  as  set  *
#  *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
#  *   or  in  FAR 52.227-19, as applicable.                       *
#  *                                                               *
#  *****************************************************************
#

usage()
{
    echo "Usage: create-comp-config.sh cxx-version [egcs-path]" >&2
}

aborting()
{
    cat 1>&2 <<EOF
Aborting

If you can correct the problem, you can rerun this script manually
by entering a command in the following format:

      create-comp-config.sh cxx-version [egcs-path]

The current invocation of create-comp-config.sh was:

$SAVE_PROG_NAME $SAVE_ARGS
EOF
    exit 1
}

generic_error()
{
    test_failed=$1
    test_file=$2
    error_log=$3

    cat 1>&2 <<EOF

The Compaq C++ Compiler failed its installation verification
procedure. Please submit a problem report to The University of New
Hampshire (UNH). UNH, in partnership with Compaq, is developing an
innovative support network for Alpha Linux developers. UNH supports a
web site and forums for Alpha Linux developers to exchange information
and ask questions.  To report problems experienced with Compaq C++ for 
Linux Alpha and its components, use the UNH website:

       http://www.iol.unh.edu/linux/alpha/development

UNH has a direct connection to Compaq's compiler and tools development
organization for fast resolution of identified technical problems.
You may not receive a personal reply to your query, but we appreciate
your input on the product.


When submitting your problem, please include the following:

      o All output generated by this installation procedure
      o The name and version of your Linux distribution
      o The name of the compiler RPM kit
      o The command you used to install the kit
      o The version of GCC installed on your system

EOF

    cxx -V -v 1>&2
    echo " " 1>&2
    
    echo "$test_failed" 1>&2
    if [ ! "$test_file" = "" ]; then
      cat $test_file 1>&2
    fi
    echo " " 1>&2

    cat $error_log 1>&2
    
    aborting
}


Make_compconfig_file()
{
    # RELOCATE_LD only if not relying on the ld symbolic link in the
    # bin directory.  This would need to precede the '|' in the
    # comp.config file so the user could override it.
    RELOCATE_LD=""

    if [ "$PREFIX" = "cxx" ]; then
       #
       # use ccc in /usr/bin
       #
       RELOCATE_CCC="-tk -h/usr/bin -B"

       #
       # Collect2 does demangling, until driver is smart enough to
       # stop this, we need to just use /usr/bin/ld
       #
       RELOCATE_LD="-tl -h/usr/bin -B"

       #
       # Specify how to tell linker to use Compaq demangler
       #
       DEMANGLE_SWITCH="-Wl,--$DEMANGLE=compaq"

       # RELOCATE_STDLIB
       # This would need to precede the '|' in the comp.config file so the 
       # user could override it, although I cannot see why they would.
       probe_libc
       if [ "`echo $LIBC | cut -f1,2 -d.`" = "2.0" ]; then
          RELOCATE_STDLIB="-ts -h${CCC_ROOT}/bin -B_rh52"
       else
          RELOCATE_STDLIB="-ts -h${CCC_ROOT}/bin -B_rh60"
       fi
    fi      

    # For any /usr/alpha-*linux directories containing "include" or
    # "lib" subdirectories, change the names of those directories
    # into appropriate -SysIncDir or -L command line arguments.

    # -SysIncDir options - if wildcard didn't expand, empty option string
    if [ "$USR_DIST_INC" = "`echo $USR_DIST_INC`" ]
    then
	H_DIRS=""
    else
	H_DIRS=`echo ' ' $USR_DIST_INC | sed -n -e 's| [^ ][^ ]*| -SysIncDir&|gp'`
    fi
    SYS_INCDIR="-SysIncDir ${CCC_ROOT}/include -SysIncDir /usr/local/include $H_DIRS -SysIncDir $1/include"

    if [ "$PREFIX" = "cxx" ]; then 
      SYS_INCDIR="-SysIncDir ${CCC_ROOT}/include/cxx $SYS_INCDIR"
    fi

    # For each -SysIncDir directory, generate a -SD option for that directory.
    # Unlike -SysIncDir, which has a whitespace-separated argument, the
    # argument to -SD must be attached.

    SD_OPTIONS=`echo ${SYS_INCDIR} | sed -e 's/-SysIncDir[ 	]*/-SD/g'`

    # -L options - if wildcard didn't expand, empty option string
    if [ "$USR_DIST_LIB" = "`echo $USR_DIST_LIB`" ]
    then
	L_DIRS=""
    else
	L_DIRS=`echo ' ' $USR_DIST_LIB | sed -n -e 's| \([^ ][^ ]*\)| -L\1|gp'`
    fi
    SYS_LIBDIR="-L${CCC_ROOT}/lib -L/usr/local/lib $L_DIRS -L$1"

#
#   C++ does not have -SysIncDir, replace with -I
#
    if [ "$PREFIX" = "cxx" ]; then
      SYS_INCDIR=`echo ${SYS_INCDIR} | sed -e 's/-SysIncDir[ 	]*/-I/g'`
    fi

    COMPCONFIG="${RELOCATE_CCC} ${RELOCATE_LD} ${RELOCATE_STDLIB} ${SD_OPTIONS} $2 $DEMANGLE_SWITCH | ${SYS_INCDIR} ${SYS_LIBDIR}"

    # It's not useful to compare the comp.config we're going to generate
    # with one shipped on the kit, because at best they would match for
    # only one distribution.  Just silently generate what we think is
    # right for the system we're on.
    echo "${COMPCONFIG}" > ${CCC_ROOT}/bin/comp.config
    ls -l ${CCC_ROOT}/bin/ld | grep -q  ">[ 	]*$1" || {
	if [ -x $1/ld ]
	then
	    mv ${CCC_ROOT}/bin/ld ${CCC_ROOT}/bin/ld.bak
	    ln -s $1/ld ${CCC_ROOT}/bin/ld
	elif [ -x $1/collect2 ]
	then
	    mv ${CCC_ROOT}/bin/ld ${CCC_ROOT}/bin/ld.bak
	    ln -s $1/collect2 ${CCC_ROOT}/bin/ld
	else
	    echo "Warning: cannot find ld or collect2 in $1" 1>&2
	fi
    }

    # Static kit assumes all of the .o's are in /usr/lib.  If a .o
    # of the same name is in the egcs directory we selected, use that
    # one, instead.

    for obj in gcrt1.o crt1.o crti.o crtbegin.o crtend.o crtn.o ; do
	if [ -L ${CCC_ROOT}/bin/${obj} -a -r $1/${obj} ]
	then
	    mv ${CCC_ROOT}/bin/${obj} ${CCC_ROOT}/bin/${obj}.bak
	    ln -s $1/${obj} ${CCC_ROOT}/bin/${obj}
	fi
    done
}

run_cxx()
{
  CXX_LOG="/tmp/cxx_install$$.log"

  rm -f /tmp/cxx_install$$
  /usr/bin/cxx -ptr /tmp/cxx_install$$_repo \
               -c $1 -o /tmp/cxx_install$$.o > $CXX_LOG 2>&1
  /usr/bin/cxx -ptr /tmp/cxx_install$$_repo \
               -o /tmp/cxx_install$$  /tmp/cxx_install$$.o >> $CXX_LOG 2>&1

  FAILED="0"
  if [ "`cat $CXX_LOG`" != "" ]; then
    FAILED="1"
  elif [ ! -x /tmp/cxx_install$$ ]; then
    FAILED="1"
  else
    /tmp/cxx_install$$ > /dev/null
    if [ ! "$?" = "0" ]; then
      FAILED="1"
    fi
  fi

  if [ "$FAILED" = "1" ]; then
    generic_error "Program compilation:" "$1" "$CXX_LOG"
  fi

#
# Cleanup after ourselves to be sure 
#
  rm -f /tmp/cxx_install$$ /tmp/cxx_install$$.log /tmp/cxx_install$$.o $1
  rm -rf /tmp/cxx_install$$_repo
}

run_ivp()
{
#
# If the current directory is not writable, then go to /tmp
# 
touch cxx_install$$.touch > /dev/null 2>&1
if [ $? -eq 0 ]; then
  rm -f cxx_install$$.touch
else
  cd /tmp
fi

#
# Verify the version of cpml we have is high enough
#
  package_high_enough cpml /usr/lib/libcpml.so 5.1.0
  status=$?

  if [ $status -eq 2 ]; then 
    cat 1>&2 <<EOF 
The Compaq Portable Math Library (CPML) is not installed. See the
README documentation provided with the compiler kit for instructions
to obtain a version appropriate for your system.
EOF
    aborting
  elif [ $status -eq 3 ]; then
    cat 1>&2 <<EOF
The Compaq Portable Math Library (CPML) installed on this system is
not new enough to support this version of the compiler. See the README
documentation provided with the compiler kit for instructions to
obtain a version appropriate for your system.  
EOF
    aborting 
  elif [ $status -eq 1 ]; then 
    generic_error "CPML Version Identification" "" /tmp/test_package_version$$ 
  fi

#
# Verify binutils with simple template example
#
  cat > /tmp/cxx_install$$.cxx <<EOF
template <class T> void f(const T&) { }
int main()
{
  f(1);
  return 0;    
}
EOF

  run_cxx /tmp/cxx_install$$.cxx

#
# Verify headers with stdio include
#
  cat > /tmp/cxx_install$$.cxx <<EOF
#include <stdio.h>
int main()
{
  printf("Hello world\n");
  return 0;    
}
EOF

  run_cxx /tmp/cxx_install$$.cxx

#
# Verify stdlib with cout test
#
  cat > /tmp/cxx_install$$.cxx <<EOF
#include <iostream>
int main()
{
  cout << "Hello world" << endl;
  return 0;    
}
EOF

  run_cxx /tmp/cxx_install$$.cxx
}
      
#
# Main Body

SAVE_PROG_NAME="$0"
SAVE_ARGS="$*"

CCC_VERSION="$1"
if [ "$1" = "" ] ; then
    usage
    exit 1
fi

GCC_PATH="$2"

#
# Load the functions used for probing the system information. Do this
# after we grab the arguments for this script, because they will be clobbered.
#
. `dirname $SAVE_PROG_NAME`/probe_linux.sh init

PREFIX=`echo $CCC_VERSION | cut -b 1-3`
if [ "$PREFIX" != "cxx" -a "$PREFIX" != "ccc" ]; then
    echo "Usage: create-comp-config.sh ccc-version [egcs-path]" >&2
    exit 1
fi

if [ "$PREFIX" = "ccc" ]; then
  TEMP=${CPQ_COMP_ROOT}/usr/lib/compaq/${CCC_VERSION}
  if [ -d ${TEMP}/alpha-linux ]
  then
    CCC_ROOT=${TEMP}/alpha-linux
  elif [ -d ${TEMP}/alpha-redhat-linux ]
  then
    CCC_ROOT=${TEMP}/alpha-redhat-linux
  fi
else
  CCC_ROOT=${CPQ_COMP_ROOT}/usr/lib/compaq/${CCC_VERSION}/alpha-linux
fi

# Put ccc rpm kit version in bin/.version file for use by ccc -V;
  echo "Installed as ${CCC_VERSION}" > ${CCC_ROOT}/bin/.version

# Put linux distribution identification info in bin/.version file for use 
# by ccc -V;

if [ "$DIST_VENDOR" = "UNKNOWN" ]; then
    echo "Warning: unknown distribution" \
	| tee -a ${CCC_ROOT}/bin/.version 1>&2
    if [ ! $DIST_FILES = "" ]; then
       cat $DIST_FILES | tee -a ${CCC_ROOT}/bin/.version 1>&2
    fi
else
    cat $DIST_FILES >> ${CCC_ROOT}/bin/.version
fi

#
# Figure our extra includes and libs we should be using
#
for dir in /usr/alpha-*linux/include ; do
    if [ "${dir}" != /usr/alpha-glibc20-linux/include ] ; then
	USR_DIST_INC="${USR_DIST_INC} ${dir}"
    fi
done 

for dir in /usr/alpha-*linux/lib ; do
    if [ "${dir}" != /usr/alpha-glibc20-linux/lib ] ; then
	USR_DIST_LIB="${USR_DIST_LIB} ${dir}"
    fi
done 

#
# Find version of GCC to use.
#
if [ "$GCC_PATH" = "" ]; then
  probe_gcc
  if [ "$GCC_PATH" = "Unknown GCC" ]; then
    echo "Error: Unable to find GCC. GCC must be installed." 1>&2
    echo "If GCC is installed, try adding it to your path." 1>&2
    aborting
  fi
fi

if [ ! -r "$GCC_PATH/specs" ]; then
  echo "Error: Unable to find GCC at $GCC_PATH. GCC must be installed." 1>&2
  aborting
fi

if [ "$PREFIX" = "cxx" ]; then
  probe_demangle
  if [ "$DEMANGLE" = "unsupported" ]; then

    cat 1>&2 <<EOF
The linker (ld) installed on this system does not support the Compaq
C++ Compiler.  The linker is provided as part of a set of tools known
as binutils. See the README documentation provided with the compiler
kit for instructions to obtain a version of binutils appropriate for
your system.

EOF
    aborting
  elif [ "$DEMANGLE" != "style" -a "$DEMANGLE" != "demangle" ]; then
      generic_error "Testing linker demangler support" "" "$DEMANGLE"
  fi
fi

#
# Create comp.config
#
Make_compconfig_file "$GCC_PATH"
echo "Using ${GCC_PATH}." >> ${CCC_ROOT}/bin/.version

run_ivp

if [ "$PREFIX" = "cxx" ]; then
#
# All done, reset /usr/bin/cxx to point to the license accepter
# 
  rm -f /usr/bin/cxx
  ln -s \
    ../lib/compaq/${CCC_VERSION}/alpha-linux/bin/lnxexx_driver.accept_license \
    /usr/bin/cxx
  chmod 4555 ${CCC_ROOT}/bin/lnxexx_driver.accept_license
fi

exit 0
