/**************************************************************************
 *
 * nthelem.cpp - Example program for rearranging a collection based on nth
 *               element. See Class Reference Section
 *
 * $Id: nthelem.cpp,v 1.13 1996/08/28 01:17:43 smithey Exp $
 *
 ***************************************************************************
 *    
 *  Copyright 2000 Compaq Computer Corporation
 *
 *  COMPAQ Registered in U.S. Patent and Trademark Office.
 *
 *  Confidential computer software. Valid license from Compaq required for
 *  possession, use or copying. Consistent with FAR 12.211 and 12.212,
 *  Commercial Computer Software, Computer Software Documentation, and
 *  Technical Data for Commercial Items are licensed to the U.S. Government
 *  under vendor's standard commercial license.
 *
 ***************************************************************************
 *
 * (c) Copyright 1994-1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/
 
 #ifndef _RWSTD_HEADER_REQUIRES_HPP
 #include<algorithm>
 #include<vector>
 #else
 #include<algorithm.hpp>
 #include<vector.hpp>
 #endif
 
 #ifdef __USE_STD_IOSTREAM
 #include <iostream>
 #else
 #include <iostream.h>
 #endif     

#ifndef _RWSTD_NO_NAMESPACE
  using namespace std;
#endif

template<class RandomAccessIterator>
 void quik_sort(RandomAccessIterator start, RandomAccessIterator end)
 {
   size_t dist = 0;
#if defined(__DECCXX) && !defined(__DECFIXCXXL542) && !defined(_RWSTD_NO_CLASS_PARTIAL_SPEC)
  dist = distance(start, end);
#else
   distance(start, end, dist);
#endif
   //
   // Stop condition for recursion.
   //
   if(dist > 2)
   {
     //
     // Use nth_element to do all the work for quik_sort.
     //
     nth_element(start, start+(dist/2), end);
     //
     // Recursive calls to each remaining unsorted portion.
     //
     quik_sort(start, start+(dist/2-1));
     quik_sort(start+(dist/2+1), end);
   }

   if(dist == 2 && *end < *start)
     swap(start, end);
 }

 int main ()
 {
   //
   // Initialize a vector using an array of integers.
   //
   int arr[10] = {37, 12, 2, -5, 14, 1, 0, -1, 14, 32};
   vector<int,allocator<int> > v(arr+0, arr+10);
   //
   // Print the initial vector.
   //
   cout << "The unsorted values are: " << endl << "     ";
   vector<int,allocator<int> >::iterator i; 
   for(i = v.begin(); i != v.end(); i++)
     cout << *i << ", ";
   cout << endl << endl;
   //
   // Use the new sort algorithm.
   //
   quik_sort(v.begin(), v.end());
   //
   // Output the sorted vector.
   //
   cout << "The sorted values are: " << endl << "     ";
   for(i = v.begin(); i != v.end(); i++)
     cout << *i << ", ";
   cout << endl << endl;

   return 0;
 }

