#ifndef		__CXXL_HXX
#define		__CXXL_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	General-purpose utilities
//
//  MODULE:	cxxl
//
//  INFORMATION HIDDEN:
//
//	General-purpose aspects of the platform which is being used.
//
//	The set of available packages in the DEC C++ Class Library.
//
//  MODULE DESCRIPTION:
//
//	This header is used for misc. platform-hiding definitions or other,
//	possibly "temporary", definitions which may affect every module in the
//	DEC C++ Class Library.
//
//	This header is used for things which know the set of available
//	packages, such as the message set numbers passed to the Messages class
//	constructor by other packages which issue messages, and how they are
//	initialized before they are used at run-time.
//
//  IMPLICIT INPUT PARAMETERS:
//
//	__DECCXX:
//	    Defined:	Identifies a DEC C++ compiler.
//	    Undefined:	Identifies another C++ compiler.
//			(Probably for an older C++ dialect.)
//	__CXXL_MODULE_NAME:
//	    Defined:	Identifies a DEC C++ Class Library (CXXL) source module.
//	    Undefined:	Identifies a module which is using CXXL.
//
//	__CXXL_MODULE_VERSION:
//	    The CXXL module version number (if __CXXL_MODULE_NAME is defined).
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	__CXXL_PI:	Defined as the value for pi.
//	__CXXL_PI2:	Defined as the value for 2pi.
//	__SCOPE(x):	Expands to nothing for older dialects, x for newer; used
//			where scope classname:: is now needed.
//	__NOSCOPE(x):	Expands to x for older dialects, nothing for newer; used
//			where size of array to be deleted is no longer needed.
//	volatile:	Only defined to be nothing for older dialects.
//
//  DESIGN ISSUES:
//
//	Preprocessor symbols are defined to hide platform-dependencies.
//
//	On VMS, preprocessor code is used to construct the prefixes required by
//	the naming conventions.  pragma __extern_prefix is used to cause the
//	compiler to prepend the appropriate prefixes.  Also, pragma
//	__extern_model is used to select the appropriate external linkage model.
//
//	In order to ensure that message set numbers etc. are initialized before
//	they can be used, a class similar to (Iostream_init in the 'iostream'
//	package) is used.
//
//	In order to isolate other code from changes to the message set numbers
//	etc., local constants are not used.
//
//--



//
//
//  MACRO DEFINITIONS FOR SHAREABLE IMAGES.
//  These macros must be defined before any class declarations.
//
//

#ifdef _NT
#include <cxxl_sha.hxx>
#else
#include <cxxl_share.hxx>
#endif



//
//
//  LINKAGE
//
//

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   if defined __VAX
#      pragma __nomember_alignment
#   else
#      pragma __member_alignment
#   endif
#endif


//
//
//  MACRO DEFINITIONS
//
//

// This will have to be revised for the precision available on the platform

#ifndef __CXXL_PI
#   define	__CXXL_PI	3.14159265358979323846
#endif

#ifndef __CXXL_PI2
#   define __CXXL_PI2	(2 * __CXXL_PI)
#endif

#   define __SCOPE(x) x		/* expand to parameter */
#   define __NOSCOPE(x)		/* expand to nothing */

//
//
//  STATIC DATA
//
//

// When building CXXL itself, store the module name and version (etc.)
#if defined(__CXXL_MODULE_NAME)

#   if defined(__VMS) && defined(__DECCXX) && !defined (__CXXL_INIT_CXX)
#	pragma __module \
	    __CXXL_FLOAT_OPTION_MODULE_EVAL( \
		__CXXL_FLOAT_FUNC_PREFIX, \
		__CXXL_MODULE_NAME \
	    ) \
	    __CXXL_MODULE_VERSION
#   endif

    // Note that __FILE__ is useless here; it always points to ./cxxl.hxx and
    // not the name of the module top-level source file.  This also seems to
    // apply to the case where a macro is defined as __FILE__ in the module
    // top-level source file and that macro is used here.  So we have to settle
    // for using the module name instead of the file name in the "what" string
    // (since we don't want to go modify every module top-level source file to
    // have the following gunk in it).  (An alternative might be to define a
    // macro for this in here and then invoke the macro in every top-level
    // source file.)

    static const char *__cxxl_ident =
	"@(#)"				// recognized by "what" on ULTRIX (etc.)
	__CXXL_QUOTED_STRING_EVAL(__CXXL_MODULE_NAME) "\t"
	__CXXL_MODULE_VERSION "\t"
	"DEC C++ Class Library\t"
	__DATE__ "\t"
	__TIME__;

#endif

//
//
//  TYPE DEFINITIONS
//
//

// type for an integer register (for when we have to get at the machine itself)

#if defined(__VMS) && defined(__alpha) && defined(__32BITS)
typedef __int64 __cxxl_register_t;
#else
typedef long __cxxl_register_t;
#endif

class __cxxl_packages_t

//++
//  CLASS DESCRIPTION:
//
//	This class has information on which packages exist in the DEC C++ Class
//	Library and where their messages are located.
//
//  DESIGN:
//
//	In order to reduce name space pollution, static class members are used
//	instead of global variables or local constants in every module.  They
//	are initialized by __cxxl_init_t::initialize().
//
//--

{
public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    static const char *__CXXL_XFER_DECL(file_name);
    static const char *__CXXL_XFER_DECL(file_location);
    static int __CXXL_XFER_DECL(set_cxxl);
    static int __CXXL_XFER_DECL(set_complex);
    static int __CXXL_XFER_DECL(set_generic);
    static int __CXXL_XFER_DECL(set_iostream);
    static int __CXXL_XFER_DECL(set_objection);
    static int __CXXL_XFER_DECL(set_stopwatch);
    static int __CXXL_XFER_DECL(set_string);
    static int __CXXL_XFER_DECL(set_task);
    static int __CXXL_XFER_DECL(set_vector);
    static int __CXXL_XFER_DECL(set_messages);
};

class __cxxl_init_t

//++
//  CLASS DESCRIPTION:
//
//	Constructing an object of this class initializes the CXXL package if it
//	has not yet been initialized, in particular, the static data members of
//	the __cxxl_packages_t class.
//
//  DESIGN:
//
//	This class is modeled after the Iostream_init class.
//
//--

{
private:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    static int __CXXL_XFER_DECL(initcount);

public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    __cxxl_init_t();
    ~__cxxl_init_t();

    static void initialize();
};



#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// end of cxxl.hxx
