
#ifndef		__CXXL_SHARE_HXX
#define		__CXXL_SHARE_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	General-purpose utilities
//
//  MODULE:	cxxl_share.hxx
//
//  INFORMATION HIDDEN:
//
//	Specific aspects of shareable image implementation on the platform 
//	which is being used.
//
//  MODULE DESCRIPTION:
//
//	This header is used for defining macros which are used to support 
//	shareable images on various platforms.
//
//  IMPLICIT INPUT PARAMETERS:
//
//	__CXXL_MODULE_NAME:
//	    Defined:	Identifies the module that includes this header as 
//			belonging to Digital's implementation of the C++ Class 
//			Library.  Several macros are defined in this case.
//	    Undefined:	Identifies the module that includes this header as
//			not part of Digital's C++ Class Library implementation.
//
//	__CXXL_NO_XFER_VEC:
//	    Defined:	Prevent creation of indirection for use through a 
//			transfer vector on platforms where one normally would 
//			be created.
//	    Undefined:	Create indirection for use through a transfer vector 
//			on platforms where one normally would be created.
//
//	__CXXL_INCLUDE_ONLY_DOUBLE_FLOAT:   See below.
//
//	__CXXL_INCLUDE_ONLY_EITHER_FLOAT:   See below.
//
//	__VMS:
//	    Defined:	Run on the OpenVMS operating system.
//	    Undefined:	Run on another operating system.
//
//  DESIGN ISSUES:
//
//	Preprocessor symbols are defined to hide platform-dependencies.
//
//	On VMS, preprocessor code is used to construct the prefixes required by
//	the naming conventions.  pragma __extern_prefix is used to cause the
//	compiler to prepend the appropriate prefixes.  Also, pragma
//	__extern_model is used to select the appropriate external linkage model.
//
//	In order to ensure that message set numbers etc. are initialized before
//	they can be used, a class similar to (Iostream_init in the 'iostream'
//	package) is used.
//
//	In order to isolate other code from changes to the message set numbers
//	etc., local constants are not used.
//
//--



//
//
//  LINKAGE PREFIX DEFINITIONS
//
//

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#endif

#define __CXXL_INCLUDING_NO_FLOAT	 1
#define __CXXL_INCLUDING_SINGLE_FLOAT    1
#define __CXXL_INCLUDING_DOUBLE_FLOAT    1
#define __CXXL_INCLUDING_EITHER_FLOAT    1

//--------------------------------------------------------------------------
// Macros to put quotes around names that are constructed out of
// separate pieces.
//--------------------------------------------------------------------------

#define __CXXL_QUOTED_STRING(CXXL_SYMBOL)	  #CXXL_SYMBOL
#define __CXXL_QUOTED_STRING_EVAL(CXXL_SYMBOL) \
	      __CXXL_QUOTED_STRING(CXXL_SYMBOL)

#if defined(__VMS) && defined(__DECCXX)

    //--------------------------------------------------------------------------
    // Define the prefixes that will be used in generating both the module 
    // name (which is NOT a quoted string) and the #pragma __extern_prefix
    // (which is a quoted string).  
    //
    // Use __CXXL_xxxxx_xxxx_PREFIX  in building the full module name.
    // Use __CXXL_xxxxx_xxxx_LINKAGE with #pragma __extern_prefix.
    //--------------------------------------------------------------------------

#   if defined(__VMS_POSIX)
#	define __CXXL_PLAIN_FUNC_PREFIX	PXXL$
#	define __CXXL_PREFIX(body)	PXXL$ ## body
#    else
#	define __CXXL_PLAIN_FUNC_PREFIX	CXXL$
#	define __CXXL_PREFIX(body)	CXXL$ ## body
#    endif
#   define __CXXL_PLAIN_DATA_PREFIX	__CXXL_PREFIX(GA_)

#   if (__G_FLOAT)
#     define __CXXL_FLOAT_FUNC_PREFIX	__CXXL_PREFIX(FG_)
#     define __CXXL_FLOAT_DATA_PREFIX	__CXXL_PREFIX(GA_FG_)
#   elif (__IEEE_FLOAT)
#     define __CXXL_FLOAT_FUNC_PREFIX	__CXXL_PREFIX(ST_)
#     define __CXXL_FLOAT_DATA_PREFIX	__CXXL_PREFIX(GA_ST_)
#   else
#     define __CXXL_FLOAT_FUNC_PREFIX	__CXXL_PREFIX(FD_)
#     define __CXXL_FLOAT_DATA_PREFIX	__CXXL_PREFIX(GA_FD_)
#   endif

    //--------------------------------------------------------------------------
    // Define the linkage names to be used in #pragma __external_prefix
    // Use __CXXL_xxxxx_xxxx_LINKAGE with #pragma __extern_prefix.
    //--------------------------------------------------------------------------

#   define __CXXL_PLAIN_FUNC_LINKAGE    __CXXL_QUOTED_STRING_EVAL(__CXXL_PLAIN_FUNC_PREFIX)
#   define __CXXL_PLAIN_DATA_LINKAGE    __CXXL_QUOTED_STRING_EVAL(__CXXL_PLAIN_DATA_PREFIX)
#   define __CXXL_FLOAT_FUNC_LINKAGE	__CXXL_QUOTED_STRING_EVAL(__CXXL_FLOAT_FUNC_PREFIX)
#   define __CXXL_FLOAT_DATA_LINKAGE	__CXXL_QUOTED_STRING_EVAL(__CXXL_FLOAT_DATA_PREFIX)

    //--------------------------------------------------------------------------
    // Generate the module name for this compilation.  It is created by
    // concatinating the proper prefix with the module name.
    //--------------------------------------------------------------------------

#   define  __CXXL_FLOAT_OPTION_MODULE(CXXL_LINKAGE_PREFIX, CXXL_MODULE_NAME) \
	      CXXL_LINKAGE_PREFIX##CXXL_MODULE_NAME

#   define __CXXL_FLOAT_OPTION_MODULE_EVAL(CXXL_LINKAGE_PREFIX, CXXL_MODULE_NAME) \
	     __CXXL_FLOAT_OPTION_MODULE(CXXL_LINKAGE_PREFIX, CXXL_MODULE_NAME)


#   if defined(__CXXL_MODULE_NAME)

      //------------------------------------------------------------------------
      // Switches to select which routines to be included for compiles with
      // floating point selection switches.
      //
      // Each section of the CXXL source code will have one of the following
      // conditionals surrounding it.
      //
      //    #if __CXXL_INCLUDING_NO_FLOAT	    or
      //    #if __CXXL_INCLUDING_SINGLE_FLOAT	    or
      //    #if __CXXL_INCLUDING_DOUBLE_FLOAT	    or
      //    #if __CXXL_INCLUDING_EITHER_FLOAT
      //
      // By default ALL of these values will be 1.
      //
      // When compiling the CXXL source code multiple times to get different 
      // floating point representations, only portions of the source code 
      // should be included on subsequent compiles.  If it is necessary to
      // exclude portions of the source, define the following to get the
      // portion of the source code that is needed.
      //
      //    $ CXX/define=__CXXL_INCLUDE_ONLY_DOUBLE_FLOAT
      //	will include only those portions surrounded with:
      //	    __CXXL_INCLUDING_DOUBLE_FLOAT	      and
      //	    __CXXL_INCLUDING_EITHER_FLOAT
      //
      //    $ CXX/define=__CXXL_INCLUDE_ONLY_EITHER_FLOAT
      //	will include only those portions surrounded with:
      //	    __CXXL_INCLUDING_SINGLE_FLOAT	      and
      //	    __CXXL_INCLUDING_DOUBLE_FLOAT	      and
      //	    __CXXL_INCLUDING_EITHER_FLOAT
      //
      //------------------------------------------------------------------------

#     if defined(__CXXL_INCLUDE_ONLY_DOUBLE_FLOAT)
#	undef  __CXXL_INCLUDING_NO_FLOAT
#	undef  __CXXL_INCLUDING_SINGLE_FLOAT
#	define __CXXL_INCLUDING_NO_FLOAT      0
#	define __CXXL_INCLUDING_SINGLE_FLOAT  0
#     endif

#     if defined(__CXXL_INCLUDE_ONLY_EITHER_FLOAT)
#	undef  __CXXL_INCLUDING_NO_FLOAT
#	define __CXXL_INCLUDING_NO_FLOAT      0
#     endif

#   endif
#endif



//
//
//  SHAREABLE IMAGE TRANSFER VECTOR MACROS
//
//

//++
// __CXXL_XFER_VEC should be defined as 1 when compiling class library code
// for shared images that contain a transfer vector and 0 otherwise.  To disable
// this from the command line (e.g. sometimes useful for building a copy of _the
// library_ for debugging purposes), use /DEFINE=__CXXL_NO_XFER_VEC ... but do
// NOT reference __CXXL_NO_XFER_VEC elsewhere in the code.
//--

#if defined(__VMS) && defined(__DECCXX) && !defined(__CXXL_NO_XFER_VEC)
#   define __CXXL_XFER_VEC      1
#else
#   define __CXXL_XFER_VEC      0
#endif

#undef __CXXL_NO_XFER_VEC		/* prevent extraneous uses */

//++
//  MACRO NAME: __CXXL_XFER_DECL
//
//  FUNCTIONAL DESCRIPTION:
//
//	Change name of identifier to a reference to that identifier (adds one
//	level of indirection) if __CXXL_XFER_VEC is defined to be 1.  
//	Otherwise the identifier will be left alone.
//
//	To be used in declarations of static member or global non-member data
//	that need to be placed in a transfer vector for shareable image.
//
//	example:	extern int __CXXL_XFER_DECL(mydata);
//
//  FORMAL PARAMETERS:
//
//	NAME: Identifier being declared.
//
//  EXPANSION VALUE:
//
//	The identifier itself or prepended by &.
//
//--

#if __CXXL_XFER_VEC
#   define __CXXL_XFER_DECL(NAME)		      (&NAME)
#else
#   define __CXXL_XFER_DECL(NAME)		      NAME
#endif

//++
//  MACRO NAME: __CXXL_XFER_DEF
//
//  FUNCTIONAL DESCRIPTION:
//
//	Change name of identifier to the name of the target data that will be
//	pointed to by the tranfer vector if __CXXL_XFER_VEC is defined to be 1. 
//	Otherwise the only the identifier will be used with no modifications.
//
//	To be used in definitions of global non-member data that need to be 
//	placed in a transfer vector for shareable image.
//
//	example:	extern int __CXXL_XFER_DEF(mydata);
//
//  FORMAL PARAMETERS:
//
//	NAME:	Identifier being defined.
//
//  EXPANSION VALUE:
//
//	Either the identifier with no modifications or suffixed by the 
//	string "_data".
//
//  DESIGN:
//
//	The ## operator is used to concatenate identifier segments.
//
//--

#if __CXXL_XFER_VEC
#   define __CXXL_XFER_DEF(NAME)		      NAME##_data
#else
#   define __CXXL_XFER_DEF(NAME)		      NAME
#endif

//++
//  MACRO NAME: __CXXL_XFER_MEM_DEF
//
//  FUNCTIONAL DESCRIPTION:
//
//	Change name of identifier to the name of the target data that will be
//	pointed to by the tranfer vector if __CXXL_XFER_VEC is defined to be 1. 
//	Otherwise the identifier will just be prefixed with "CLASS::"
//
//	To be used in definitions of static member data that need to be 
//	placed in a transfer vector for shareable image.
//
//	example:	extern int __CXXL_XFER_MEM_DEF(myclass, mydata);
//
//  FORMAL PARAMETERS:
//
//	CLASS:	The class or package which is defining this data.
//	NAME:	Identifier being defined.
//
//  EXPANSION VALUE:
//
//	This will either expand to the identifier suffixed with "_data" when
//	__CXXL_XFER_VEC is 1 or prefixed with CLASS:: otherwise.
//
//  DESIGN:
//
//	The ## operator is used to concatenate identifier segments.
//
//--

#if __CXXL_XFER_VEC
#   define __CXXL_XFER_MEM_DEF(CLASS, NAME)	      CLASS##_##NAME##_data
#else
#   define __CXXL_XFER_MEM_DEF(CLASS, NAME)	      CLASS::NAME
#endif

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#endif

#endif
// end of cxxl_share.hxx
