/***************************************************************************
 *
 * deque.cc - Non-iniline definitions for the Standard Library deque class
 *
 ***************************************************************************
 *    
 *  Copyright 2000 Compaq Computer Corporation
 *
 *  COMPAQ Registered in U.S. Patent and Trademark Office.
 *
 *  Confidential computer software. Valid license from Compaq required for
 *  possession, use or copying. Consistent with FAR 12.211 and 12.212,
 *  Commercial Computer Software, Computer Software Documentation, and
 *  Technical Data for Commercial Items are licensed to the U.S. Government
 *  under vendor's standard commercial license.
 *
 ****************************************************************************
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Hewlett-Packard Company makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 *
 ***************************************************************************
 *
 * (c) Copyright 1994, 1998 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/
#include <stdcomp>

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __save
#      pragma __environment __header_defaults
#   endif
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#pragma __extern_prefix __save
#pragma __extern_prefix "CXXL$" 
#endif

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif


  template <class T, class Allocator>
  deque<T,Allocator>::~deque()
  {
    while (!empty()) pop_front();
  }

  template <class T, class Allocator>
  deque<T,Allocator>::size_type deque<T,Allocator>::_RWbuffer_size ()
  {
    static size_type b_size = 
    max((size_type)1,__RWSTD::_RWrw_allocation_size((value_type*)0,(size_type)0,(size_type)0));
    return b_size; 
  }

  template <class T, class Allocator>
  void deque<T,Allocator>::_RWallocate_at_begin ()
  {
    pointer p = _RWvalue_alloc_type(_RWmap_size).allocate(_RWbuffer_size(),_RWstart.current);
    if (!empty())
    {
      if (_RWstart.node == _RWmap)
      {
        _RWmap_alloc_type ma(_RWmap_size);
        difference_type i = _RWfinish.node - _RWstart.node;
        size_type new_map_size = (i + 1) * 2;
        _RWmap_pointer tmp;
#ifndef _RWSTD_NO_EXCEPTIONS
        try {
          tmp = ma.allocate(new_map_size,_RWmap);
        } catch(...) {
          _RWvalue_alloc_type(_RWmap_size).deallocate(p,_RWbuffer_size());
          throw;
        }      
#else
        tmp = ma.allocate(new_map_size,_RWmap);
#endif //  _RWSTD_NO_EXCEPTIONS
        copy(_RWstart.node, _RWfinish.node + 1, tmp + new_map_size / 4 + 1);
        ma.deallocate(_RWmap,_RWmap_size.data());
        _RWmap = tmp;
        _RWmap[new_map_size / 4] = p;
        _RWstart = iterator(p + _RWbuffer_size(), _RWmap + new_map_size / 4);
        _RWfinish = iterator(_RWfinish.current, _RWmap + new_map_size / 4 + i + 1);
        _RWmap_size = new_map_size;
      }
      else
      {
        *--_RWstart.node = p;
        _RWstart = iterator(p + _RWbuffer_size(), _RWstart.node);
      }
    }
    else
    {
#ifndef _RWSTD_NO_EXCEPTIONS
      try {
        _RWmap = _RWmap_alloc_type(_RWmap_size).allocate(_RWbuffer_size(),_RWmap);
      } catch(...) {
        _RWvalue_alloc_type(_RWmap_size).deallocate(p,_RWbuffer_size());
        throw;
      }      
#else
      _RWmap = _RWmap_alloc_type(_RWmap_size).allocate(_RWbuffer_size(),_RWmap);
#endif // _RWSTD_NO_EXCEPTIONS
      _RWmap_size = _RWbuffer_size();
      _RWmap[_RWmap_size.data() / 2] = p;
      _RWstart = iterator(p + _RWbuffer_size() / 2 + 1, _RWmap + _RWmap_size.data() / 2);
      _RWfinish = _RWstart;
    }
  }


  template <class T, class Allocator>
  void deque<T,Allocator>::_RWallocate_at_end ()
  {
    pointer p = _RWvalue_alloc_type(_RWmap_size).allocate(_RWbuffer_size(),_RWstart.current);

    if (!empty())
    {
      if (_RWfinish.node == _RWmap + _RWmap_size.data() - 1)
      {
        _RWmap_alloc_type ma(_RWmap_size);
        difference_type i = _RWfinish.node - _RWstart.node;
        size_type new_map_size = (i + 1) * 2;
        _RWmap_pointer tmp;
#ifndef _RWSTD_NO_EXCEPTIONS
        try {
          tmp = ma.allocate(new_map_size,_RWmap);
        } catch(...) {
          _RWvalue_alloc_type(_RWmap_size).deallocate(p,_RWbuffer_size());
          throw;
        }      
#else
        tmp = ma.allocate(new_map_size,_RWmap);
#endif // _RWSTD_NO_EXCEPTIONS
        copy(_RWstart.node, _RWfinish.node + 1, tmp + new_map_size / 4);
        ma.deallocate(_RWmap,_RWmap_size.data());
        _RWmap = tmp;
        _RWmap[new_map_size / 4 + i + 1] = p;
        _RWstart = iterator(_RWstart.current, _RWmap + new_map_size / 4);
        _RWfinish = iterator(p, _RWmap + new_map_size / 4 + i + 1);
        _RWmap_size = new_map_size;
      }
      else
      {
        *++_RWfinish.node = p;
        _RWfinish = iterator(p, _RWfinish.node);
      }
    }
    else
    {
      _RWmap_size = _RWbuffer_size();
#ifndef _RWSTD_NO_EXCEPTIONS
      try {
        _RWmap = _RWmap_alloc_type(_RWmap_size).allocate(_RWmap_size.data(),_RWmap);
      } catch(...) {
        _RWvalue_alloc_type(_RWmap_size).deallocate(p,_RWbuffer_size());
        throw;
      }      
#else
      _RWmap = _RWmap_alloc_type(_RWmap_size).allocate(_RWmap_size.data(),_RWmap);
#endif // _RWSTD_NO_EXCEPTIONS
      _RWmap[_RWmap_size.data() / 2] = p;
      _RWstart = iterator(p + _RWbuffer_size() / 2, _RWmap + _RWmap_size.data() / 2);
      _RWfinish = _RWstart;
    }
  }

  template <class T, class Allocator>
  void deque<T,Allocator>::_RWdeallocate_at_begin ()
  {
    _RWvalue_alloc_type(_RWmap_size).deallocate(*_RWstart.node++,_RWbuffer_size());
    if (empty())
    {
      _RWstart = iterator();
      _RWfinish = _RWstart;
      _RWmap_alloc_type(_RWmap_size).deallocate(_RWmap,_RWmap_size.data());
    }
    else
      _RWstart = iterator(*_RWstart.node, _RWstart.node);
  }

  template <class T, class Allocator>
  void deque<T,Allocator>::_RWdeallocate_at_end ()
  {
    _RWvalue_alloc_type(_RWmap_size).deallocate(*_RWfinish.node--,_RWbuffer_size());
    if (empty())
    {
      _RWstart = iterator();
      _RWfinish = _RWstart;
      _RWmap_alloc_type(_RWmap_size).deallocate(_RWmap,_RWmap_size.data());
    }
    else
      _RWfinish = iterator(*_RWfinish.node + _RWbuffer_size(), _RWfinish.node);
  }

  template <class T, class Allocator>
  void deque<T,Allocator>::resize (size_type new_size)
  {
    T value;
    if (new_size > size())
      insert(end(), new_size - size(), value);
    else if (new_size < size())
      erase(begin() + new_size,end());
  }

  template <class T, class Allocator>
  void deque<T,Allocator>::resize (size_type new_size, T value)
  {
    if (new_size > size())
      insert(end(), new_size - size(), value);
    else if (new_size < size())
      erase(begin() + new_size,end());
  }

  template <class T, class Allocator>
  _TYPENAME deque<T,Allocator>::iterator 
  deque<T,Allocator>::insert (iterator position, const T& x)
  {
    if (position == begin())
    {
      push_front(x); return begin();
    }
    else if (position == end())
    {
      push_back(x); return end() - 1;
    }
    else
    {
      difference_type index = position - begin();
#if defined(__DECCXX) && !defined(__DECFIXCXXL1306)
      if ((size_type)index < _RWlength / 2)
#else
      if ((size_type)index < _RWlength)
#endif
      {
        push_front(*begin());
        copy(begin() + 2, begin() + index + 1, begin() + 1);
      }
      else
      {
        push_back(*(end() - 1));
        copy_backward(begin() + index, end() - 2, end() - 1);
      }
      *(begin() + index) = x;
      return begin() + index;
    }
  }

  template <class T, class Allocator>
  void deque<T,Allocator>::_RWinsert_aux(iterator position,      
                                      size_type n, const T& x)
  {
    difference_type index     = position - begin();
    difference_type remainder = _RWlength   - index;

    if (remainder > index)
    {
      if (n > (size_type)index)
      {
        difference_type m = n - index;
        while (m-- > 0) push_front(x);
        difference_type i = index;
        while (i--) push_front(*(begin() + n - 1));
        fill(begin() + n, begin() + n + index, x);
      }
      else
      {
        difference_type i = n;
        while (i--) push_front(*(begin() + n - 1));
        copy(begin() + n + n, begin() + n + index, begin() + n);
        fill(begin() + index, begin() + n + index, x);
      }
    }
    else
    {
      difference_type orig_len = index + remainder;
      if (n > (size_type)remainder)
      {
        difference_type m = n - remainder;
        while (m-- > 0) push_back(x);
        difference_type i = 0;
        while (i < remainder) push_back(*(begin() + index + i++));
        fill(begin() + index, begin() + orig_len, x);
      }
      else
      {
        difference_type i = 0;
        while ((size_type)i < n) push_back(*(begin() + orig_len - n + i++));
        copy_backward(begin() + index, begin() + orig_len - n,
                      begin() + orig_len);
        fill(begin() + index, begin() + index + n, x);
      }
    }
  }


#ifndef _RWSTD_NO_MEMBER_TEMPLATES

#if defined(__DECCXX) && !defined(__DECFIXCXXL1306)
  template <class T, class Allocator>
  template <class InputIterator>
  void deque<T,Allocator>::_RWinsert_aux2 (iterator position, 
                                   InputIterator first, InputIterator last)
  {
    difference_type index     = position - begin();
    difference_type remainder = _RWlength   - index;
    size_type n;
    _RWinitialize(n, size_type(0));
#if defined(__DECCXX) && !defined(__DECFIXCXXL542) && !defined(_RWSTD_NO_EXPLICIT_ARG)
    n = distance(first, last);
#else
    distance(first, last, n);
#endif

    if (remainder > index)
    {
      if (n > index)
      {
        InputIterator m = last - index;
        while (m != first) push_front(*--m);
        difference_type i = index;
        while (i--) push_front(*(begin() + n - 1));
        copy(last - index, last, begin() + n);
      }
      else
      {
        difference_type i = n;
        while (i--) push_front(*(begin() + n - 1));
        copy(begin() + n + n, begin() + n + index, begin() + n);
        copy(first, last, begin() + index);
      }
    }
    else
    {
      difference_type orig_len = index + remainder;
      if (n > remainder)
      {
        InputIterator m = first + remainder;
        while (m != last) push_back(*m++);
        difference_type i = 0;
        while (i < remainder) push_back(*(begin() + index + i++));
        copy(first, first + remainder, begin() + index);
      }
      else
      {
        difference_type i = 0;
        while (i < n) push_back(*(begin() + orig_len - n + i++));
        copy_backward(begin() + index, begin() + orig_len - n,
                      begin() + orig_len);
        copy(first, last, begin() + index);
      }
    }
  }
#endif

  template <class T, class Allocator>
  template <class InputIterator>
  void deque<T,Allocator>::insert (iterator position, 
                                   InputIterator first, InputIterator last)
  {
#if defined(__DECCXX) && !defined(__DECFIXCXXL1306)
         _RWinsert_interval_dispatch  (position, first, last, iterator_traits<InputIterator>::iterator_category()); 
#else
    difference_type index     = position - begin();
    difference_type remainder = _RWlength   - index;
    size_type n;
    _RWinitialize(n, size_type(0));
#if defined(__DECCXX) && !defined(__DECFIXCXXL542) && !defined(_RWSTD_NO_EXPLICIT_ARG)
    n = distance(first, last);
#else
    distance(first, last, n);
#endif

    if (remainder > index)
    {
      if (n > index)
      {
        InputIterator m = last - index;
        while (m != first) push_front(*--m);
        difference_type i = index;
        while (i--) push_front(*(begin() + n - 1));
        copy(last - index, last, begin() + n);
      }
      else
      {
        difference_type i = n;
        while (i--) push_front(*(begin() + n - 1));
        copy(begin() + n + n, begin() + n + index, begin() + n);
        copy(first, last, begin() + index);
      }
    }
    else
    {
      difference_type orig_len = index + remainder;
      if (n > remainder)
      {
        InputIterator m = first + remainder;
        while (m != last) push_back(*m++);
        difference_type i = 0;
        while (i < remainder) push_back(*(begin() + index + i++));
        copy(first, first + remainder, begin() + index);
      }
      else
      {
        difference_type i = 0;
        while (i < n) push_back(*(begin() + orig_len - n + i++));
        copy_backward(begin() + index, begin() + orig_len - n,
                      begin() + orig_len);
        copy(first, last, begin() + index);
      }
    }
#endif
  }

#else

  template<class T, class Allocator>
  void deque<T,Allocator>::insert (iterator position, 
                                   const_iterator first, 
                                   const_iterator last)
  {
    difference_type index     = position - begin();
    difference_type remainder = _RWlength   - index;
    size_type n;
    _RWinitialize(n, size_type(0));
#if defined(__DECCXX) && !defined(__DECFIXCXXL542) && !defined(_RWSTD_NO_EXPLICIT_ARG)
    n = distance(first, last);
#else
    distance(first, last, n);
#endif

    if (remainder > index)
    {
      if (n > (size_type)index)
      {
        const_iterator m = last - index;
        while (m != first) push_front(*--m);
        difference_type i = index;
        while (i--) push_front(*(begin() + n - 1));
        copy(last - index, last, begin() + n);
      }
      else
      {
        difference_type i = n;
        while (i--) push_front(*(begin() + n - 1));
        copy(begin() + n + n, begin() + n + index, begin() + n);
        copy(first, last, begin() + index);
      }
    }
    else
    {
      difference_type orig_len = index + remainder;
      if (n > (size_type)remainder)
      {
        const_iterator m = first + remainder;
        while (m != last) push_back(*m++);
        difference_type i = 0;
        while (i < remainder) push_back(*(begin() + index + i++));
        copy(first, first + remainder, begin() + index);
      }
      else
      {
        difference_type i = 0;
        while ((size_type)i < n) push_back(*(begin() + orig_len - n + i++));
        copy_backward(begin() + index, begin() + orig_len - n,
                      begin() + orig_len);
        copy(first, last, begin() + index);
      }
    }
  }

  template<class T, class Allocator>
  void deque<T,Allocator>::insert (iterator position, 
                                   const T* first, const T* last)
  {
    difference_type index     = position - begin();
    difference_type remainder = _RWlength   - index;
    size_type n;
    _RWinitialize(n, size_type(0));
#if defined(__DECCXX) && !defined(__DECFIXCXXL542) && !defined(_RWSTD_NO_EXPLICIT_ARG)
    n = distance(first, last);
#else
    distance(first, last, n);
#endif

    if (remainder > index)
    {
      if (n > (size_type)index)
      {
        const T* m = last - index;
        while (m != first) push_front(*--m);
        difference_type i = index;
        while (i--) push_front(*(begin() + n - 1));
        copy(last - index, last, begin() + n);
      }
      else
      {
        difference_type i = n;
        while (i--) push_front(*(begin() + n - 1));
        copy(begin() + n + n, begin() + n + index, begin() + n);
        copy(first, last, begin() + index);
      }
    }
    else
    {
      difference_type orig_len = index + remainder;
      if (n > (size_type)remainder)
      {
        const T* m = first + remainder;
        while (m != last) push_back(*m++);
        difference_type i = 0;
        while (i < remainder) push_back(*(begin() + index + i++));
        copy(first, first + remainder, begin() + index);
      }
      else
      {
        difference_type i = 0;
        while ((size_type)i < n) push_back(*(begin() + orig_len - n + i++));
        copy_backward(begin() + index, begin() + orig_len - n,
                      begin() + orig_len);
        copy(first, last, begin() + index);
      }
    }
  }

#endif /*_RWSTD_NO_MEMBER_TEMPLATES*/

  template <class T, class Allocator>
  _TYPENAME deque<T,Allocator>::iterator 
  deque<T,Allocator>::erase (iterator position)
  {
    if (end() - position > position - begin())
    {
      copy_backward(begin(), position, position + 1); 
      pop_front();
      return position + 1;
    }
    else
    {
      copy(position + 1, end(), position); 
      pop_back();
      return position;
    }
  }
    
  template <class T, class Allocator>
  _TYPENAME deque<T,Allocator>::iterator 
  deque<T,Allocator>::erase (iterator first, iterator last)
  {
    difference_type n = last - first;
    if (end() - last > first - begin())
    {
      copy_backward(begin(), first, last);
      while(n-- > 0) pop_front();
      return last;
    }
    else
    {
      copy(last, end(), first);
      while(n-- > 0) pop_back();
      return first;
    }
  }



#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#   pragma __extern_prefix __restore
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __restore
#   endif
#endif
