#ifndef __FSTREAM_HXX
#define __FSTREAM_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	The iostream package.
//
//  MODULE:	Include file for the fstream subpackage.
//
//  MODULE DESCRIPTION:
//
//      The fstream subpackage provides the file I/O functionality of the
//	iostream package.  It relies on the iostream (core) subpackage's base
//	classes to derive classes that are specialized for file I/O operations.
//
//  DESIGN ISSUES:
//
//      The description of this subpackage in AT&T's documentation is quite
//	incomplete.  Also, it wasn't always possible to experimentally determine
//	the compatible behavior because of the problems in the version of cfront
//	(Glockenspiel) we were using.
//
//--

//++
//  INCLUDE FILES
//--

#include <compnent.hxx>
#ifdef __USE_STD_IOSTREAM 
#error "cannot include fstream.hxx -- do not compile strict_ansi OR define __NO_USE_STD_IOSTREAM - see section 7.1.2 of the C++ Using Guide"
#else

#include <cxxl.hxx>
#include <iostream.hxx>

extern "C"
{
#if defined(__VMS_POSIX)
#   include <unistd.h>
#   include <sys/file.h>
#elif defined(__VMS)
#   include <file.h>
#   include <unixio.h>
#   ifndef SEEK_SET
#       define SEEK_SET 0
#   endif
#   ifndef SEEK_CUR
#       define SEEK_CUR 1
#   endif
#   ifndef SEEK_EOF
#       define SEEK_EOF 2
#   endif
#   ifndef SEEK_END
#       define SEEK_END 2	/* ANSI C defintion */
#   endif
#endif
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//++
//  LINKAGE
//--

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model  __relaxed_refdef
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

class filebuf : public streambuf

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class provides a stream buffer for I/O through file descriptors
//	(common uses, however, do not require the program to manipulate file
//	descriptors).
//--

{
// Static data member definitions

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif
public:
    static const int __CXXL_XFER_DECL(openprot); // Default protection for open

// Function member definitions

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif
			filebuf();
			filebuf(int fd);
			filebuf(int fd, char *p, int len);
			~filebuf();

    filebuf		*attach(int fd);
    filebuf		*close();
#if defined(__VMS) && (__VMS_VER < 70000000) 
    inline int		fd();
#else
    int			fd();
#endif
#ifdef __VMS
    inline int		is_open();
#else
    int			is_open();
#endif
    filebuf		*open(const char *name, int mode, int prot = openprot);
    virtual int		overflow(int = EOF);
    virtual streampos	seekoff(streamoff, __SCOPE(ios::)seek_dir, int mode);
    virtual streampos	seekpos(streampos, int mode);
    virtual streambuf	*setbuf(char *p, int len);
    virtual int		sync();
    virtual int		underflow();
    filebuf		*__attach(int fd);
    filebuf		*__close();
    inline int		__fd();
    inline int		__is_open();
    filebuf		*__open(const char *name, int mode, int prot=openprot);


// Data member definitions -- each platform has own backward compatible layout
#if defined(__VMS)
private:
    unsigned long	m_lcchFile_bOpen;       // open flag and file size
						// open flag is lsb
    long		m_fpBuffer;             // file position
#endif

#if defined(__unix__)
private:
    int			m_fd;
    int			m_cchDeltaFp;
    char                m_bFileIsOpen;
    char                m_bFilebufOwnsFd;
    char                m_bNeedsSync;
    char                __fill1;
    int			__fill2;
    long                m_fpBuffer;
    long		__fill3;
    long                m_lcchFile;
#endif

#if defined(_NT)
private:
    int                 m_fd;
    long                m_fpBuffer;
    long                m_lcchFile;
    int                 m_bFileIsOpen;
    int                 m_bFilebufOwnsFd;
    int                 m_bNeedsSync;
    int			m_cchDeltaFp;
#endif

};

#if !defined(__VMS)
// Class fstreambase provides a mechanism to include a single filebuf
// in all fstream classes.  It saves a memory allocation when
// creating a filebuf object, but it complicates initialization
// and sync_with_stdio().  Its primary purpose is compatibility
// with previous versions of the C++ class library provided on DEC OSF/1

class fstreambase : virtual public ios {
public:
			fstreambase();
			fstreambase(int);
			fstreambase(int, char *, int);
			fstreambase(const char *, int, int);
			~fstreambase();
    void		attach(int);
    void		close() ;
    void		open(const char *, int, int);
    void		setbuf(char *, int) ;
private:
    filebuf		m_filebuf;
};
#endif

#if !defined(__VMS)
class ifstream : public fstreambase, public istream
#else
class ifstream : public istream
#endif

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class supports formatted input from files.  It uses a filebuf to do
//	the I/O.  Common operations (such as opening and closing) can be done
//	directly on streams without the explicit mention of filebufs.
//--

{
public:
			ifstream();
			ifstream(const char *name, int mode = ios::in,
				int prot = filebuf::openprot);
			ifstream(int fd);
			ifstream(int fd, char *p, int len);
			~ifstream();

    void		attach(int fd);
    void		close();
    void		open(const char *name, int mode = ios::in, 
				int prot = filebuf::openprot);
    filebuf		*rdbuf();
    void		setbuf(char *p, int len);
};

#if !defined(__VMS)
class ofstream : public fstreambase, public ostream
#else
class ofstream : public ostream
#endif
//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class supports formatted output to files.  It uses a filebuf to do
//	the I/O.  Common operations (such as opening and closing) can be done
//	directly on streams without the explicit mention of filebufs.
//--

{
public:
			ofstream();
			ofstream(const char *name, int mode = ios::out,
				int prot = filebuf::openprot);
			ofstream(int fd);
			ofstream(int fd, char *p, int len);
			~ofstream();

    void		attach(int fd);
    void		close();
    void		open(const char *name, int mode = ios::out, 
				int prot = filebuf::openprot);
    filebuf		*rdbuf();
    void		setbuf(char *p, int len);
};

#if !defined(__VMS)
class fstream : public fstreambase, public iostream
#else
class fstream : public iostream
#endif

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class supports formatted I/O to and from files.  It uses a filebuf
//	to do the I/O.  Common operations (such as opening and closing) can be
//	done directly on streams without the explicit mention of filebufs.
//--

{
public:
			fstream();
			fstream(const char *name, int mode,
				int prot = filebuf::openprot);
			fstream(int fd);
			fstream(int fd, char *p, int len);
			~fstream();

    void		attach(int fd);
    void		close();
    void		open(const char *name, int mode,
				int prot = filebuf::openprot) ;
    filebuf		*rdbuf();
    void		setbuf(char *p, int len);
};

//++
//  INLINED FUNCTIONS
//--

inline int filebuf::__fd(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the file descriptor the filebuf is connected to, or EOF if the
//	filebuf is closed.
//--

{
    if (__is_open())
#if defined(__VMS)
        return streambuf::fd();
#else
        return m_fd;
#endif
    else
	return EOF;
}
#if defined(__VMS) && (__VMS_VER < 70000000)
inline int filebuf::fd() {return __fd();}
#endif //if defined(__VMS) && (__VMS_VER < 70000000) 

#ifdef __VMS
inline int filebuf::is_open(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns non-zero when the filebuf is connected to a file descriptor, and
//	zero otherwise.
//--

{
   return __is_open();
}
#endif

inline int filebuf::__is_open(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns non-zero when the filebuf is connected to a file descriptor, and
//	zero otherwise.
//--

{
#if defined(__VMS)
    return m_lcchFile_bOpen & 01;
#else
    return m_bFileIsOpen;
#endif
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif // __USE_STD_IOSTREAM

#endif
// End of fstream.hxx
