/***************************************************************************
 *
 * fstream.cc - Definition for the Standard Library file streams
 *
 ***************************************************************************
 *    
 *  Copyright 2000 Compaq Computer Corporation
 *
 *  COMPAQ Registered in U.S. Patent and Trademark Office.
 *
 *  Confidential computer software. Valid license from Compaq required for
 *  possession, use or copying. Consistent with FAR 12.211 and 12.212,
 *  Commercial Computer Software, Computer Software Documentation, and
 *  Technical Data for Commercial Items are licensed to the U.S. Government
 *  under vendor's standard commercial license.
 *
 ****************************************************************************
 *
 * (c) Copyright 1994, 1998 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#ifndef _RWSTD_NO_NEW_HEADER
#include <cstdio>
#else
#include <stdio.h>
#endif // _RWSTD_NO_NEW_HEADER

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1348)
#include <sys/stat.h>
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __save
#      pragma __environment __header_defaults
#   endif
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#pragma __extern_prefix __save
#pragma __extern_prefix "CXXL$" 
#endif

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif

#ifdef __BORLANDC__
  template <class T>
  void _RWio_initialize (T& t, T val) { t = val; }
#pragma option -w-par
#endif

  /*
   * class basic_filebuf
   */

  /*
   * basic_filebuf()
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits>::basic_filebuf()
  : basic_streambuf<charT, traits>()
  , _RWfile(_RWclosed())
  , _RWdata(0)
  , _RWread_buff(false)
  , _RWwrite_buff(false)
  , _RWlast_numb_read(0)
  , _RWrwBufferSize(512)
  , _RWabsolute_pos(0)
  {
    _RWstate_beg = new state_t;
    _RWstate_end = new state_t;
  }



  /*
   * basic_filebuf(_RWfile_type)
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits>::basic_filebuf(_RWfile_type fd)
  : basic_streambuf<charT, traits>()
  , _RWfile(fd)
  , _RWread_buff(false)
  , _RWwrite_buff(false)
  , _RWlast_numb_read(0)
  , _RWrwBufferSize(512)
  , _RWabsolute_pos(0)
  {
    _RWdata = new char_type[_RWrwBufferSize+1];
    _RWstate_beg = new state_t;
    _RWstate_end = new state_t;

#if defined(__DECCXX) && !defined(__DECFIXCXXL1218)
    if ( fd == 0 )
      basic_streambuf<charT,traits>::mode_ = ios_base::in;
    else
    {
      if ( fd < 3 )
        basic_streambuf<charT,traits>::mode_ = ios_base::out;
    }
#else
#ifndef _RWSTD_STRICT_ANSI
    if ( fd == 0 )
      basic_streambuf<charT,traits>::mode_ = ios_base::in;
    else
    {
      if ( fd < 3 )
        basic_streambuf<charT,traits>::mode_ = ios_base::out;
    }
#else
    if ( fd == stdin )
      basic_streambuf<charT,traits>::mode_ = ios_base::in;
    else
    {
      if ( _RWfile_is_std(fd) )
        basic_streambuf<charT,traits>::mode_ = ios_base::out;
    }
#endif // _RWSTD_STRICT_ANSI
#endif

  }


  /*
   * ~basic_filebuf()
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits>::~basic_filebuf()
  {
    close();
    delete _RWstate_beg;
    delete _RWstate_end;
  }

  /*
   * bool is_open() const
   */

  template<class charT, class traits>
  bool basic_filebuf<charT, traits>::is_open() const
  {
#if defined(__DECCXX) && !defined(__DECFIXCXXL1218)
    return !(_RWfile == -1);
#else
#ifndef _RWSTD_STRICT_ANSI
    return !(_RWfile == -1);
#else
    return !(_RWfile == 0);
#endif
#endif
  }

  /*
   * basic_filebuf *open(int fd)
   * fstream compatibility
   */

#ifndef _RWSTD_NO_EXTENSION

  template<class charT, class traits>
  basic_filebuf<charT, traits> *
  basic_filebuf<charT, traits>::
  open(int fd )
  {   
    if(_RWfile != _RWclosed())
      return 0;

    this->streambuf_init(false); 

    _RWfile           = fd;
    _RWread_buff      = false;
    _RWwrite_buff     = false;
    _RWlast_numb_read = 0; 

    delete [] _RWdata;
    _RWdata = new char_type[_RWrwBufferSize+1];

    if ( fd == 0 )
      basic_streambuf<charT,traits>::mode_ = ios_base::in;
    else
    {
      if ( fd < 3 )
        basic_streambuf<charT,traits>::mode_ = ios_base::out;
    }
    return this;
  }
#endif // _RWSTD_NO_EXTENSION
  /*
   * basic_filebuf *open(const char *, ios_base::openmode, long )
   * opens a file and allocates a buffer
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits> *
  basic_filebuf<charT, traits>::
  open(const char *s, ios_base::openmode mode, long protection)
  {
    long    m;
    if ( mode & ios_base::ate )
      m = mode & (~ios_base::ate);
    else
      m = mode;
    if(_RWfile != _RWclosed())
      return 0;

    basic_streambuf<charT,traits>::mode_ = mode;
    this->streambuf_init(false);

#if defined(__DECCXX) && !defined(__DECFIXCXXL1218)
    if(__RWSTD::_RWrwOpenTable[m] == -1) 
      return 0;
    if((_RWfile = ::open(s, __RWSTD::_RWrwOpenTable[m], protection)) == _RWclosed())
      return 0;
#else
#ifndef _RWSTD_STRICT_ANSI
    if(__RWSTD::_RWrwOpenTable[m] == -1) 
      return 0;
    if((_RWfile = ::open(s, __RWSTD::_RWrwOpenTable[m], protection)) == _RWclosed())
      return 0;
#else
    if(__RWSTD::_RWrwOpenTable[m] == 0) 
      return 0;
    if((_RWfile = fopen(s, __RWSTD::_RWrwOpenTable[m])) == _RWclosed())
      return 0;
    setvbuf(_RWfile,0,_IONBF,0);
#endif // _RWSTD_STRICT_ANSI
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1348)
    {
      struct stat x;
      _RWdisk_file = 0; // assume non-disk device
      if ( stat(s, &x) == 0 && (x.st_ino[0] + x.st_ino[1] + x.st_ino[2] != 0) )
        _RWdisk_file = 1;
    }
#endif

    delete [] _RWdata;
    _RWdata = new char_type[_RWrwBufferSize+1];

    if(mode & ios_base::ate)
    {
      if( (_RWabsolute_pos = _RWfile_seek(_RWfile, 0, SEEK_END))== -1  )
      {
        close();
        delete [] _RWdata;  
        _RWdata = 0;
        _RWfile = _RWclosed();
        return 0;
      }
    }
    return this;
  }

  /*
   * basic_filebuf *close()
   * closes a file if one was already open.  Also deletes the
   * data, if it was allocated
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits> *
  basic_filebuf<charT, traits>::close()
  {
    delete _RWstate_beg;
    delete _RWstate_end;
    _RWstate_beg = new state_t;
    _RWstate_end = new state_t;

    _RWabsolute_pos = 0;

    if ( (_RWfile_is_std(_RWfile))  && (_RWfile != _RWclosed()) )
    {
      if ( _RWwrite_buff )
        overflow( traits::eof() );
      _RWread_buff=false;
      _RWwrite_buff=false;
      _RWlast_numb_read=0;
      return this;
    }
   
    if(_RWfile != _RWclosed())
    {          
      if ( _RWwrite_buff )
        overflow( traits::eof() );
      delete [] _RWdata;
      _RWdata = 0;
      _RWread_buff=false;
      _RWwrite_buff=false;
      _RWlast_numb_read=0;
 

      if(_RWfile_close(_RWfile) == -1)
        return 0;

      _RWfile = _RWclosed();

      return this;
    }
    return 0;          
  }

  /*
   * streamsize showmanyc()
   */

  template<class charT, class traits>
#if defined(__DECCXX) && !defined(__DECFIXCXXL1051)
  streamsize
#else
  int
#endif
  basic_filebuf<charT, traits>::showmanyc()
  { 
    if ( this->gptr() )
      return (int)(this->egptr()-this->gptr());
   
    if ( this->pptr() && _RWread_buff )
      return (int)(this->epptr()-this->pptr());   

    return 0;

  }

  /*
   * int_type overflow(int_type)
   * this is called when the put pointer overruns the buffer.
   * it should flush what was in the put buffer, and move the pointer
   * back to the beginning
   */

  template<class charT, class traits>
  _TYPENAME basic_filebuf<charT, traits>::int_type
  basic_filebuf<charT, traits>::overflow(int_type c)
  {
    long  count = this->pptr() - this->pbase();
    bool   do_noconv;

    if(_RWfile == _RWclosed())        
      return traits::eof();

    if ( (this->pptr()==0) && this->gptr() )
    {
      this->setp(this->eback(),this->eback()+_RWrwBufferSize);
      pbump((int)(this->gptr()-this->eback()));
      this->setg(0,0,0);
      if ( this->pptr()<this->epptr() )
      { 
        if( !traits::eq_int_type(c,traits::eof()) ) 
        {
          sputc( traits::to_char_type(c) );
          this->gbump(1);
          _RWwrite_buff=true;
          return traits::to_int_type(*this->pptr());
        }
      } 
      count=  this->pptr() - this->pbase();
    }

    if ( (this->pptr()==0) && (this->gptr()==0) )
      this->setp(_RWdata,_RWdata+ _RWrwBufferSize);  

    _RWwrite_buff= false;

    if(count)
    {        
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
      do_noconv = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
      do_noconv = use_facet( this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
      .always_noconv();

      if ( do_noconv )
      {
        if ( _RWread_buff ) 
        {
          if ( _RWfile_seek(_RWfile, -_RWlast_numb_read, SEEK_CUR) == -1 )
            return traits::eof();
        }
        if(!_RWfile_write(_RWfile,(char *)this->pbase(),sizeof(charT),count))
          return traits::eof();
      }
      else
      {
        long size_to = count*sizeof(charT);
        const charT *from_next =0;
        char *to= new char[size_to];
        char *to_next = 0;
        codecvt_base::result conv_result;

        if ( _RWread_buff )
        {
          if ( _RWfile_seek(_RWfile, -_RWlast_numb_read, SEEK_CUR) == -1 )
            return traits::eof();
          else
          {
            state_t *tmp = _RWstate_beg;
            _RWstate_beg = _RWstate_end;
            _RWstate_end = tmp;
          }
        }
 
        *_RWstate_beg = *_RWstate_end;

        do {

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
          conv_result = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
          conv_result = use_facet( this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
          .out(*_RWstate_end,this->pbase(),this->pptr(),from_next,
               to,to+size_to,to_next); 

          if ( conv_result == codecvt_base::noconv ) break;

          if (  (conv_result == codecvt_base::partial) || ( from_next != this->pptr() ) )
          {
            size_to *= 2;
            char *to_bis = new char[size_to];
            long diff_to = to_next - to;
            memcpy(to_bis,to,diff_to);
            delete [] to;
            to = to_bis;
            to_next = to + diff_to;
          }

        }while( (conv_result == codecvt_base::partial) || ( from_next != this->pptr()));

        if ( conv_result==codecvt_base::error )
        {
          delete [] to;
          return traits::eof();
        }

        if (conv_result == codecvt_base::noconv) 
        {
          if(!_RWfile_write(_RWfile, (char*)this->pbase(), sizeof(charT), count))
          {
            delete [] to;
            return traits::eof();              
          }
        } 
        else 
        {
          if(!_RWfile_write(_RWfile, to, sizeof(char),to_next-to))
          {
            delete [] to;
            return traits::eof();                
          }
        }
        delete [] to;
      }
    }

    this->setp(_RWdata, _RWdata+_RWrwBufferSize);    
    this->setg(0,0,0);                       
    _RWread_buff = false;              
    _RWwrite_buff= false; 

    _RWabsolute_pos = _RWfile_seek(_RWfile,0,SEEK_CUR);

    if( !traits::eq_int_type(c,traits::eof()) ) 
    {
      sputc(traits::to_char_type(c));
      this->gbump(1);
      _RWwrite_buff = true;
    }
    else
      this->setp(0,0);

    return traits::not_eof(c);
  }

  /*
   * int_type pbackfail(int_type)
   */

  template<class charT, class traits>
  _TYPENAME basic_filebuf<charT, traits>::int_type
  basic_filebuf<charT, traits>::pbackfail(int_type c)
  {
    if(_RWfile == _RWclosed())
      return traits::eof();

    if ( (this->gptr()==0) && this->pptr() )
    {
      if ( _RWread_buff )
        this->setg(this->pbase(),this->pptr(),this->epptr());
      else
        this->setg(this->pbase(),this->pptr(),this->pptr()); 
    }

    if ( (!traits::eq_int_type(c,traits::eof())) && (this->gptr()>this->eback()) )
    {
      if ( traits::eq(*(this->gptr()-1),traits::to_char_type(c)) ) 
      {
        this->gbump(-1);
        return c;
      }
      else
      {
        this->gbump(-1);
        *this->gptr()=traits::to_char_type(c);
        return c;
      }
    }       

    if ( (traits::eq_int_type(c,traits::eof())) && (this->gptr()>this->eback()) )  
    { 
      this->gbump(-1);
      return traits::not_eof(c);
    }
                                                
    return traits::eof();
  }

  /*
   * basic_streambuf<charT,traits>* setbuf(char_type*,streamsize)
   */

  template<class charT, class traits>
  basic_streambuf<charT, traits>*
  basic_filebuf<charT, traits>::setbuf(char_type *s, streamsize n)
  {

    if (n>0)
    { 
      if ( _RWfile != _RWclosed() )
      {
        if ( !traits::eq_int_type(overflow( traits::eof() ),traits::eof()) )
        {
          if ( s )
          { 
            delete [] _RWdata;
            _RWrwBufferSize = n-1; 
            _RWdata=s;
          }
          else
          {
            charT *tmp;
            _RWrwBufferSize = n;  
            tmp = new char_type[_RWrwBufferSize+1];
            delete [] _RWdata;
            _RWdata = tmp;
          }
          this->setp(0,0);    
          this->setg(0,0,0);                         
          _RWread_buff=false;
          _RWwrite_buff=false;
        }
      }
      else
      {
        if ( s )
        {
          _RWrwBufferSize = n-1;
          _RWdata =s;
          this->setp(0,0);    
          this->setg(0,0,0);                        
          _RWwrite_buff= false;
          _RWread_buff = false; 
        }
        else
          _RWrwBufferSize = n;
      }  
    } 
    return (basic_streambuf<charT,traits>*)(this); 
  }


  /*
   * int_type underflow()
   * this is called when the get pointer "underflows" the buffer,
   * or when there are no more "get" characters.
   */

  template<class charT, class traits>
  _TYPENAME basic_filebuf<charT, traits>::int_type
  basic_filebuf<charT, traits>::underflow()
  {
    bool do_noconv;
    long offset;
#ifndef __BORLANDC__
    offset = 0;
#else
    _RWio_initialize(offset,0L);
#endif

    if(_RWfile == _RWclosed())
      return traits::eof();
 
    if ( (this->gptr()==0) && this->pptr() )
    {
      if ( _RWread_buff )
        this->setg(this->pbase(),this->pptr(),this->epptr());
      else
        this->setg(this->pbase(),this->pptr(),this->pptr());

      this->setp(0,0);
    }

    if(this->gptr() && (this->gptr()<this->egptr()) ) return traits::to_int_type(*this->gptr());     

    if( ((_RWwrite_buff) && this->gptr() ) || ((_RWwrite_buff) && this->pptr() ) )
    {
      if ( traits::eq_int_type(overflow( traits::eof() ),traits::eof()) ) 
        return traits::eof();
      this->setg(this->pbase(),this->pptr(),this->epptr());
      this->setp(0,0);
      _RWwrite_buff = false; 
    }                 

    _RWabsolute_pos = _RWfile_seek(_RWfile,0,SEEK_CUR);

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
    do_noconv = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
    do_noconv = use_facet( this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
      .always_noconv();

    if ( do_noconv )
    {
      streamoff last_numb_read = _RWfile_read(_RWfile, (char *)_RWdata,sizeof(charT),_RWrwBufferSize);
      if( last_numb_read <= 0 ) 
        return traits::eof();
      offset = last_numb_read/sizeof(charT);
      _RWlast_numb_read = _RWget_chars(_RWdata,_RWdata + last_numb_read);
    }
    else
    {
      long from_size = _RWrwBufferSize*sizeof(charT);
      const char *from_next =0;
      char *from= new char[from_size];
      charT *to_next = 0;
      codecvt_base::result conv_result;

      streamoff last_numb_read = 0;
      long new_numb_read;

      do {
        new_numb_read = _RWfile_read(_RWfile, from+last_numb_read
                             ,sizeof(char),_RWrwBufferSize);

        if( new_numb_read <= 0 ) 
          break;

        last_numb_read += new_numb_read;
        *_RWstate_beg = *_RWstate_end;

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
        conv_result = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
        conv_result = use_facet(this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
        .in(*_RWstate_end,from,from+last_numb_read,from_next,_RWdata,
            _RWdata+_RWrwBufferSize,to_next); 
 
        if ( (conv_result == codecvt_base::error) ||
             (conv_result == codecvt_base::noconv) ) break;

        if (  (to_next != (_RWdata + _RWrwBufferSize) ) &&
              ( (size_t)new_numb_read == _RWrwBufferSize*sizeof(charT) ) )
        {
          from_size *= 2;
          char *from_bis = new char[from_size];
          long diff_from = from_next - from;
          memcpy(from_bis,from,last_numb_read);
          delete [] from;
          from = from_bis;
          from_next = from + diff_from;
        }

      } while ( (to_next != (_RWdata + _RWrwBufferSize) ) &&
                ( (size_t)new_numb_read == _RWrwBufferSize*sizeof(charT) ) ); 

      if( last_numb_read <= 0 ) 
        return traits::eof();
      _RWlast_numb_read = last_numb_read;
               
      if ( (conv_result==codecvt_base::error) ||
           (conv_result==codecvt_base::partial) )
      {
        delete [] from;
        return traits::eof();
      } 
      else 
      {
        if (conv_result==codecvt_base::noconv)
        {
          memcpy((char*)_RWdata,from,_RWlast_numb_read);
          offset = _RWlast_numb_read;
        }
        else
        {
          long diff_minus = new_numb_read - (from_next - from);
          if ( _RWfile_seek(_RWfile, -diff_minus, SEEK_CUR) == -1 )
            return traits::eof();
          _RWlast_numb_read -= diff_minus;
          offset = to_next - _RWdata;
        }
      }

      delete [] from;
    }
        
    if(_RWlast_numb_read) 
    {   
      this->setg(_RWdata, _RWdata, _RWdata+offset); 
      this->setp(0, 0);
      _RWread_buff=true;
      return traits::to_int_type(*this->gptr());
    }

    return traits::eof();
  }

  /*
   * pos_type seekoff(off_type, ios_base::seekdir, ios_base::openmode)
   */

  template<class charT, class traits>
  _TYPENAME basic_filebuf<charT, traits>::pos_type
  basic_filebuf<charT, traits>::seekoff(off_type off,
                                        ios_base::seekdir way,
                                        ios_base::openmode which)
  {
    off_type            loff = off;
    pos_type            newoff;

    if(_RWfile == _RWclosed())
      return pos_type(off_type(-1));
#if defined(__DECCXX) && !defined(__DECFIXCXXL970)
    if ( (off == off_type(-1)) && (way==ios_base::beg) )
#else
    if ( off == off_type(-1) )
#endif
      return pos_type(off_type(-1));
#if defined(__DECCXX) && !defined(__DECFIXCXXL970)
    if ( (way==ios_base::cur) )
#else
    if ( (way==ios_base::cur) && (off==off_type(0)) )
#endif
    {
      streamsize remain_in_buffer=0;
      charT * remain_start=0;
      charT * remain_end=0;
     
      if ( _RWread_buff )
      {
        if ( this->gptr() )  
        {
          if (!this->eback())
            remain_in_buffer= this->egptr()-this->gptr();
          else
            remain_in_buffer= _RWget_chars(this->gptr(),this->egptr());
          remain_start = this->gptr();
          remain_end = this->egptr();
        }
        else
        {
          if ( this->pbase() )  
          {
            remain_in_buffer= _RWget_chars(this->pptr(),this->epptr());
            remain_start = this->pptr();
            remain_end = this->epptr();
          }
        }
      }
      else 
      {
        if ( _RWwrite_buff )
        {
          if ( this->pbase() )  
          {
            remain_in_buffer= -(_RWget_chars(this->pbase(),this->pptr()));
            remain_start = this->pbase();
            remain_end = this->pptr();
          }

          else
          { 
            if ( this->eback() )  
            {
              remain_in_buffer= -(_RWget_chars(this->eback(),this->gptr()));
              remain_start = this->eback();
              remain_end = this->gptr();
            }
          }
        }     
      }

      int const_conv;

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
      const_conv = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
      const_conv = use_facet( this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
      .encoding(); 
    
      // If characters are not a constant size can't seek from
      // current position
      if ( (off != off_type(0)) && (const_conv <= 0) ) {
        return pos_type(off_type(-1));
      }

      if ( const_conv > 0 )
      {
#if defined(__DECCXX) && !defined(__DECFIXCXXL970)
         if (off != off_type(0)) {
                 if (_RWwrite_buff ) {
                   overflow( traits::eof() );
                       _RWwrite_buff = false;
                       return ( _RWfile_seek(_RWfile,loff*const_conv,SEEK_CUR)/const_conv);
                 }
                 this->setp(0,0);
                 this->setg(0,0,0);
           return ( _RWfile_seek(_RWfile,-((remain_in_buffer-loff)*const_conv),SEEK_CUR)/const_conv);
         }
#endif
        return ( (_RWfile_seek(_RWfile,0,SEEK_CUR)-remain_in_buffer*const_conv)/const_conv);
      }
      else 
      {
        // First find current position in file
        newoff = _RWfile_seek(_RWfile, 0, SEEK_CUR);

        // If necessary convert characters in buffer to 
        // find offset from file position
        if (remain_in_buffer != 0 && this->eback())
        {
          long to_size = _RWrwBufferSize;
          const charT *from = remain_start;
          const charT *from_next = from;
          const charT *from_end = remain_end;
          char *to_base = new char[to_size];
          char *to = to_base;
          char *to_next = to;
          char *to_end = to + to_size;
          codecvt_base::result conv_result;

          do {
            *_RWstate_beg = *_RWstate_end;

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            conv_result = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
            conv_result = use_facet(this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
             .out(*_RWstate_end,from,from_end,from_next,to,to_end,to_next); 
 
            if ( (conv_result == codecvt_base::error) ||
               (conv_result == codecvt_base::noconv) ) break;

            if ( to_next == to_end && from_next != from_end )
            {
              long diff_to = to_next - to_base;
              to_size *= 2;
              to = new char[to_size];
              memcpy(to,to_base,diff_to);
              delete [] to_base;
              to_base = to;
              to = to_base + diff_to;
              to_next = to;
              from = from_next;
              to_end = to_base + to_size;
            }

          } while ( from_next != from_end );
          
          // Calculate new file position
          // The  + remain_in_buffer - ... stuff is used to deal
          // with WIN32 CR/LF translations (see _RWget_chars())
          if (remain_in_buffer > 0)
            newoff -= (to_next - to_base) 
                      + (remain_in_buffer - (remain_end-remain_start));
          else
            newoff += (to_next - to_base)
                      + (-remain_in_buffer - (remain_end-remain_start));

          delete [] to;
        }
        return newoff;
      }
    } 

    int const_conv;

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
    const_conv = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
    const_conv = use_facet( this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
    .encoding(); 

    if ( const_conv < 1 )
    {
      if ( (way==ios_base::beg) && (off==off_type(0)) )
      {
        seekpos(0);
      }
      else 
        return pos_type(off_type(-1));
    }

    if ( _RWwrite_buff)
    {  
      overflow( traits::eof() );
      _RWwrite_buff =false;  
    }

    int w = (way == ios_base::beg) ?
          SEEK_SET : ((way == ios_base::cur) ? SEEK_CUR : SEEK_END);  
    newoff = _RWfile_seek(_RWfile, loff*const_conv, w);

    this->setp(0,0);
    this->setg(0,0,0);

    return newoff;
  }

  /*
   * pos_type seekpos(pos_type, ios_base::openmode)
   */

  template<class charT, class traits>
  _TYPENAME basic_filebuf<charT, traits>::pos_type
  basic_filebuf<charT, traits>::
  seekpos(pos_type sp, ios_base::openmode which)
  { 
    if(_RWfile == _RWclosed())
      return pos_type(off_type(-1));

    if( sp == pos_type(off_type(-1)) )
      return pos_type(off_type(-1)); 

    int const_conv;

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
    const_conv = use_facet< codecvt<charT,char,state_t> >(this->getloc())
#else
    const_conv = use_facet( this->getloc(), (codecvt<charT,char,state_t>* )0)
#endif
    .encoding();

    if ( const_conv > 0 )
      return seekoff(off_type(sp.offset()), ios_base::beg, which);

    _TYPENAME traits::off_type end_pos = _RWfile_seek(_RWfile,0,SEEK_END);
   
    if ( _RWfile_seek(_RWfile, sp.offset(), SEEK_SET) == -1 )
      return pos_type(off_type(-1));

    this->setp(0,0);
    this->setg(0,0,0); 

    *_RWstate_beg = *_RWstate_end = sp.state();

    if ( traits::eq_int_type(underflow(),traits::eof()) && (end_pos > sp.offset()))
      return pos_type(off_type(-1));

    return sp;   
  }

  /*
   * int sync()
   * this synchronizes the buffers and their streams
   */

  template<class charT, class traits>
  int basic_filebuf<charT, traits>::sync()
  {
    if ( this->which_open_mode( ) & ios_base::out )
    {
#if defined(__DECCXX) && !defined(__DECFIXCXXL1218)
      if ( _RWfile == 1 || _RWfile == 2)
#else
#ifndef _RWSTD_STRICT_ANSI
      if ( _RWfile == 1 || _RWfile == 2)
#else
      if ( _RWfile == stdout || _RWfile == stderr )
#endif
#endif
      {
        if( traits::eq_int_type(overflow(traits::eof()),traits::eof()) ) 
          return -1;
      }
      else
      { 
        pos_type p_out = seekoff(0,ios_base::cur,ios_base::out);
        pos_type tmp_out = _RWfile_seek(_RWfile,0,SEEK_CUR);
        pos_type end_out = _RWfile_seek(_RWfile,0,SEEK_END);
        _RWfile_seek(_RWfile,tmp_out,SEEK_SET);
        if ( end_out > p_out )
        {
          if( seekpos(p_out) == pos_type(off_type(-1)))
            return -1;
        }
        else if (_RWwrite_buff)
        {
          if( traits::eq_int_type(overflow(traits::eof()),traits::eof()) )
            return -1;
        }
      }
    }

    if ( this->which_open_mode( ) & ios_base::in )
    {
#if defined(__DECCXX) && !defined(__DECFIXCXXL1218)
      if ( _RWfile != 0 )
#else
#ifndef _RWSTD_STRICT_ANSI
      if ( _RWfile != 0 )
#else
      if ( _RWfile != stdin )
#endif
#endif
      {
        pos_type p_in = seekoff(0,ios_base::cur,ios_base::in);
        pos_type tmp_in = _RWfile_seek(_RWfile,0,SEEK_CUR);
        pos_type end_in = _RWfile_seek(_RWfile,0,SEEK_END);
        _RWfile_seek(_RWfile,tmp_in,SEEK_SET);
        if ( end_in > p_in )
        {
          if( seekpos(p_in) == pos_type(off_type(-1)))
            return -1;
        }
      }
    }

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1348)
    if ( this->which_open_mode( ) & ios_base::out )
    {
      if ( _RWdisk_file ) ::fsync( _RWfile );
    }
#endif

    return 0;
  }


  /*
   * streamsize xsputn(const char_type *, streamsize)
   */

  template<class charT, class traits>
  streamsize basic_filebuf<charT, traits>::
  xsputn(const char_type *s, streamsize n)
  {
    if ( !s || (n == 0) ) return 0;

    if ( n >  _RWrwBufferSize )
    {

      if ( traits::eq_int_type(overflow( traits::eof() ),traits::eof()) )
        return 0;

      char_type   *eback_s = this->eback();
      char_type   *gptr_s  = this->gptr();
      char_type   *egptr_s = this->egptr();

      char_type   *pbase_s = this->pbase();
      char_type   *pptr_s  = this->pptr();
      char_type   *epptr_s = this->epptr();

      this->setg(0,0,0);
      this->setp((char_type *)s,(char_type *)(s+n));
      basic_streambuf<charT, traits>::pbump(n);

      if ( traits::eq_int_type(overflow( traits::eof() ),traits::eof()) )
        return 0; 
  
      this->setg(eback_s,gptr_s,egptr_s);
      this->setp(pbase_s,epptr_s);

      pbump(pptr_s-pbase_s); 

      return n;
    }
    else
    {
      int         i=0;
      while((i < n) && ( !traits::eq_int_type(sputc(*s++),traits::eof())))
        i++;
      return i;
    }

  }

 
  /*
   * class basic_ifstream : public basic_istream
   */

  /*
   * basic_ifstream()
   */

  template<class charT, class traits>
  basic_ifstream<charT, traits>::basic_ifstream()
  : basic_istream<charT, traits>( )
  {
    init(&_RWfb);
  }

  /*
   * basic_ifstream(const char *, ios_base::openmode, long)
   * opens a filebuf for (most likely) reading
   */

  template<class charT, class traits>
  basic_ifstream<charT, traits>::basic_ifstream(const char *s,
                                                ios_base::openmode mode,
                                                long protection )
  : basic_istream<charT, traits>( )
  {
    init(&_RWfb);
    open(s, mode, protection);
  }

  /*
   * basic_ifstream(int fd)
   * opens a filebuf for reading
   */
#ifndef _RWSTD_NO_EXTENSION

  template<class charT, class traits>
  basic_ifstream<charT, traits>::
  basic_ifstream(int fd)
  : basic_istream<charT, traits>( )
  {

    init(&_RWfb);

    if ( !_RWfb.open(fd) ) 
      this->setstate(ios_base::failbit);
  }

  /*
   * basic_ifstream(int fd, char_type *buf, int len)
   * opens a filebuf for reading
   */

  template<class charT, class traits>
  basic_ifstream<charT, traits>::
  basic_ifstream(int fd, char_type *buf, int len)
  : basic_istream<charT, traits>( )
  {

    init(&_RWfb);

    if ( ( buf!=0 ) && ( len>0 ) )
      _RWfb.pubsetbuf(buf,len);

    if ( !_RWfb.open(fd) ) 
      this->setstate(ios_base::failbit);
  }
#endif // _RWSTD_NO_EXTENSION

  /*
   * ~basic_ifstream()
   */

  template<class charT, class traits>
  basic_ifstream<charT, traits>::~basic_ifstream()
  {
  }

  /*
   * basic_filebuf *rdbuf() const
   * returns the streambuf associated with this stream
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits> *
  basic_ifstream<charT, traits>::rdbuf() const
  {
    return (basic_filebuf<charT, traits> *)&_RWfb;
  }

  /*
   * bool is_open()
   */

  template<class charT, class traits>
  bool basic_ifstream<charT, traits>::is_open()
  {
    return _RWfb.is_open();
  }

  /*
   * void open(const char *, ios_base::openmode, long )
   * opens up a file
   */

  template<class charT, class traits>
  void basic_ifstream<charT, traits>::open(const char *s,
                                           ios_base::openmode mode,
                                           long protection )
  {
    this->clear();
    mode |= ios_base::in; 

    if ( !_RWfb.open(s, mode, protection) )  
      this->setstate(ios_base::failbit);
  }

  /* 
   * void close()
   */

  template<class charT, class traits>
  void basic_ifstream<charT, traits>::close()
  {
    if(!is_open())
      return;
  
    if(!_RWfb.close())
      this->setstate(ios_base::failbit);
    else
      this->clear();
  }


  /*
   * class basic_ofstream : public basic_ostream
   */

  /*
   * basic_ofstream()
   */

  template<class charT, class traits>
  basic_ofstream<charT, traits>::basic_ofstream()
  : basic_ostream<charT, traits>( )
  {
    init(&_RWfb);
  }

  /*
   * basic_ofstream(const char *, ios_base::openmode, long )
   */

  template<class charT, class traits>
  basic_ofstream<charT, traits>::
  basic_ofstream(const char *s, ios_base::openmode mode, long protection)
  : basic_ostream<charT, traits>()
  {
       
    init(&_RWfb);
    open(s, mode, protection);
  }

  /*
   * basic_ofstream(int fd)
   * opens a filebuf for writing
   */
#ifndef _RWSTD_NO_EXTENSION

  template<class charT, class traits>
  basic_ofstream<charT, traits>::
  basic_ofstream(int fd)
  : basic_ostream<charT, traits>( )
  {

    init(&_RWfb);

    if ( !_RWfb.open(fd) ) 
      this->setstate(ios_base::failbit);
  }

  /*
   * basic_ofstream(int fd, char_type *buf, int len)
   * opens a filebuf for writing
   */

  template<class charT, class traits>
  basic_ofstream<charT, traits>::
  basic_ofstream(int fd, char_type *buf, int len)
  : basic_ostream<charT, traits>( )
  {

    init(&_RWfb);

    if ( ( buf!=0 ) && ( len>0 ) )
      _RWfb.pubsetbuf(buf,len);

    if ( !_RWfb.open(fd) ) 
      this->setstate(ios_base::failbit);
  }
#endif // _RWSTD_NO_EXTENSION

  /*
   * ~basic_ofstream()
   */

  template<class charT, class traits>
  basic_ofstream<charT, traits>::~basic_ofstream()
  {
  }

  /*
   * basic_filebuf *rdbuf() const
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits> *
  basic_ofstream<charT, traits>::rdbuf() const
  {
    return (basic_filebuf<charT, traits> *)&_RWfb;
  }

  /*
   * bool is_open()
   */

  template<class charT, class traits>
  bool basic_ofstream<charT, traits>::is_open()
  {
    return _RWfb.is_open();
  }

  /*
   * void open(const char *, ios_base::openmode)
   */

  template<class charT, class traits>
  void basic_ofstream<charT, traits>::open(const char *s,
                                           ios_base::openmode mode,
                                           long protection )
  {
    this->clear();
    mode |= ios_base::out;

    if ( !_RWfb.open(s, mode, protection) )
      this->setstate(ios_base::failbit);
  }

  /*
   * void close()
   */

  template<class charT, class traits>
  void basic_ofstream<charT, traits>::close()
  {
    if(!is_open())
      return;

    if(!_RWfb.close())
      this->setstate(ios_base::failbit);
    else
      this->clear();
  }

  /*
   * class basic_fstream : public basic_iostream
   */

  /*
   * basic_fstream()
   */

  template<class charT, class traits>
  basic_fstream<charT, traits>::basic_fstream()
  : basic_iostream<charT, traits>( )
  {
    init(&_RWfb);
  }

  /*
   * basic_fstream(const char *, ios_base::openmode, long)
   *
   * opens a filebuf for reading and writing
   */

  template<class charT, class traits>
  basic_fstream<charT, traits>::
  basic_fstream(const char *s, ios_base::openmode mode, long protection)
  : basic_iostream<charT, traits>( )
  {
    init(&_RWfb);
    open(s, mode, protection);
  }

  /*
   * basic_fstream(int fd)
   * opens a filebuf for reading and writing
   */
#ifndef _RWSTD_NO_EXTENSION

  template<class charT, class traits>
  basic_fstream<charT, traits>::
  basic_fstream(int fd)
  : basic_iostream<charT, traits>( )
  {

    init(&_RWfb);

    if ( !_RWfb.open(fd) ) 
      this->setstate(ios_base::failbit);
  }

  /*
   * basic_fstream(int fd, char_type *buf, int len)
   * opens a filebuf for reading and writing
   */

  template<class charT, class traits>
  basic_fstream<charT, traits>::
  basic_fstream(int fd, char_type *buf, int len)
  : basic_iostream<charT, traits>( )
  {

    init(&_RWfb);

    if ( ( buf!=0 ) && ( len>0 ) )
      _RWfb.pubsetbuf(buf,len);

    if ( !_RWfb.open(fd) ) 
      this->setstate(ios_base::failbit);
  }
#endif // _RWSTD_NO_EXTENSION


  /*
   * ~basic_fstream()
   */

  template<class charT, class traits>
  basic_fstream<charT, traits>::~basic_fstream()
  {
  }

  /*
   * basic_filebuf *rdbuf() const
   * returns the streambuf associated with this stream
   */

  template<class charT, class traits>
  basic_filebuf<charT, traits> *
  basic_fstream<charT, traits>::rdbuf() const
  {
    return (basic_filebuf<charT, traits> *)&_RWfb;
  }

  /*
   * bool is_open()
   */

  template<class charT, class traits>
  bool basic_fstream<charT, traits>::is_open()
  {
    return _RWfb.is_open();
  }

  /*
   * void open(const char *, ios_base::openmode, long)
   * opens up a file
   */

  template<class charT, class traits>
  void basic_fstream<charT, traits>::
  open(const char *s, ios_base::openmode mode, long protection)
  {
    this->clear();
    if ( !_RWfb.open(s, mode, protection) ) 
      this->setstate(ios_base::failbit);
  }

  /* 
   * void close()
   */

  template<class charT, class traits>
  void basic_fstream<charT, traits>::close()
  {
    if(!is_open())
      return;
  
    if(!_RWfb.close())
      this->setstate(ios_base::failbit);
    else
      this->clear();
  }

#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#   pragma __extern_prefix __restore
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __restore
#   endif
#endif
