#ifndef __IOMANIP_HXX
#define __IOMANIP_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	The iostream package
//
//  MODULE:	Header file of the iomanip subpackage
//
//  MODULE DESCRIPTION:
//
//	The iomanip subpackage defines macros for parameterized manipulators.
//	It consists of just this header file, because the code of all predefined
//	parameterized manipulators is contained in the iostream (core)
//	subpackage.
//
//  DESIGN ISSUES:
//
//	The second argument of operator<< is declared as const everywhere below
//	to prevent the creation of temporaries for nonconst references.  This
//	conforms to the Annotated C++ Reference Manual, unlike the interface of
//	iomanip documented for AT&T's Release 2.1.
//
//--

//++
//  INCLUDE FILES
//--

#include <compnent.hxx>
#ifdef __USE_STD_IOSTREAM 
#error "cannot include iomanip.hxx -- do not compile strict_ansi OR define __NO_USE_STD_IOSTREAM - see section 7.1.2 of the C++ Using Guide"
#else

#include <cxxl.hxx>
#include <generic.hxx>
#include <iostream.hxx>
#include <mutex.hxx>

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//+
//  LINKAGE
//--

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model  __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//++
//  MACRO DEFINITIONS
//--

#define SMANIP(T)name2(smanip_, T)
#define SAPP(T)name2(sapply_, T)

#define IMANIP(T)name2(imanip_, T)
#define IAPP(T)name2(iapply_, T)

#define OMANIP(T)name2(omanip_, T)
#define OAPP(T)name2(oapply_, T)

#define IOMANIP(T)name2(iomanip_, T)
#define IOAPP(T)name2(ioapply_, T)
//
// Same as above, only for reference objects
//
#define SMANIPREF(T)name2(smanipref_, T)
#define SAPPREF(T)name2(sapplyref_, T)

#define IMANIPREF(T)name2(imanipref_, T)
#define IAPPREF(T)name2(iapplyref_, T)

#define OMANIPREF(T)name2(omanipref_, T)
#define OAPPREF(T)name2(oapplyref_, T)

#define IOMANIPREF(T)name2(iomanipref_, T)
#define IOAPPREF(T)name2(ioapplyref_, T)


#define IOMANIPdeclare(T) \
\
class SMANIP(T) \
{ \
private: \
    ios &(*fct)(ios &, T); \
    T arg; \
public:	\
    SMANIP(T)(ios &(*f)(ios &, T), T a) : fct(f), arg(a) { } \
    friend istream &operator>>(istream &i, const SMANIP(T) &m) \
    { \
        ios *s = &i; \
        (*m.fct)(*s, m.arg); \
        return i; \
    } \
    friend ostream &operator<<(ostream &o, const SMANIP(T) &m) \
    { \
        ios *s = &o; \
        (*m.fct)(*s, m.arg); \
        return o; \
    } \
}; \
\
class SAPP(T) \
{ \
private: \
    ios &(*fct)(ios &, T); \
public: \
    SAPP(T)(ios &(*f)(ios &, T)) : fct(f) { } \
    SMANIP(T) operator()(T a) \
    { \
        return SMANIP(T)(fct, a); \
    } \
}; \
\
class IMANIP(T) \
{ \
private: \
    istream &(*fct)(istream &, T); \
    T arg; \
public: \
    IMANIP(T)(istream &(*f)(istream &, T), T a) : fct(f), arg(a) { } \
    friend istream &operator>>(istream &s, const IMANIP(T) &m) \
    { \
        return (*m.fct)(s, m.arg); \
    } \
}; \
\
class IAPP(T) \
{ \
private: \
    istream &(*fct)(istream &, T); \
public: \
    IAPP(T)(istream &(*f)(istream &, T)) : fct(f) { } \
    IMANIP(T) operator()(T a) \
    { \
        return IMANIP(T)(fct, a); \
    } \
}; \
\
class OMANIP(T) \
{ \
private: \
    ostream &(*fct)(ostream &, T); \
    T arg; \
public: \
    OMANIP(T)(ostream &(*f)(ostream &, T), T a ) : fct(f), arg(a) { } \
    friend ostream &operator<<(ostream &s, const OMANIP(T) &m) \
    { \
        return(*m.fct)(s, m.arg); \
    } \
}; \
\
class OAPP(T) \
{ \
private: \
    ostream &(*fct)(ostream &, T); \
public: \
    OAPP(T)(ostream &(*f)(ostream &, T)) : fct(f) { } \
    OMANIP(T) operator()(T a) \
    { \
        return OMANIP(T)(fct, a); \
    } \
}; \
\
class IOMANIP(T) \
{ \
private: \
    iostream &(*fct)(iostream &, T); \
    T arg; \
public: \
    IOMANIP(T)(iostream &(*f)(iostream &, T), T a) : fct(f), arg(a) { } \
    friend istream &operator>>(iostream &s, const IOMANIP(T) &m) \
    { \
        return(*m.fct)(s, m.arg); \
    } \
    friend ostream &operator<<(iostream &s, const IOMANIP(T) &m) \
    { \
        return(*m.fct)(s, m.arg); \
    } \
}; \
\
class IOAPP(T) \
{ \
private: \
    iostream &(*fct)(iostream &, T); \
public: \
    IOAPP(T)(iostream &(*f)(iostream &, T)) : fct(f) { } \
    IOMANIP(T) operator()(T a) \
    { \
        return IOMANIP(T)(fct, a); \
    } \
};

//
// The macros for REF manipulators.
//
#define IOMANIPREFdeclare(T) \
\
class SMANIPREF(T) \
{ \
private: \
    ios &(*fct)(ios &, T & ); \
    T &arg; \
public:	\
    SMANIPREF(T)(ios &(*f)(ios &, T &), T &a) : fct(f), arg(a) { } \
    friend istream &operator>>(istream &i, const SMANIPREF(T) &m) \
    { \
        ios *s = &i; \
        (*m.fct)(*s, m.arg); \
        return i; \
    } \
    friend ostream &operator<<(ostream &o, const SMANIPREF(T) &m) \
    { \
        ios *s = &o; \
        (*m.fct)(*s, m.arg); \
        return o; \
    } \
}; \
\
class SAPPREF(T) \
{ \
private: \
    ios &(*fct)(ios &, T &); \
public: \
    SAPPREF(T)(ios &(*f)(ios &, T &)) : fct(f) { } \
    SMANIPREF(T) operator()(T &a) \
    { \
        return SMANIPREF(T)(fct, a); \
    } \
}; \
\
class IMANIPREF(T) \
{ \
private: \
    istream &(*fct)(istream &, T &); \
    T &arg; \
public: \
    IMANIPREF(T)(istream &(*f)(istream &, T &), T &a) : fct(f), arg(a) { } \
    friend istream &operator>>(istream &s, const IMANIPREF(T) &m) \
    { \
        return (*m.fct)(s, m.arg); \
    } \
}; \
\
class IAPPREF(T) \
{ \
private: \
    istream &(*fct)(istream &, T &); \
public: \
    IAPPREF(T)(istream &(*f)(istream &, T &)) : fct(f) { } \
    IMANIPREF(T) operator()(T &a) \
    { \
        return IMANIPREF(T)(fct, a); \
    } \
}; \
\
class OMANIPREF(T) \
{ \
private: \
    ostream &(*fct)(ostream &, T &); \
    T &arg; \
public: \
    OMANIPREF(T)(ostream &(*f)(ostream &, T &), T &a ) : fct(f), arg(a) { } \
    friend ostream &operator<<(ostream &s, const OMANIPREF(T) &m) \
    { \
        return(*m.fct)(s, m.arg); \
    } \
}; \
\
class OAPPREF(T) \
{ \
private: \
    ostream &(*fct)(ostream &, T &); \
public: \
    OAPPREF(T)(ostream &(*f)(ostream &, T &)) : fct(f) { } \
    OMANIPREF(T) operator()(T &a) \
    { \
        return OMANIPREF(T)(fct, a); \
    } \
}; \
\
class IOMANIPREF(T) \
{ \
private: \
    iostream &(*fct)(iostream &, T &); \
    T &arg; \
public: \
    IOMANIPREF(T)(iostream &(*f)(iostream &, T &), T &a) : fct(f), arg(a) { } \
    friend istream &operator>>(iostream &s, const IOMANIPREF(T) &m) \
    { \
        return(*m.fct)(s, m.arg); \
    } \
    friend ostream &operator<<(iostream &s, const IOMANIPREF(T) &m) \
    { \
        return(*m.fct)(s, m.arg); \
    } \
}; \
\
class IOAPPREF(T) \
{ \
private: \
    iostream &(*fct)(iostream &, T &); \
public: \
    IOAPPREF(T)(iostream &(*f)(iostream &, T &)) : fct(f) { } \
    IOMANIPREF(T) operator()(T &a) \
    { \
        return IOMANIPREF(T)(fct, a); \
    } \
};

//++
//  MANIPULATORS
//
//	Declare manipulators and applicators for types int and long, Mutex  
//	Then declare all the predefined parameterized manipulators:  
//	resetiosflags, setfill, setiosflags, setprecision, setw, lock and
//	unlock.
//--

IOMANIPdeclare(int)
IOMANIPdeclare(long)

SMANIP(long) resetiosflags(long);
SMANIP(int)  setfill(int);
SMANIP(long) setiosflags(long);
SMANIP(int)  setprecision(int);
SMANIP(int)  setw(int);

IOMANIPREFdeclare(Mutex)

SMANIPREF(Mutex) lock(Mutex &);
SMANIPREF(Mutex) unlock(Mutex &);


#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif // __USE_STD_IOSTREAM

#endif

// End of iomanip.hxx
