/***************************************************************************
 *
 * ios.cc - Definition for the Standard Library iostreams
 *
 ***************************************************************************
 *    
 *  Copyright 2000 Compaq Computer Corporation
 *
 *  COMPAQ Registered in U.S. Patent and Trademark Office.
 *
 *  Confidential computer software. Valid license from Compaq required for
 *  possession, use or copying. Consistent with FAR 12.211 and 12.212,
 *  Commercial Computer Software, Computer Software Documentation, and
 *  Technical Data for Commercial Items are licensed to the U.S. Government
 *  under vendor's standard commercial license.
 *
 ****************************************************************************
 *
 * (c) Copyright 1994, 1998 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#include <streambuf>
#include <iosfwd>


#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __save
#      pragma __environment __header_defaults
#   endif
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#pragma __extern_prefix __save
#pragma __extern_prefix "CXXL$" 
#endif


#ifndef _RWSTD_NO_NAMESPACE
namespace std { 
#endif
  
  extern istream _RWSTDExport cin;

#ifndef _RWSTD_NO_WIDE_CHAR
  extern wistream _RWSTDExport wcin;
#endif


  /*
   * Class basic_ios  member functions   
   */

  /*
   * basic_ios(basic_streambuf *)
   */

  template<class charT, class traits>
  basic_ios<charT, traits>::
  basic_ios(basic_streambuf<charT, traits> *sb_arg)
  {
    init(sb_arg);
  }

  /*
   * ~basic_ios()
   */

  template<class charT, class traits>
  basic_ios<charT, traits>::~basic_ios()
  {

  }



  /*
   * ios_type& copyfmt(const ios_type& )
   */

  template<class charT, class traits>
  basic_ios<charT, traits>&
  basic_ios<charT, traits>::
  copyfmt(const basic_ios<charT, traits>& rhs)
  {
#ifdef _RWSTD_MULTI_THREAD
    _RWSTDGuard guard(this->_RWstream_mutex);
#endif
    _RWtiestr = rhs._RWtiestr;
    _RWfillch = rhs._RWfillch;
    ios_base::_RW_UNSAFE_copyfmt( rhs );
    _Except = rhs._Except;

    return *this;
  }


  /*
   * basic_ostream *tie(basic_ostream *)
   */

  template<class charT, class traits>
  basic_ostream<charT, traits> *
  basic_ios<charT, traits>::
  tie(basic_ostream<charT, traits> *tie_arg)
  {
#ifdef _RWSTD_MULTI_THREAD
    _RWSTDGuard guard(this->_RWstream_mutex);
#endif
    basic_ostream<charT, traits>   *temp = _RWtiestr;
    _RWtiestr = tie_arg;
    return temp;
  }

  /*
   * basic_streambuf *rdbuf(basic_streambuf *)
   */

  template<class charT, class traits>
  _TYPENAME basic_ios<charT, traits>::streambuf_type* 
  basic_ios<charT, traits>::                     
  rdbuf( basic_streambuf<charT, traits> *sb)
  {
#ifdef _RWSTD_MULTI_THREAD
    _RWSTDGuard guard(this->_RWstream_mutex);
#endif

    basic_streambuf<charT, traits>   *temp = _RWsb;

    _RWsb = sb;

    if ( sb==0 ) 
      _RWstate |= ios_base::badbit;
 
    _RW_UNSAFE_clear();

    return temp;
  }

  /*
   * basic_ios()
   */

  template<class charT, class traits>
  basic_ios<charT, traits>::basic_ios()
  {
    init(0);
  }

  /*
   * void imbue(const locale& )
   */

  template<class charT, class traits>
  locale basic_ios<charT, traits>::
  imbue(const locale& loc)
  {
    locale tmp = getloc();
   
    ((ios_base *)(this))->imbue(loc);
   
    if ( rdbuf() )
      rdbuf()->pubimbue(loc);

    return tmp;  
  }

  /*
   * void init(basic_streambuf *)
   */

  template<class charT, class traits>
  void basic_ios<charT, traits>::
  init(basic_streambuf<charT, traits> *sb)
  {
#ifdef _RWSTD_MULTI_THREAD
    _RWSTDGuard guard(this->_RWstream_mutex);
#endif
    _RWsb = sb;
    _RWtiestr = 0;

    if(_RWsb)
      _RWstate = goodbit;
    else
      _RWstate = badbit;

    _Except = goodbit;
    _RWfmtfl  = skipws | dec;
    _RWwide   = 0;
    _RWprec   = 6;

    _RWfillch = widen(' ');

  }

  /*
   * lock the buffer
   */

#ifdef _RWSTD_MULTI_THREAD
  template <class charT, class traits>
  basic_ios<charT,traits>& _RWlock(basic_ios<charT,traits>& strm)
  {
    if ( strm.rdbuf() )
      strm.rdbuf()->_RW_lock_buffer();

    return strm;
  }

  template <class charT, class traits>
  basic_ios<charT,traits>& _RWunlock(basic_ios<charT,traits>& strm)
  {
    if ( strm.rdbuf() )
      strm.rdbuf()->_RW_unlock_buffer();

    return strm;
  }

#endif //  _RWSTD_MULTI_THREAD


#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#   pragma __extern_prefix __restore
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __restore
#   endif
#endif
