#ifndef	__IOSTREAM_HXX
#define	__IOSTREAM_HXX
#define	IOSTREAMH

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	The iostream package.
//
//  MODULE:	Include file for the iostream (core) subpackage.
//
//  MODULE DESCRIPTION:
//
//	The iostream (core) subpackage provides all the basic functionality
//	of the iostream package in a self-contained subset of classes.  In
//	addition, some of its base classes are used for derivation of classes
//	in iostream's other subpackages, and in other packages such as String.
//
//  DESIGN ISSUES:
//
//	The design of the Iostream_init class, as described in AT&T's
//	documentation, seems to rely on cfront's bug in its handling of extern
//	declarations, and therefore could not be implemented here in a
//	compatible fashion.
//
//--

//++
//  INCLUDE FILES
//--

#include <compnent.hxx>
#ifdef __USE_STD_IOSTREAM 
#error "cannot include iostream.hxx -- do not compile strict_ansi OR define __NO_USE_STD_IOSTREAM - see section 7.1.2 of the C++ Using Guide"
#else

#include <cxxl.hxx>
#include <messages.hxx>

extern "C"
{
// platform specific include files, for backwards compatibility
#if defined(__VMS)
#   include <stdlib.h>
#   include <stdio.h>
#   include <string.h>
#   if defined(__VMS_POSIX)
#	include <unistd.h>
#   else
#	include <unixio.h>
#   endif
#endif

#if defined(__unix__)
#   include <memory.h>
#endif
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//++
//  LINKAGE
//--

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model  __relaxed_refdef
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif


//++
//  GLOBAL DEFINITIONS
//--

typedef long streamoff, streampos;

#ifndef EOF
#   define EOF (-1)
#endif

#ifndef zapeof
#   define zapeof(c) ((c)&0377)
#endif

class streambuf;
class ostream;
class Mutex;

class ios

//++
//  FUNCTIONAL DESCRIPTION:
//
//	The stream classes derived from class ios provide a high level
//	interface that supports transferring formatted and unformatted
//	information into and out of streambufs.
//
//	Several enumerations are declared in class ios (open_mode, io_state,
//	seek_dir, and format flags) to avoid polluting the global name space.
//--

{
public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    enum io_state	{ goodbit = 0, eofbit = 01, failbit = 02,
			  badbit = 04 };
    enum open_mode      { in = 01, out = 02, ate = 04, app = 010, trunc = 020,
                          nocreate = 040, noreplace = 0100 };
    enum seek_dir       { beg = 0, cur = 01, end = 02 };

    // Flags for controlling format.
    enum                { skipws = 01,
                            // Skip whitespace on input.
                          left = 02, right = 04, internal = 010,
                            // Padding location.
                          dec = 020, oct = 040, hex = 0100, 
                            // Conversion base.
                          showbase = 0200, showpoint = 0400, uppercase = 01000,
                          showpos = 02000,
                            // Modifiers.
                          scientific = 04000, fixed = 010000,
                            // Floating point notation.
                          unitbuf = 020000, stdio = 040000 };
                            // Flushing options.

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif
                        ios(streambuf *);
    virtual             ~ios();

    inline int		bad() const;
    static long         bitalloc();
    inline void		clear(int state = 0);
    inline int		eof() const;
    inline int		fail() const;
    inline char		fill() const;
    char		fill(char);
    inline long		flags() const;
    long                flags(long);
    inline int		good() const;
    long		&iword(int);
    inline int		operator!();
    inline		operator void *();
    inline int		precision() const;
    int                 precision(int);
    void		*&pword(int);
    inline streambuf	*rdbuf();
    inline int		rdstate() const;
    long                setf(long setbits, long field);
    long                setf(long);
    static void         sync_with_stdio();
    inline ostream	*tie() const;
    ostream		*tie(ostream *);
    long                unsetf(long);
    inline int		width() const;
    int			width(int);
    static int          xalloc();
#if defined(__osf__) || defined(__VAX)
    inline Mutex 	*__get_mutex_info() { return m_mutex_info; }
#endif
#if defined(__VMS) && defined(__alpha)
    inline long 	__get_mutex_info() { return m_mutex_info; }
#endif
protected:
                        ios();
    void                init(streambuf *);

    void		setstate(int state);
    inline void		__setstate(int state);
    inline int		__width(int);
private:		
                        ios(ios &);
    void                operator=(ios &);

// Data member definitions -- each platform has own backward compatible layout
#if defined(__VMS)
private:
    streambuf* volatile	m_psb;
    volatile char       m_chFill;
#if defined(__alpha)
    char		m_mutex_info;
#endif
    volatile long       m_lFlags;
    volatile short      m_iPrecision;
protected:
    volatile int        state;
private:
    ostream * volatile  m_posTie;
    volatile short      m_cchField;
#if defined(__VAX) 
    Mutex *		m_mutex_info;	// __xalloc_buf in previous version
    int			__fill2;	// __xalloc_size in previous version
#endif
#endif

#if defined(__unix__) 
private:
    long		  m_nXalloc;
    long *		  m_pXalloc;
protected:
    streambuf* volatile	  m_psb;
    volatile int	  state;
    int			  ispecial;
    int			  ospecial;
    int			  m_is_fstream;		// allow __flush to
						// check for fstream
						// VMS will use __fd()
    int			  osfx_special;
    int			  __fill4;
    ostream* volatile 	  m_posTie;
    volatile long	  m_lFlags;
    volatile short	  m_iPrecision;
    volatile char	  m_chFill;
    char		  __fill5;
    volatile short	  m_cchField;
    short		  __fill6;
#if defined(__osf__)
    Mutex *		  m_mutex_info;
#endif
#endif

// Static data member definitions

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif
public:
    static const long   __CXXL_XFER_DECL(basefield);
                            // dec | oct | hex
    static const long   __CXXL_XFER_DECL(adjustfield);
                            // left | right | internal
    static const long   __CXXL_XFER_DECL(floatfield);
                            // scientific | fixed
private:
    static long		__CXXL_XFER_DECL(cxxl_bitalloc);
    static int		__CXXL_XFER_DECL(cxxl_xalloc);

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

#if defined(__unix__)
protected:
    static int		xalloc_count() {return cxxl_xalloc;}
public:
    int                 skip(int);     // compatibility with previous version
#endif
    friend class	Iostream_init;
    friend class	istream_withassign;
    friend class	ostream_withassign;
    friend class	iostream_withassign;
};

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

class streambuf

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This base class implements the buffer abstraction for streams.
//	Characters can be inserted (put) into buffers or extracted (gotten) out
//	of them.
//--

{
public:
                        streambuf();
                        streambuf(char *p, int len);

    void                dbp();
#if defined(__alpha) && (defined(__osf__) || defined(__VMS))
    inline void		__set_mutex_info(int);
    inline int		__get_mutex_info() { return m_mutex_info; }
#endif

private:
                        streambuf(streambuf &);
    void                operator=(streambuf &);
    int			snextcUnderflow();
    int			__gets(char *ptr, int n, int flag);
    int			__getl(long &l, int skipws, int base);
    int			__getul(unsigned long &l, int skipws, int base);
    int			__getd(double &d, int skipws);

protected:
    int			allocate();
    inline char		*base();
    inline int		blen();
    inline char		*eback();
#if defined(__VMS) && (__VMS_VER < 70000000) 
    inline char		*ebuf();
    inline void		gbump(int n);
    inline void		pbump(int n);
    inline void		unbuffered(int n);
#else
    char		*ebuf();
    void		gbump(int n);
    void		pbump(int n);
    void		unbuffered(int n);
#endif
    inline char		*egptr();
    inline char		*epptr();
    inline char		*gptr();
    inline char		*pbase();
    inline char		*pptr();
    void		setb(char *b, char *eb, int a = 0);
    void		setg(char *eb, char *g, char *eg);
    void		setp(char *p, char *ep);
    inline int		unbuffered();
#ifdef __VMS
    inline char         *GetDataMaxPtr();
    inline void         SetDataMaxPtr(char *b);
#else
    char         	*GetDataMaxPtr();
    void         	SetDataMaxPtr(char *b);
#endif
    int			__allocate();
    inline void		__gbump(int n);
    inline void		__pbump(int n);
    void		__setb(char *b, char *eb, int a = 0);
    void		__setg(char *eb, char *g, char *eg);
    void		__setp(char *p, char *ep);
    inline char         *__GetDataMaxPtr();
    inline void         __SetDataMaxPtr(char *b);

public:
#if defined(__VMS) && (__VMS_VER < 70000000) 
    inline int		in_avail();
    inline int		out_waiting();
    inline int		sbumpc();
    inline int		sgetc();
    inline int		snextc();
#else
    int		        in_avail();
    int			out_waiting();
    int			sbumpc();
    int			sgetc();
    int			snextc();
#endif
    streambuf		*setbuf(char *ptr, int len, int i);
    int			sgetn(char *ptr, int n);
    int			sputbackc(char c);
    int			sputc(int c = EOF);
    int			sputn(const char *s, int n);
    void		stossc();
    inline int		__in_avail();
    inline int		__out_waiting();
    inline int		__sbumpc();
    inline int		__sgetc();
    inline int		__sgetn(char *ptr, int n);
    inline int		__snextc();
    inline int		__sputbackc(char c);
    int			__sputc(int c = EOF);
    inline int		__sputn(const char *s, int n);
    inline void		__stossc();
    inline  int         __FastFill(const char, int);
    inline  int         __FastSputn(const char*, int);

// Virtual member functions -- each platform has own backward compatible layout
#if defined(__VMS)
public:
    virtual             ~streambuf();
protected:
    virtual int         doallocate();
public:
    virtual int         overflow(int c = EOF);
    virtual int         pbackfail(int c);
    virtual streampos   seekpos(streampos, int = ios::in | ios::out);
    virtual streampos   seekoff(streamoff, __SCOPE(ios::)seek_dir,
				int = ios::in | ios::out);
    virtual streambuf	*setbuf(char *ptr, int len);
    virtual int         sync();
    virtual int         underflow();
#endif

#if defined(__osf__) || defined(__linux)
protected:
    virtual int         doallocate();
public:
    virtual int         overflow(int c = EOF);
    virtual int         underflow();
    virtual int         pbackfail(int c);
    virtual int         sync();
    virtual streampos   seekoff(streamoff, __SCOPE(ios::)seek_dir,
				int = ios::in | ios::out);
    virtual streampos   seekpos(streampos, int = ios::in | ios::out);
#if defined (__osf__)
    virtual int		xsputn(const char *, int);
    virtual int		xsgetn(char *, int);
#endif
    virtual streambuf	*setbuf(char *ptr, int len);
    virtual             ~streambuf();
#endif

// System specific member functions
#if defined(__VMS)
public:
// VMS has public functions for accessing streambuf file info
    inline int		fd();
    inline FILE		*fp();
#if (__VMS_VER < 70000000) 
    inline void		fd(int);
    inline void		fp(FILE *);
#else
    void		fd(int);
    void		fp(FILE *);
#endif
    void		__fd(int);
private:
// VMS has private obsolete functions accessed through the shard library vector
    int			StdioSgetc();
    int			StdioSnextc();
#endif

#if defined(__osf__)
private:
// OSF has private obsolete functions accessed through old inline code
    int			x_snextc();
public:
    streambuf*		setbuf(unsigned char *, int);
#endif

// Data member definitions -- each platform has own backward compatible layout
#if defined(__VMS)
protected:
    // The following field contains two flags and a 2**29-1 count for filebuf
    // the flags and the count were placed here to avoid increasing the size
    // of filebuf objects and breaking backward compatiblity.
    // The flags are in the 3 least significant bits of the field,
    // the count is in the rest of the field.
    // This field is private to the streambuf and filebuf classes.

    enum buffer_state 	{ DoAutoDelete = 01,	// streambuf flag
			  OwnsFd = 02,		// filebuf flag
			  NeedsSync = 04 };	// filebuf flag
    unsigned int	m_bBufferFlags;		// 3 flags for stream/file buf 
						// changed file pos for filebuf
    char * volatile 	m_pchDataMax;		// buffer maximum size
    char * volatile 	m_pchBase;		// base of buffer
    volatile int        m_cchBuffer;		// length of buffer
    char * volatile 	m_pchEback;		// 
    char * volatile 	m_pchEgptr;		//
    char * volatile 	m_pchEpptr;		//
    volatile int	m_fd;			// file descriptor for filebuf
    FILE * volatile 	m_pfile;		// file variable
    char * volatile 	m_pchGptr;		//
    char * volatile 	m_pchPbase;		//
    char * volatile 	m_pchPptr;		//
    volatile short      m_bIsOkToAllocate;	// buffered/unbuffered flag
#if defined(__alpha)
    int			m_mutex_info;		// Mutex array index
#endif
#endif

#if defined(__unix__)
private:
    short	        m_bDoAutoDelete;
    volatile short      m_bIsOkToAllocate;
#if defined(__osf__)
    int  		m_mutex_info;		// Mutex array index
#endif
    char * volatile	m_pchBase;
    char * volatile	m_pchPbase;
    char * volatile	m_pchPptr;
    char * volatile	m_pchEpptr;
    char * volatile	m_pchGptr;
    char * volatile	m_pchEgptr;
    char * volatile	m_pchEback;
    volatile int        m_cchBuffer;
    volatile int        m_cchDataMax;
#endif

    friend class	istream;
    friend void 	ios::sync_with_stdio();
};



class istream : virtual public ios

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class provides formatted and unformatted input (extraction) from
//	an associated streambuf.
//--

{
public:
                        istream(streambuf *);
    virtual             ~istream();

    istream		&get(char *ptr, int len, char delim = '\n');
    istream		&get(unsigned char *ptr, int len, char delim = '\n');
    istream		&get(char &);
    inline istream	&get(unsigned char &);
    istream		&get(streambuf &sb, char delim = '\n');
    int			get();
    istream		&getline(char *ptr, int len, char delim = '\n');
    istream		&getline(unsigned char *ptr, int len,
						     char delim = '\n');
    istream             &ignore(int len = 1, int delim = EOF);
    int			ipfx(int need = 0);
    void		isfx();
    int			peek();
    istream		&putback(char);
    istream             &read(char *s, int n);
    inline istream	&read(unsigned char *s, int n);
    istream		&seekg(streampos);
    istream		&seekg(streamoff, __SCOPE(ios::)seek_dir);
    void                skipwhite();
    int			sync();
    streampos           tellg();
    istream		&operator>>(char *);
    istream		&operator>>(char &);
    istream		&operator>>(short &);
    istream		&operator>>(int &);
    istream		&operator>>(long &);
#if defined(__VMS) && defined(__alpha)
    istream		&operator>>(__int64 &);
#else
#if defined(__DECCXX) && !(defined(__STD_STRICT_ANSI) || defined(__STD_STRICT_ANSI_ERRORS)) && !defined(__CXXLFIX634)
    istream		&operator>>(long long& ld) 
	{ long l; if (*this >> l) ld = (long long) l; return *this; }
    istream		&operator>>(unsigned long long& ld) 
	{ long l; if (*this >> l) ld = (unsigned long long) l; return *this; }
#endif
#endif
    inline int		gcount();

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_FLOAT_FUNC_LINKAGE
#endif

    istream		&operator>>(float &);
    istream		&operator>>(double &);
#if __X_FLOAT && defined(__osf__)
    istream		&operator>>(long double & ld);
#else
    istream		&operator>>(long double & ld) 
	{ double d; if (*this >> d) ld = (long double) d; return *this; }
#endif
#if defined(__DECCXX) && defined(__DECCXX_VER) && (__DECCXX_VER >= 60000000) && defined(__BOOL_IS_A_RESERVED_WORD)
    istream		&operator>>(bool& b) 
	{ int i; if (*this >> i) b = (bool) i; return *this; }
#endif

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    istream		&operator>>(unsigned char *);
    istream		&operator>>(unsigned char &);
    istream		&operator>>(unsigned short &);
    istream		&operator>>(unsigned int &);
    istream		&operator>>(unsigned long &);
#if defined(__VMS) && defined(__alpha)
    istream		&operator>>(unsigned __int64 &);
#endif
    istream		&operator>>(streambuf *);
    inline istream	&operator>>(istream &(*f)(istream &));
    istream		&operator>>(ios &(*f)(ios &));

protected:
			istream();

private:
    volatile int        m_cchGet;
    int			__ipfx(int need = 0);

#if defined(__osf__)	
//  The following are provided for binary compatibility with previous
//  versions of DEC OSF/1
    int			do_ipfx(int need = 0);
    void		xget(char *);
//  The following are provided for compatibility with obsolete stream
//  interfaces that are provided for binary compatibility with previous
//  versions of DEC OSF/1
public:
    /* obsolete */	istream(streambuf *, int, ostream * = 0);
    /* obsolete */	istream(int, int, ostream * = 0);
    /* obsolete */	istream(int, char *, int);
#endif
    friend class	istream_withassign;
    friend class	iostream_withassign;
};

class ostream : virtual public ios

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class provides formatted and unformatted output (insertion) into
//	an associated streambuf.
//--

{
public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

			ostream(streambuf *);
    virtual		~ostream();

    ostream             &flush();
    inline ostream	&__flush_tie();
    int			opfx();
    void		osfx();
    ostream		&put(char);
    ostream             &seekp(streampos);
    ostream             &seekp(streamoff, __SCOPE(ios::)seek_dir);
    streampos           tellp();                   
    ostream		&write(const char *ptr, int n);
    inline ostream	&write(const unsigned char *ptr, int n);
    ostream             &operator<<(const char *);
    ostream		&operator<<(char);
    inline ostream	&operator<<(short);
    ostream             &operator<<(int);
    ostream             &operator<<(long);
#if defined(__VMS) && defined(__alpha)
    ostream             &operator<<(__int64);
#endif
    inline ostream	&operator<<(unsigned char);
    inline ostream	&operator<<(unsigned short);
    ostream		&operator<<(unsigned int); 
    ostream             &operator<<(unsigned long);
#if defined(__VMS) && defined(__alpha)
    ostream             &operator<<(unsigned __int64);
#else
#if defined(__DECCXX) && !(defined(__STD_STRICT_ANSI) || defined(__STD_STRICT_ANSI_ERRORS)) && !defined(__CXXLFIX634)
    inline ostream      &operator<<(long long);
    inline ostream      &operator<<(unsigned long long);
#endif
#endif
    ostream             &operator<<(const void *);
    ostream             &operator<<(streambuf *);
    inline ostream	&operator<<(ostream &(*f)(ostream &));
    ostream             &operator<<(ios &(*f)(ios &));

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_FLOAT_FUNC_LINKAGE
#endif

    ostream             &operator<<(float);
    ostream             &operator<<(double);
#if __X_FLOAT && defined(__osf__)
    ostream      &operator<<(long double);
#else
    inline ostream      &operator<<(long double);
#endif
#if defined(__DECCXX) && defined(__DECCXX_VER) && (__DECCXX_VER >= 60000000) && defined(__BOOL_IS_A_RESERVED_WORD)
    inline ostream	&operator<<(bool);
#endif

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

#if defined(__VMS)
    // OpenVMS specific function
    ostream		&operator<<(const unsigned char *);
#endif

protected:
                        ostream();

private:
    void		__WriteJustifiedString(const char *,  const char *);
    int			__opfx();
    ostream             &__flush();

#if defined(__osf__)
//  The following functions are obsolete and only provided for compatibility
//  with the stream interface that was provided on DEC OSF/1
public:
    			ostream(int);
			ostream(int, char*);
#if defined(__CXXL_IOSTREAM_CXX)
//  The following functions are obsolete and only provided for binary
//  compatibility with the previous iostream implementation on DEC OSF/1
    int			do_opfx();
    void		do_osfx();
    ostream             &operator<<(void *);
#endif
#endif
};

class iostream : public istream, public ostream

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class, derived from both istream and ostream, is intended for
//	bidirectional I/O operations on a single stream of characters.
//--

{
public:
			iostream(streambuf *);
    virtual		~iostream();

protected:
			iostream();
};

class istream_withassign : public istream

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class adds an assignment operator, as well as a constructor with
//	no arguments, to the istream class.
//--

{
public:
			istream_withassign();
    virtual		~istream_withassign();

    istream_withassign	&operator=(istream &);
    istream_withassign	&operator=(streambuf *);
    istream_withassign	&operator=(istream_withassign& s) { return operator=((istream&) s); }
};

class ostream_withassign : public ostream

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class adds an assignment operator, as well as a constructor with
//	no arguments, to the ostream class.
//--

{
public:
			ostream_withassign();
    virtual		~ostream_withassign();

    ostream_withassign	&operator=(ostream &);
    ostream_withassign	&operator=(streambuf *);
    ostream_withassign	&operator=(ostream_withassign& s) { return operator=((ostream&) s); }
};

class iostream_withassign : public iostream

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class adds an assignment operator, as well as a constructor with
//	no arguments, to the iostream class.
//--

{
public:
			iostream_withassign();
    virtual		~iostream_withassign();

    iostream_withassign	&operator=(iostream &);
    iostream_withassign	&operator=(streambuf *);
    iostream_withassign	&operator=(iostream_withassign& s) { return operator=((iostream&) s); }
};

class Iostream_init

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This class is present for technical reasons relating to initialization.
//	It has no public members.  The Iostream_init constructor initializes
//	the predefined streams.  Because an object of this class is declared
//	here in the header file, the constructor is called once each time the
//	header is included (although the real initialization is done only
//	once), and therefore the predefined streams will be initialized before
//	they are used (in some cases, global constructors may need to call the
//	Iostream_init constructor explicitly to ensure this).
//--

{
public:
			Iostream_init();
			~Iostream_init();

    void		initialize();

private:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    static int		__CXXL_XFER_DECL(initcount);

    friend class	ios;
};

//++
//  GLOBAL DECLARATIONS
//--


//  Predefined streams
//
//	cin   - The standard input (file descriptor 0), buffered.
//	cout  - The standard output (file descriptor 1), buffered.
//	cerr  - The standard error (file descriptor 2), unit-buffered (that
//		    is, characters are flushed after each inserter operation).
//	clog  - The standard error (file descriptor 2), buffered.

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

extern istream_withassign __CXXL_XFER_DECL(cin);
extern ostream_withassign __CXXL_XFER_DECL(cout);
extern ostream_withassign __CXXL_XFER_DECL(cerr);
extern ostream_withassign __CXXL_XFER_DECL(clog);


//  Manipulators
//
//	A manipulator is a value that can be inserted into or extracted from a
//	stream to cause some special side effect.

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

ios	&dec(ios &);
ios	&hex(ios &);
ios	&oct(ios &);
ostream	&endl(ostream &i);
ostream	&ends(ostream &i);
ostream	&flush(ostream &);
istream	&ws(istream &);
ios	&lock(ios &);
ios	&unlock(ios &);


inline int ios::bad(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to check if badbit is set in the error state.
//	If the bit is set, it usually indicates that some operation has failed
//	with a severe error from which recovery is probably impossible.  I.e.,
//	it will probably be impossible to continue I/O operations on the stream.
//
//  RETURN VALUE:
//
//	Non-zero if badbit is set in the error state; zero otherwise.
//--

{
    return state & badbit;
}

inline void ios::clear(
    int __state  // The new value of the state variable; defaults to zero.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Stores the value of its argument as the error state.  If the value is
//	zero, this clears all state bits.  To set a bit without clearing other
//	bits requires something like clear(ios::badbit | s.rdstate()).
//
//  SIDE EFFECTS:
//
//	Modifies the error state as described above.
//--

{
    state = __state & (eofbit | failbit | badbit);
}

inline int ios::eof(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to check if eofbit is set in the error state.
//	Normally, this bit is set when and end-of-file has been encountered
//	during an extraction.
//
//  RETURN VALUE:
//
//	Non-zero if eofbit is set in the error state; zero otherwise.
//--

{
    return state & eofbit;
}

inline int ios::fail(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to check if either badbit or failbit is set in
//	the error state.  See the description of the bad() function above for
//	information about the bad bit.  If the fail bit is set, this normally
//	indicates that some extraction or conversion has failed, but the stream
//	is still usable.  That is, once the failbit is cleared, I/O on the
//	stream can usually continue.
//
//  RETURN VALUE:
//
//	Non-zero if either badbit or failbit is set in the error state; zero
//	otherwise.
//
//--

{
    return state & (failbit | badbit);
}

inline char ios::fill(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get the current fill character.
//
//  RETURN VALUE:
//
//	The value of the fill-character format state variable.
//--

{
    return m_chFill;
}

inline long ios::flags(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get the current format flags mask.
//
//  RETURN VALUE:
//
//	The value of the format flags variable.
//--

{
    return m_lFlags;
}

inline int ios::good(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to check if the stream is in "good" state,
//	meaning that it has no error bits set.
//
//  RETURN VALUE:
//
//	Non-zero if the error state has no bits set; zero otherwise.
//
//--

{
    return state == goodbit;
}

inline int ios::operator!(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This operator allows convenient checking of the error state of an ios
//	by using expressions such as "if (cin >> x) ...".
//
//  RETURN VALUE:
//
//	Non-zero if failbit or badbit is set in the error state; zero otherwise.
//--

{
    return state & (failbit | badbit);
}

inline ios::operator void *(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This operator allows convenient checking of the error state of an ios
//	by using expressions such as "if (cin) ..." and "if (cin >> x) ...".
//	The operator converts an ios to a pointer so that it can be compared to
//	zero.  The conversion will return a non-zero pointer value (not meant to
//	be used) if failbit and badbit are not set in the error state.
//
//  RETURN VALUE:
//
//	As described above.
//--

{
    return (state & (failbit | badbit)) ? 0 : this;
}

inline int ios::precision(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get the current precision used by floating
//	point inserters.
//
//  RETURN VALUE:
//
//	The precision format state variable.
//--

{
    return m_iPrecision;
}

inline streambuf *ios::rdbuf(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the streambuf associated
//	with the current stream.
//
//  RETURN VALUE:
//
//	The pointer-to-streambuf variable.
//--

{
    return m_psb;
}

inline int ios::rdstate(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get the current error state of the stream.
//
//  RETURN VALUE:
//
//	The error state variable.
//--

{
    return state;
}

//==============================================================
//  Method: ios::__setstate()
//
//  FUNCTIONAL DESCRIPTION:
//
//      This function is called to selectively set state bits.  Unlike
//      clear(int state), this function only changes the bits
//      specified in the state argument.  This function is for internal
//	use only.  It does no locking.
//
//  SIDE EFFECTS:
//
//      The streambuf's state variable is modified.
//
//--------------------------------------------------------------
inline void ios::__setstate(
    int __state)  // A mask of the state bits to be set.
{
    state |= __state & (eofbit | failbit | badbit);
}

inline ostream *ios::tie(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the ostream which is
//	automatically flushed when the current stream needs more characters or
//	has characters to be consumed.
//
//  RETURN VALUE:
//
//	The tie variable.
//--

{
    return m_posTie;
}

inline int ios::width(
    ) const

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the current field width used by inserters and extractors. 
//
//  RETURN VALUE:
//
//	The field-width format variable.
//--

{
    return m_cchField;
}

inline char *streambuf::base(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the first byte of the
//	reserve area.  Thus, the reserve area is the space between base() and
//	ebuf().
//
//  RETURN VALUE:
//
//	The streambuf's base variable.
//--

{
    return m_pchBase;
}

inline int streambuf::blen(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get the size, in chars, of the current
//	reserve area.
//
//  RETURN VALUE:
//
//	The streambuf's length variable.
//--

{
    return m_cchBuffer;
}

inline char *streambuf::eback(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the lower bound imposed on
//	gptr().  Space between eback() and gptr() is available for putback.
//
//  RETURN VALUE:
//
//	The streambuf's end-of-putback variable.
//--

{
    return m_pchEback;
}

inline char *streambuf::egptr(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the byte after the last byte
//	of the get area.
//
//  RETURN VALUE:
//
//	The streambuf's end-of-get-pointer variable.
//--

{
    return m_pchEgptr;
}

inline char *streambuf::epptr(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the byte after the last byte
//	of the put area.
//
//  RETURN VALUE:
//
//	The streambuf's end-of-put-pointer variable.
//--

{
    return m_pchEpptr;
}

inline void streambuf::__gbump(
    int n  // Value to be added to the get pointer.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function increments gptr() by n, which may be positive or negative.
//	No checks are made on whether the new value of gptr() is in bounds.
//
//  SIDE EFFECTS:

//
//	The value of the get pointer is changed.
//--

{
    m_pchGptr += n;
}

inline char *streambuf::gptr(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the first byte of the get
//	area.  The available characters are those between gptr() and egptr().
//	Thus, the next character fetched will be *gptr(), unless egptr() is less
//	than or equal to gptr().
//
//  RETURN VALUE:
//
//	The streambuf's get-pointer variable.
//--

{
    return m_pchGptr;
}

inline char *streambuf::pbase(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the base of the put area.
//	Characters between pbase() and pptr() have been stored into the buffer
//	and not yet consumed.
//
//  RETURN VALUE:
//
//	The streambuf's put-area-base variable.
//--

{
    return m_pchPbase;
}

inline void streambuf::__pbump(
    int n  // Value to be added to the put pointer.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function increments pptr() by n, which may be positive or negative.
//	No checks are made on whether the new value of pptr() is in bounds.
//
//  SIDE EFFECTS:
//
//	The put pointer is changed.
//--

{
    m_pchPptr += n;
}

inline char *streambuf::pptr(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get a pointer to the first byte of the put
//	area.  The space between pptr() and epptr() is the put area.
//
//  RETURN VALUE:
//
//	The streambuf's put-pointer variable.
//--

{
    return m_pchPptr;
}

inline int streambuf::unbuffered(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This function is called to get the current buffering state flag, which
//	is independent of the actual allocation of a reserve area, and its
//	primary purpose is to control whether a reserve area is allocated
//	automatically by allocate().  If the flag is set to non-zero, the
//	automatic allocation is disabled; otherwise, it is enabled.
//
//  RETURN VALUE:
//
//	The streambuf's unbuffered variable.
//--

{
    return m_bIsOkToAllocate;
}

#if defined(__VMS) && (__VMS_VER < 70000000) 
    inline char	*streambuf::ebuf() {return m_pchBase + m_cchBuffer;}
    inline void  streambuf::gbump(int n) {__gbump(n);}
    inline void  streambuf::pbump(int n) {__pbump(n);}
    inline void  streambuf::unbuffered(int n) {m_bIsOkToAllocate = (n != 0);}
#endif

#if defined (__VMS)
inline int streambuf::fd(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the file descriptor associated with the streambuf.
//
//  RETURN VALUE:
//
//	The file descriptor associated with the streambuf.
//--

{
    return m_fd;
}
#endif

#if defined(__VMS)
inline FILE *streambuf::fp(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the file pointer associated with the streambuf.
//
//  RETURN VALUE:
//
//	The file pointer associated with the streambuf.
//--

{

    return m_pfile;
}
#endif

#if defined(__VMS) && (__VMS_VER < 70000000)
inline void streambuf::fd(int f) {
    m_fd = f;
    m_pfile = 0;
}
inline void streambuf::fp(FILE *f) {
    m_pfile = f;
    m_fd = -1;
}
#endif //if defined(__VMS) && (__VMS_VER < 70000000)

#if defined(__VMS) && (__VMS_VER < 70000000)
inline int streambuf::in_avail() {return __in_avail();}
inline int streambuf::out_waiting() {return __out_waiting();}
inline int streambuf::sbumpc() {return __sbumpc();}
inline int streambuf::sgetc() {return __sgetc();}
inline int streambuf::snextc() {return __snextc();}
#endif //if defined(__VMS) && (__VMS_VER < 70000000) 

inline int streambuf::__in_avail(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the number of characters that are immediately available in the
//	get area for fetching.  It is the number of characters that may be
//	fetched with a guarantee that no errors will be reported.
//
//  RETURN VALUE:
//
//	Number of characters that are immediately available in the get area.
//--

{
    return m_pchGptr < m_pchEgptr ? m_pchEgptr - m_pchGptr : 0;
}

inline int streambuf::__out_waiting(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the number of characters in the put area that have not been
//	consumed (by the ultimate consumer).
//
//  RETURN VALUE:
//
//	Number of characters in the put area that have not yet been consumed.
//--

{
    return (m_pchPbase && m_pchPptr) ? m_pchPptr - m_pchPbase : 0;
}

inline int streambuf::__sbumpc(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Moves the get pointer forward one character, and returns the character
//	it moved past.  Returns EOF if the get pointer is currently at the end
//	of the sequence.
//
//  RETURN VALUE:
//
//	The character pointed to by the original get pointer.
//--

{
    if (m_pchGptr >= m_pchEgptr)
        if (underflow() == EOF)
	    return EOF;
    return zapeof(*m_pchGptr++);
}

inline int streambuf::__sgetc(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the character after the get pointer.  Contrary to what most
//	people expect, sgetc does not move the get pointer.  It returns EOF if
//	there is no character available.
//
//  RETURN VALUE:
//
//	The character currently pointed to by the get pointer.
//--

{
    if (m_pchGptr >= m_pchEgptr)
        if (underflow() == EOF)
	    return EOF;
    return zapeof(*m_pchGptr);
}

inline int streambuf::__snextc(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Moves the get pointer forward one character and returns the character
//	following the new position.  It returns EOF if the pointer is currently
//	at the end of the sequence, or is at the end of sequence after moving
//	forward.
//
//  RETURN VALUE:
//
//	The character pointed to by the new get pointer.
//--

{
    if (++m_pchGptr >= m_pchEgptr)
        return snextcUnderflow();
    else
        return zapeof(*m_pchGptr);
}

inline char *streambuf::__GetDataMaxPtr(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the buffer pointer.  Used only for class strstream.
//
//  RETURN VALUE:
//
//	The address of the buffer.
//--

{
#if defined(__VMS)
    return m_pchDataMax;
#else
    return m_pchBase + m_cchDataMax;
#endif
}

#ifdef __VMS
inline char *streambuf::GetDataMaxPtr(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Returns the buffer pointer.  Used only for class strstream.
//
//  RETURN VALUE:
//
//	The address of the buffer.
//--

{
    return __GetDataMaxPtr();
}
#endif

inline void streambuf::__SetDataMaxPtr(
    char *b	// The new buffer pointer associated with the streambuf.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Sets the buffer pointer.  Used only for class strstream.
//
//  RETURN VALUE:
//
//	None.
//--

{
#if defined(__VMS)
    m_pchDataMax = b;
#else
    m_cchDataMax = b - m_pchBase;
#endif
}

#ifdef __VMS
inline void streambuf::SetDataMaxPtr(
    char *b	// The new buffer pointer associated with the streambuf.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Sets the buffer pointer.  Used only for class strstream.
//
//  RETURN VALUE:
//
//	None.
//--

{
    __SetDataMaxPtr(b);
}
#endif

inline int istream::gcount(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//      Returns the number of characters extracted by the last unformatted input
//      function (formatted input functions may call unformatted input functions
//      and thereby reset this number).
//
//  RETURN VALUE:
//
//	Number of characters extracted by the last unformatted input operation.
//--

{
    return m_cchGet;
}

inline istream &istream::get(
    unsigned char &c  // Reference to the unsigned char being read into.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like get(char &c), except for the different type of its first
//	argument.
//
//  RETURN VALUE:
//
//	Reference to the istream being acted upon.
//--

{
    return get((char &)c);
}

inline istream &istream::read(
    unsigned char *s,  // Pointer to the unsigned char string being read into.
    int           n    // The number of characters to be read.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like read(char *s, int n), except for the different type of its
//	first argument.
//
//  RETURN VALUE:
//
//	Reference to the istream being acted upon.
//--

{
    return read((char *) s, n);
}

inline istream &istream::operator>>(
    istream &(*f)(istream &)  // The function to be used as a manipulator.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	A direct extraction from one istream to another, with no other
//	processing invoved.
//
//  RETURN VALUE:
//
//	Reference to the istream being acted upon.
//--

{
    return (*f)(*this);
}

inline ostream &ostream::write(
    const unsigned char *ptr,  // Pointer to the char string to be written to.
    int                 n      // The number of characters to be written.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like write(const char *ptr, int n), except for the different type
//	of its first argument.
//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return write((const char *)ptr, n);
}

inline ostream &ostream::operator<<(
    short i  // The variable whose value is to be output.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	For general information on formatted inserters, outs << x, see comments
//	about operator<<(const char *str).
//
//	If x is of any integral type except char and unsigned char, then the
//	following applies.  If x is positive, the representation contains a
//	sequence of decimal, octal, or hexadecimal digits with no leading zeros
//	according to whether ios::dec, ios::oct, or ios::hex, respectively, is
//	set in ios's format flags.  If none of those flags are set, conversion
//	defaults to decimal.  If x is negative, decimal conversion includes a
//	minus sign (-) followed by decimal digits.  If x is positive and
//	ios::showpos is set, decimal conversion includes a plus sign (+)
//	followed by decimal digits.  The other conversions treat all values as
//	unsigned.  If ios::showbase is set, the hexadecimal representation
//	contains 0x before the hexadecimal digits, or 0X if ios::uppercase is
//	set.  If ios::showbase is set, the octal representation contains a
//	leading 0.
//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return *this << (long)i;
}

inline ostream &ostream::operator<<(
    unsigned char c  // The variable whose value is to be output.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like operator<<(char), except for the different type of its
//	argument.
//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return *this << (char)c;
}

inline ostream &ostream::operator<<(
    unsigned short i  // The variable whose value is to be output.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like operator<<(short), except for the different type of its
//	argument.
//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return *this << (unsigned long)i;
}

inline ostream &ostream::operator<<(
    ostream &(*f)(ostream &)  // The function to be inserted into the ostream.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	"outs << manip" is equivalent to "manip(outs)", where outs is an
//	outstream and manip is a function with the type ostream &(*)(ostream &).
//	Syntactically, this looks like an insertion operation, but semantically,
//	it does an arbitrary operation rather than converting manip to a
//	sequence of characters as do the insertion operators.
//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return (*f)(*this);
}

#if !__X_FLOAT || defined(__VMS) 
inline ostream &ostream::operator<<(
    long double d  // The variable whose value is to be output.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like operator<<(double).

//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return *this << (double)d;
}
#endif
#if !defined(__VMS) && defined(__DECCXX) && !(defined(__STD_STRICT_ANSI) || defined(__STD_STRICT_ANSI_ERRORS)) && !defined(__CXXLFIX634)

inline ostream &ostream::operator<<(
    long long d  // The variable whose value is to be output.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like operator<<(long).

//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return *this << (long)d;
}

inline ostream &ostream::operator<<(
    unsigned long long d  // The variable whose value is to be output.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like operator<<(long).

//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return *this << (long)d;
}
#endif

#if defined(__DECCXX) && defined(__DECCXX_VER) && (__DECCXX_VER >= 60000000) && defined(__BOOL_IS_A_RESERVED_WORD)
inline ostream &ostream::operator<<(
    bool b  // The variable whose value is to be output.
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Just like operator<<(int).
//
//  RETURN VALUE:
//
//	Reference to the ostream being acted upon.
//--

{
    return *this << (int)b;
}
#endif

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif // __USE_STD_IOSTREAM

#endif

// End of iostream.hxx
