#ifndef		__MESSAGES_HXX
#define		__MESSAGES_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	Internationalizable messages
//
//  MODULE:	messages
//
//  INFORMATION HIDDEN:
//
//	Data structure required to represent a set of messages on any supported
//	platform.
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of objects of the Messages
//	class, which is an abstraction of an internationalizable message file
//	or message catalog.
//
//  DESIGN ISSUES:
//
//	Some data (the filename and the default location of the message catalog)
//	are only used on ULTRIX, although for portability values are required on
//	VMS too.  (NULL is an adequate value for these.)
//
//	The message set number is originally from ULTRIX, but is used on VMS as
//	a "base" message id to which the message number (a small integer) is
//	added (after shifting it over the severity bits).
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
#include <stdio.h>
#include <limits.h>
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//
//
//  STATIC DATA
//
//

extern int cxxl_messages_print_cerr(const char *msg_arg);
extern int cxxl_messages_print_cerr_clog(const char *msg_arg);

class Messages

//++
//  CLASS DESCRIPTION:
//
//	Messages is an abstraction of a message file, which associates a message
//	number with a text string.
//
//  DESIGN:
//
//	cxxl_filename: Points to a copy of the message file name string that was
//	    passed to the constructor.
//	    NULL: ok if a platform (e.g. VMS) doesn't open a file at runtime.
//		(However, a value must explicitly be given.)
//
//	cxxl_default_file_location: Points to a copy of the default message file
//	    pathname (a valid value for the NLSPATH environment variable) string
//	    that was passed to the constructor.
//	    NULL: ok if a platform (e.g. VMS) doesn't need open a file at
//		runtime.  If the platform does (e.g. ULTRIX), and this is NULL,
//		then there is one less place to check (e.g. NLSPATH had better
//		point to where the file is).
//
//	cxxl_set: The default message set number passed to the constructor.
//	    0: None was provided; a default set or a set passed to the text
//		function is to be used.
//
//	cxxl_buffer: Pointer to dynamically-allocated message buffer
//	    NULL: ok if a platform (e.g. VMS) doesn't need to explicitly
//		allocate a message buffer
//
//--

{
private:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    char *cxxl_filename;		// Copy of message file name
    int cxxl_set;			// Message set number (0 = default set)
    char *cxxl_default_file_location;	// Copy of default message file location
    char *cxxl_buffer;			// Message text buffer
    int cxxl_buffer_len;		// Message text buffer length (in chars)
#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

public:
    Messages(const char *filename_arg, int set_arg = 0,
	const char *default_file_location_arg = (const char *)(NULL));
    ~Messages();

    const char *text(int msg_arg, const char *fallback_text_arg,
	int set_arg = 0);
};

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif


#endif
// End of messages.hxx
