#ifndef __STDIOSTREAM_HXX
#define __STDIOSTREAM_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//==============================================================
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	The iostream package.
//
//  MODULE:	Include file for the stdiostream subpackage.
//
//  MODULE DESCRIPTION:
//
//      The stdiostream subpackage of the iostream package performs I/O
//	operations on stdio FILEs.  It relies on the iostream (core) subpackage
//	for base classes to derive classes that are specialized for stdio FILEs.
//
//  DESIGN ISSUES:
//
//      The description of this subpackage in AT&T's documentation is virtually
//      nonexistent.  Also, it wasn't always possible to experimentally
//      determine the compatible behavior because of the problems in the version
//      of cfront (Glockenspiel) we were using.  On the other hand, AT&T's
//      documentation does show the stdiostream class as being directly derived
//      from class ios.  This is far insufficient for any useful applications,
//      and that's why expand the functionality provided in class stdiostream by
//      deriving it from iostream rather than ios.
//
//--

//++
//  INCLUDE FILES
//--

#include <compnent.hxx>
#ifdef __USE_STD_IOSTREAM 
#error "cannot include stdiostream.hxx -- do not compile strict_ansi OR define __NO_USE_STD_IOSTREAM - see section 7.1.2 of the C++ Using Guide"
#else

#include <cxxl.hxx>
#include <iostream.hxx>
extern "C"
{
#include <stdio.h>
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//++
//  LINKAGE
//--

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model  __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//==============================================================
//  Class: stdiobuf
//
//  FUNCTIONAL DESCRIPTION:
//
//	This class specializes the streambuf class for stdio FILEs.  It uses
//	unbuffered mode, causing all operations to be reflected immediately in
//	the FILE.
//--------------------------------------------------------------
class stdiobuf : public streambuf
{
public:
			stdiobuf(FILE *f);
    virtual int		overflow(int = EOF);
    virtual streampos	seekoff(streamoff, __SCOPE(ios::)seek_dir, int mode);
    virtual int		sync();
    virtual int		underflow();
#if ! defined(__VMS)	// OpenVMS does not allocate space for the following
    virtual int         pbackfail(int);
			~stdiobuf();
private:
    FILE*               m_pfile;
    int			__fill1;
    char                m_achBuffer[2];
    char		__fill2[2];
#endif
public:
    FILE*		stdiofile()
#if defined(__VMS)	// OpenVMS implementation is not inline
			;
#else
			{return m_pfile; }
#endif
};


//==============================================================
//  Class: stdiostream
//
//  FUNCTIONAL DESCRIPTION:
//
//	This class specializes the iostream class for stdio FILEs and to use an
//	stdiobuf as its associated streambuf.
//--------------------------------------------------------------
class stdiostream
#if defined(__osf__)
    : public ios
#else
    : public iostream
#endif
{
public:
			stdiostream(FILE *f);
			~stdiostream();
    stdiobuf*		rdbuf();
#if defined (__VMS)	// OpenVMS defines this public member function
    void		fp(FILE *f);
#elif defined (__osf__) // DEC OSF/1 requires backward compatibility to AT&T
private:
    stdiobuf		m_stdiobuf;
#else
			// placeholder for other implementations' special needs
#endif // defined(__VMS)
};


#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif // __USE_STD_IOSTREAM

#endif
// End of stdiostream.hxx
