#ifndef __STREAM_HXX
#define __STREAM_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	The iostream package.
//
//  MODULE:	Include file for the stream subpackage.
//
//  MODULE DESCRIPTION:
//
//	The stream subpackage of the iostream package provides the functionality
//	that is no longer supported (referred to as anachronisms), for the
//	purpose of backward compatibility with AT&T release 1.2.
//
//  DESIGN ISSUES:
//
//	For inclusion in this subpackage, we have selected these anachronisms
//	that continue appearing in the current literature, and specifically, in
//	the 2nd edition of "The C++ Programming Language" by Bjarne Stroustrup.
//
//	This implementation is not thread-safe, since all threads share the
//	same buffer.
//
//--

//
//  INCLUDE FILES
//

#include <compnent.hxx>
#ifdef __USE_STD_IOSTREAM 
#error "cannot include stream.hxx -- do not compile strict_ansi OR define __NO_USE_STD_IOSTREAM - see section 7.1.2 of the C++ Using Guide"
#else

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
#include <stdarg.h>
}

#include <iomanip.hxx>
#include <iostream.hxx>
#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//  LINKAGE
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model  __relaxed_refdef
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

static const int __cxxl_stream_format_buffer_size = 257;
static char      __cxxl_stream_format_buffer[__cxxl_stream_format_buffer_size];

#ifndef __CXXL_STREAM_CXX
inline 
#endif
char *oct(
    long __value,
    int __width = 0
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Format a number in octal.
//
//  RETURN VALUE:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//--

{
    sprintf(__cxxl_stream_format_buffer, "%*o", __width, __value);
    return __cxxl_stream_format_buffer;
}

#ifndef __CXXL_STREAM_CXX
inline 
#endif
char *dec(
    long __value,
    int __width = 0
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Format a number in decimal.
//
//  RETURN VALUE:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//--

{
    sprintf(__cxxl_stream_format_buffer, "%*d", __width, __value);
    return __cxxl_stream_format_buffer;
}

#ifndef __CXXL_STREAM_CXX
inline 
#endif
char *hex(
    long __value,
    int __width = 0
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Format a number in hexadecimal.
//
//  RETURN VALUE:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//--

{
    sprintf(__cxxl_stream_format_buffer, "%*x", __width, __value);
    return __cxxl_stream_format_buffer;
}

#ifndef __CXXL_STREAM_CXX
inline char *chr(
    long __value,
    int __width = 0
    )
#else
char *chr(
    int __value,
    int __width = 0
    )
#endif

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Format a single character.
//
//  RETURN VALUE:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//--

{
    sprintf(__cxxl_stream_format_buffer, "%*c", __width, __value);
    return __cxxl_stream_format_buffer;
}

#ifndef __CXXL_STREAM_CXX
inline 
#endif
char *str(
    const char *__value,
    int __width = 0
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Format a string.
//
//  RETURN VALUE:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	__cxxl_stream_format_buffer: contains the formatted argument.
//
//  DESIGN:
//
//	The format argument was originally not specified to be const.
//
//--

{
    sprintf(__cxxl_stream_format_buffer, "%*s", __width, __value);
    return __cxxl_stream_format_buffer;
}

#ifndef __CXXL_STREAM_CXX
inline 
#endif
char *form(
    const char *__format ...
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Format arguments as printf does.
//
//  RETURN VALUE:
//
//	__cxxl_stream_format_buffer: contains the formatted arguments.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	__cxxl_stream_format_buffer: contains the formatted arguments.
//
//  DESIGN:
//
//	The format argument was originally not specified to be const.
//
//	This is not type-safe.
//
//--

{
    va_list __ap;
    va_start(__ap, __format);
    vsprintf(__cxxl_stream_format_buffer, __format, __ap);
    va_end(__ap);
    return __cxxl_stream_format_buffer;
}

#if defined(__osf__) && defined(__CXXL_IOSTREAM_CXX)
// Implement as library function for DEC OSF/1 binary compatibility

SMANIP(int) setbase(int);
ios &setbase(ios&, int);

#else
inline SMANIP(int) setbase(int);

inline ios &setbase(
    ios &__s,
    int __base
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This is an anachronistic parameterized manipulator.  It sets the
//	base value for formatted I/O to octal, decimal, or hexadecimal,
//	depending on the value of its explicit argument, 8, 10, or 16,
//	respectively.  It currently ignores invalid argument values.
//
//  RETURN VALUE:
//
//	Reference to the stream being acted upon.
//--

{
    if (__base == 8)
	__s.setf(ios::oct, ios::basefield);
    else if (__base == 10)
	__s.setf(ios::dec, ios::basefield);
    else if (__base == 16)
	__s.setf(ios::hex, ios::basefield);
    return __s;
}

inline SMANIP(int) setbase(int __base)
{
    return SMANIP(int)(setbase, __base);
}
#endif

#if defined(__osf__)
// Static constants for compatibility to previous version of DEC OSF/1

static const int input  = (ios::in);
static const int output = (ios::out);
static const int append = (ios::app);
static const int atend  = (ios::ate);
static const int _good  = (ios::goodbit);
static const int _bad   = (ios::badbit);
static const int _fail  = (ios::failbit);
static const int _eof   = (ios::eofbit);
#endif

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif // __USE_STD_IOSTREAM

#endif
// End of stream.hxx
