/***************************************************************************
 *
 * string.cc - Definitions for the Standard Library string classes
 *
 ***************************************************************************
 *    
 *  Copyright 2000 Compaq Computer Corporation
 *
 *  COMPAQ Registered in U.S. Patent and Trademark Office.
 *
 *  Confidential computer software. Valid license from Compaq required for
 *  possession, use or copying. Consistent with FAR 12.211 and 12.212,
 *  Commercial Computer Software, Computer Software Documentation, and
 *  Technical Data for Commercial Items are licensed to the U.S. Government
 *  under vendor's standard commercial license.
 *
 ****************************************************************************
 *
 * (c) Copyright 1994, 1998 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __save
#      pragma __environment __header_defaults
#   endif
#endif

#include <stdcomp>

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#pragma __extern_prefix __save
#pragma __extern_prefix "CXXL$" 
#endif

//
// Members for class basic_string
//

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif

// basic_string<...>::_RWnullref
#ifndef _RWSTD_NO_STATIC_DEF3
  template <class charT, class traits, class Allocator >
  const _TYPENAME basic_string<charT, traits, Allocator>::_RWnull_ref_type
  basic_string<charT, traits, Allocator>::_RWnullref;
#endif


// basic_string<...>::npos
#ifdef _RWSTD_MSC22_STATIC_INIT_BUG
#define npos (size_type)-1
#else
  template <class charT, class traits, class Allocator >
  const _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::npos
#ifdef _RWSTD_NO_STATIC_CAST
  = (_TYPENAME basic_string<charT, traits, Allocator>::size_type) -1;
#else
  = static_cast<_TYPENAME basic_string<charT, traits, Allocator>::size_type>(-1);
#endif

#endif /*_RWSTD_MSC22_STATIC_INIT_BUG*/

  template <class charT, class traits, class Allocator >
  _TYPENAME basic_string<charT, traits, Allocator>::_RWstring_ref_type *
  basic_string<charT, traits, Allocator>::_RWgetRep (size_type capac, 
                                                    size_type nchar)
                                              
  {                                                     
    if (capac < nchar)
       capac = nchar;

    if ((capac | nchar) == 0)
    {
      _RWgetNullRep()->_RWaddReference();
      return _RWgetNullRep();
    }

    //
    // Allocate, then initialize the _RWstring_ref, then
    // initialize each character in the buffer. 
    //
    _RWvalue_alloc_type va(_RWdata_);
#ifdef _RWSTD_NO_STATIC_CAST
    _RWstring_ref_type * ret = (_RWstring_ref_type *)
    va.allocate(capac + sizeof(_RWrep_type)/sizeof(charT) + 2,0);
#else
    _RWstring_ref_type * ret = 
    reinterpret_cast<_RWstring_ref_type*>
    (va.allocate(capac + sizeof(_RWrep_type)/sizeof(charT) + 2,0));
#endif

    _RWref_alloc_type(_RWdata_).construct(ret, _RWstring_ref_type());
    charT * d = ret->data();

#if defined(__DECCXX) && !defined(__DECFIXCXXL1082)
    new (d) charT(charT());
#else
    va.construct(d,charT());
#endif

    ret->_RWcapacity_ = capac;
    ret->_RWsetRefCount(1);
    ret->data()[ret->_RWnchars_ = nchar] = (charT)0; // Terminating null
    return ret;
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator>::basic_string (
      const basic_string<charT, traits, Allocator> & s,
      size_type pos, size_type  n,
      const Allocator& alloc)
    : _RWdata_(0,alloc)
  {
   _RWSTD_THROW(pos > s.length(), out_of_range,
    __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
       "basic_string( const basic_string&,size_type,size_type)", pos,s.length()).msgstr());

    size_type slen = s.length() - pos;
    size_type rlen = n < slen ? n : slen; 
    size_type nlen =  n == npos ? 0 : n;
    size_type maxlen = nlen > rlen ? nlen : rlen; 
    _RWdata_            = _RWgetRep(maxlen, rlen)->data();

    traits::copy(_RWdata_.data(), &s._RWdata_.data()[pos], rlen);
  }

  template <class charT, class traits, class Allocator >
#if defined(__DECCXX) && !defined(__DECFIXCXXL957)
  void basic_string<charT, traits, Allocator>::_RWinitn (
#else
  void basic_string<charT, traits, Allocator>::_RWinit_n (
#endif
      size_type        n,
      charT            c)
  {
#if defined(__DECCXX) && !defined(__DECFIXCXXL957)
   _RWSTD_THROW(n == npos, length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_InvalidSizeParam,
        "basic_string::_RWinitn(size_t,char)", n,npos).msgstr());
#else
   _RWSTD_THROW(n == npos, length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_InvalidSizeParam,
        "basic_string::_RWinit_n(size_t,char)", n,npos).msgstr());
#endif

    _RWdata_  = _RWgetRep(n, n)->data();

    while (n--) traits::assign(_RWdata_.data()[n], c);
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator>::basic_string (
      const charT*     s,
      size_type        n,
      const Allocator& alloc)
  : _RWdata_(0,alloc)
  {     
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
       "basic_string( const charT*,size_type,const Allocator&)").msgstr());

    _RWSTD_THROW(n == npos, length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_InvalidSizeParam,
        "basic_string( const charT*,size_type,const Allocator&)",n,npos).msgstr());

    _RWdata_  = _RWgetRep(n, n)->data();
    traits::copy(_RWdata_.data(), s, n);
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator>::basic_string (
      const charT*     s,
      const Allocator& alloc)
  : _RWdata_(0,alloc)
  {     
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
        "basic_string(const charT*,const Allocator&)").msgstr());

    size_type len = traits::length(s);
    _RWdata_         = _RWgetRep(len, len)->data();

    traits::copy(_RWdata_.data(), s, len);
  }

#ifdef _RWSTD_NO_DEFAULT_TEMPLATE_ARGS
  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator>::basic_string (
      const charT*     s,
      size_type        n)
  : _RWdata_(0,Allocator())
  {     
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
         "basic_string( const charT*,size_type)").msgstr());

    _RWSTD_THROW(n == npos,  length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_InvalidSizeParam,
        "basic_string(const charT*,size_type)",n,npos).msgstr());

    _RWdata_  = _RWgetRep(n, n)->data();

    traits::copy(_RWdata_.data(), s, n);
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator>::basic_string (
      const charT*     s)
  : _RWdata_(0,Allocator())
  {     
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
        "basic_string(const charT*)").msgstr());

    size_type len = traits::length(s);
    _RWdata_         = _RWgetRep(len, len)->data();

    traits::copy(_RWdata_.data(), s, len);
  }
#endif

#ifndef _RWSTD_NO_MEMBER_TEMPLATES
  template <class charT, class traits , class Allocator >
  template <class InputIterator>
  basic_string<charT, traits, Allocator>::basic_string (InputIterator first,
                                                        InputIterator last,
                                                        const Allocator &alloc)
  : _RWdata_(0,alloc)
  {
    _RWdata_ = _RWgetRep(1,0)->data();
    replace(_RWdata_.data(),_RWdata_.data(),first,last);
  }
#endif /* _RWSTD_NO_MEMBER_TEMPLATES */

  template <class charT, class traits , class Allocator  >
  basic_string<charT, traits, Allocator>::basic_string (const charT* first,
                                                        const charT* last,
                                                        const Allocator& alloc)
  : _RWdata_(0,alloc)
  {
    _RWdata_ = _RWgetRep(last-first,0)->data();
    replace(0,0,first,last-first,0,last-first);
  }

#ifdef _RWSTD_NO_DEFAULT_TEMPLATE_ARGS
  template <class charT, class traits , class Allocator  >
  basic_string<charT, traits, Allocator>::basic_string (const charT* first,
                                                        const charT* last)
  : _RWdata_(0,Allocator())
  {
    _RWdata_ = _RWgetRep(last-first,0)->data();
    replace(0,0,first,last-first,0,last-first);
  }
#endif /* _RWSTD_NO_DEFAULT_TEMPLATE_ARGS */


  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::operator= (
      const basic_string<charT, traits, Allocator>& str)
  {
    if (this != &str)
    {
      if (str._RWpref()->_RWreferences() > 0)
      {
        str._RWpref()->_RWaddReference();
        _RWunLink();
        _RWdata_ = str._RWdata_.data();
      }
      else
#if defined(__DECCXX) && !defined(__DECFIXCXXL1238)
        this->replace(0, length(), str.data(), str.length());
#else
        this->operator=(str.c_str());
#endif
    }
    return *this;
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::operator= (const charT* s)
  {
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
         "basic_string::operator= (const charT*)").msgstr());

    if (traits::eq(*s, _RWeos()))
    {
      if (_RWpref()->_RWreferences() == 1)
      {
        _RWpref()->_RWnchars_ = 0;
        traits::assign(_RWdata_.data()[0], _RWeos());
      }
      else
      {
        _RWunLink();
        _RWgetNullRep()->_RWaddReference();
        _RWdata_ = _RWgetNullRep()->data();
      }
      return *this;
    }

    return replace(0, length(), s, traits::length(s));
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::append (
      const basic_string<charT, traits, Allocator>& str,
      size_type                                     pos,
      size_type                                     n)
  {
     _RWSTD_THROW(pos > str.length(), out_of_range,
      __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
        "basic_string::append(basic_string&,size_type,size_type)", pos,str.length()).msgstr());

    size_type slen = str.length() - pos;
    size_type rlen = n < slen ? n : slen; 

    _RWSTD_THROW(length() >= npos - rlen, length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
          "basic_string::append(const basic_string&,size_type,size_type)",
            length(),npos-rlen).msgstr());

    replace(length(), 0, str.data(), str.length(), pos, n);

    return *this;
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::append (
      const basic_string<charT, traits, Allocator>& str)
  {
    _RWSTD_THROW(length() >= npos - str.length(), length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
          "basic_string::append(const basic_string&)",length(),str.length()).msgstr());

    replace(length(), 0, str.data(), str.length());

    return *this;
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::assign (
      const basic_string<charT, traits, Allocator>& str,
      size_type                                     pos,
      size_type                                     n)
  {
    _RWSTD_THROW(pos > str.length(), out_of_range,
      __RWSTD::except_msg_string(__RWSTD::_RWrwse_PosBeyondEndOfString,
        "basic_string::assign(basic_string&,size_type,size_type)", pos,str.length()).msgstr());

    size_type slen = str.length() - pos;
    size_type rlen = n < slen ? n : slen; 
    return replace(0, length(), str, pos, rlen);
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::assign (
      const basic_string<charT, traits, Allocator>& str)
  {
    return replace(0, length(), str);
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::insert (
      size_type                                     pos1,
      const basic_string<charT, traits, Allocator>& str,
      size_type                                     pos2,
      size_type                                     n)
  {
    _RWSTD_THROW(pos1 > length() || pos2 > str.length(), out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
        "basic_string::insert(size_t,const basic_string&,size_t,size_t)",
           pos1 > length() ? pos1:pos2,str.length()).msgstr());

    size_type slen = str.length() - pos2;
    size_type rlen = n < slen ? n : slen; 

    _RWSTD_THROW( length() >= npos - rlen, length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
        "basic_string::insert(size_t,const basic_string&,size_t,size_t)",
            length(),npos - rlen).msgstr());

    return replace(pos1, 0, str, pos2, n);
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::insert (
      size_type                                     pos1,
      const basic_string<charT, traits, Allocator>& str)
  {
    _RWSTD_THROW(pos1 > length(), out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
        "basic_string::insert(size_t,const basic_string&)",pos1,length()).msgstr());

    _RWSTD_THROW(length() >= npos - str.length(),  length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
        "basic_string::insert(size_t,const basic_string&)",
          length(), npos - str.length()).msgstr());

    return replace(pos1, 0, str);
  }

  template <class charT, class traits, class Allocator >
  _TYPENAME basic_string<charT, traits, Allocator>::iterator 
  basic_string<charT, traits, Allocator>::replace (
      size_type    pos1,
      size_type    n1,
      const charT* cs,
      size_type    cslen,
      size_type    pos2,
      size_type    n2)
  {
     _RWSTD_THROW(pos1 > length() || pos2 > cslen, out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
        "basic_string::replace(size_t,size_t,char*,size_t,size_t,size_t)",
           pos1 > length() ? pos1:pos2,length() > cslen ? length():cslen).msgstr());

    size_type slen = length() - pos1;
    size_type xlen = n1 < slen ? n1 : slen; 
    size_type clen = cslen - pos2;
    size_type rlen = n2 < clen ? n2 : clen; 

    _RWSTD_THROW(length() - xlen >= npos - rlen, length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
        "basic_string::replace(size_t,size_t,char*,size_t,size_t,size_t)",
          length()-xlen,npos - rlen).msgstr());

    size_type tot = length() - xlen + rlen;  // Final string length.

    if (!tot)
    {
      // Special case a substitution that leaves the string empty.
      _RWunLink();
      _RWgetNullRep()->_RWaddReference();
      _RWdata_ = _RWgetNullRep()->data();
    }
    else
    {
      size_type rem = length() - xlen - pos1;  // Length of bit at end of string
      // Check for shared representation, insufficient capacity, 
      //  or overlap copy.
      if ( (_RWpref()->_RWreferences() > 1)
           || (_RWgetCapac() < tot)
           || (cs && (cs >= data() && cs < data()+length())))
      {
        // Need to allocate a new reference.
        _RWstring_ref_type * temp = _RWgetRep(tot,tot);
        if (pos1) traits::copy(temp->data(), _RWdata_.data(), pos1);
        if (rlen) traits::copy(temp->data()+pos1, cs+pos2, rlen);
        if (rem ) traits::copy(temp->data()+pos1+rlen, _RWdata_.data()+pos1+n1, rem);
        _RWunLink();
        _RWdata_ = temp->data();
      }
      else
      {
        // Current reference has enough room.
        if (rem)  traits::move(_RWdata_.data()+pos1+rlen, _RWdata_.data()+pos1+n1, rem);
        if (rlen) traits::move(_RWdata_.data()+pos1, cs+pos2, rlen);
        _RWdata_.data()[_RWpref()->_RWnchars_ = tot] = _RWeos();   // Add terminating null
      }
    }
    return _RWdata_.data() + pos1;
  }

  template <class charT, class traits , class Allocator  >
  basic_string<charT, traits, Allocator> &
  basic_string<charT, traits, Allocator>::replace (size_type pos,
                                                   size_type n,
                                                   size_type n2,
                                                   charT c)
  {
    _RWSTD_THROW(pos > length(), out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
        "basic_string:::replace(size_t,size_t,size_t,char)", pos,length()).msgstr());

    size_type slen = length() - pos;
    size_type xlen = n < slen ? n : slen; 

    _RWSTD_THROW(length() - xlen >= npos - n2, length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
          "basic_string::replace(size_t,size_t,size_t,char)",
            length()-xlen,npos - n2).msgstr());

    size_type tot = length() - xlen + n2;  // Final string length.

    if (!tot)
    {
      // Special case a substitution that leaves the string empty.
      _RWunLink();
      _RWgetNullRep()->_RWaddReference();
      _RWdata_ = _RWgetNullRep()->data();
    }
    else
    {
      size_type rem = length() - xlen - pos; // Length of bit at end of string
      // Check for shared representation, insufficient capacity, 
      if ( (_RWpref()->_RWreferences() > 1) || (_RWgetCapac() < tot))
      {
        // Need to allocate a new reference.
        _RWstring_ref_type * temp = _RWgetRep(tot,tot);
        if (pos) traits::copy(temp->data(), _RWdata_.data(), pos);
        if (n2) traits::assign(temp->data()+pos, n2,c);
        if (rem ) traits::copy(temp->data()+pos+n2, _RWdata_.data()+pos+n, rem);
        _RWunLink();
        _RWdata_ = temp->data();
      }
      else
      {
        // Current reference has enough room.
        if (rem)  traits::move(_RWdata_.data()+pos+n2, _RWdata_.data()+pos+n, rem);
        if (n2) traits::assign(_RWdata_.data()+pos, n2, c);
        _RWdata_.data()[_RWpref()->_RWnchars_ = tot] = _RWeos();   // Add terminating null
      }
    }
    return *this;
  }

#ifndef _RWSTD_NO_MEMBER_TEMPLATES

  // Since we have no way of knowing how much is out there
  // between first2 and last2 we have to allocate some space
  // in chunks as needed and copy over characters in groups
  // until we reach the end of the range.  After that 
  // we can tidy things up.  We avoid this using replace_aux
  // any time we have an InputIterator capable of giving us
  // the distance information we need.  If InputIterator
  // is really a charT* then we should never even get here.
  template<class charT, class traits , class Allocator >
  template<class InputIterator>
  inline basic_string<charT, traits, Allocator>& 
  basic_string<charT, traits, Allocator>::replace (
      iterator first1, 
      iterator last1,
      InputIterator first2,
      InputIterator last2)
  {
     iterator first = first1;
     iterator last = last1;

     // Use a (probably)faster algorithm if possible
     if (_RWis_bidirectional_iterator(_RWiterator_category(first2)))
       return _RWreplace_aux(first1,last1,first2,last2);

     _RWSTD_THROW((first > end() || last > end()),out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
          "basic_string::replace(iterator,iterator,InputIterato,InputIterato)"
           ,first > end() ? first : last,end()).msgstr());

     while (first2 != last2)
     {
     _RWSTD_THROW(first  - _RWdata_.data() == npos,length_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
          "basic_string::replace(iterator,iterator,InputIterator,InputIterator)"
           ,first-_RWdata_.data(),npos).msgstr());


       if (first == last)
       {
         // Need to allocate a more room.
         // First get the amount to increase buffer size         
         size_type delta =  __RWSTD::_RWrw_allocation_size((value_type*)0,
                                                        (size_type)0,
                                                        (size_type)0);
         delta = delta > 0 ? delta : 1;
         size_type tot = delta + size();
         // Now get new buffer
         _RWstring_ref_type * temp = _RWgetRep(tot,tot);
         // Now copy data from old to new, leaving a hole for additions
         size_type current = last-begin();
         traits::copy(temp->data(),_RWdata_.data(),current);
         traits::copy(temp->data()+current+delta,last,end()-last);
         _RWunLink();
         _RWdata_ = temp->data();
         first = _RWdata_.data() + current;
         last = first + delta;
       }
       // Copy data over
       *first++ = *first2++;
     }    
     if (first != last)
       replace(first-_RWdata_.data(),last-first,0,(charT)' ');   // Clean up  

     return *this;
  }

  // Special function for random access and bi-directional iterators
  // Avoids the possibility of multiple allocations
  // We still have to copy characters over one at a time.
  template<class charT, class traits , class Allocator >
  template<class InputIterator>
  inline basic_string<charT, traits, Allocator>& 
  basic_string<charT, traits, Allocator>::_RWreplace_aux (
      iterator first1, 
      iterator last1,
      InputIterator first2,
      InputIterator last2)
  {
    difference_type n2 = 0;
    distance(first2,last2,n2);
    size_type n = last1-first1;
    size_type pos = first1 - _RWdata_.data();

    _RWSTD_THROW(pos > length(),out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_StringIndexOutOfRange,
          "basic_string::_RWreplace_aux(iterator,iterator,InputIterator,InputIterator)"
           ,pos,length()).msgstr());

    size_type slen = length() - pos;
    size_type xlen = n < slen ? n : slen; 

   _RWSTD_THROW(length() - xlen >= npos - n2,length_error,
    __RWSTD::except_msg_string(__RWSTD::_RWrwse_ResultLenInvalid,
         "basic_string::_RWreplace_aux(iterator,iterator,InputIterator,InputIterator)"
           ,length() - xlen, npos - n2).msgstr());

    size_type tot = length() - xlen + n2;  // Final string length.

    if (!tot)
    {
      // Special case a substitution that leaves the string empty.
      _RWunLink();
      _RWgetNullRep()->_RWaddReference();
      _RWdata_ = _RWgetNullRep()->data();
    }

    else
    {
      size_type d = 0;
      size_type rem = length() - xlen - pos; // Length of bit at end of string
      // Check for shared representation, insufficient capacity, 
      if ( (_RWpref()->_RWreferences() > 1) || (_RWgetCapac() < tot))
      {
        // Need to allocate a new reference.
        _RWstring_ref_type * temp = _RWgetRep(tot,tot);
        if (pos) traits::copy(temp->data(), _RWdata_.data(), pos);
        for (d = 0; d < n2; d++)
          *(temp->data()+pos+d) = *first2++;
        if (rem ) 
          traits::copy(temp->data()+pos+n2, _RWdata_.data()+pos+n, rem);
        _RWunLink();
        _RWdata_ = temp->data();
      }
      else
      {
        // Current reference has enough room.
        if (rem)  
          traits::move(_RWdata_.data()+pos+n2, _RWdata_.data()+pos+n, rem);
        for (d = 0; d < n2; d++)
          *(_RWdata_.data()+pos+d) = *first2++;
        _RWdata_.data()[_RWpref()->_RWnchars_ = tot] = _RWeos();   // Add terminating null
      }
    }
    return *this;
  }
#endif // _RWSTD_NO_MEMBER_TEMPLATES

  template <class charT, class traits, class Allocator >
  _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::copy (charT* s,
                                                size_type n,
                                                size_type pos) const
  {
    _RWSTD_THROW(pos > length(), out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_PosBeyondEndOfString,
          "basic_string::copy(char*,size_t,size_t)",pos,length()).msgstr());

    size_type slen = length() - pos;
    size_type rlen = n < slen ? n : slen;
    traits::copy(s, _RWdata_.data()+pos, rlen);
    return rlen;
  }


  template <class charT, class traits , class Allocator  >
  _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::find (const charT* s,
                                                size_type pos,
                                                size_type n) const
  {
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
          "basic_string::find(char*,size_t,size_t) const").msgstr());

    for (size_type xpos = pos; (xpos + n) <= length() ; xpos++)
    {
      if (!traits::compare(_RWdata_.data()+xpos,s,n))
        return xpos;
    }

    return npos;
  }

  template <class charT, class traits , class Allocator  >
  _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::rfind (const charT* s,
                                                 size_type pos,
                                                 size_type n) const
  {

    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
          "basic_string::rfind(char*,size_t,size_t) const").msgstr());

    if (length() < n)
      return npos;
    
    size_type slen = length() -n;
    size_type xpos_start = slen < pos ? slen : pos; 

    for (size_type xpos = xpos_start+1; xpos != 0 ; xpos--)
    {
      if (!traits::compare(_RWdata_.data()+xpos-1,s,n))
        return xpos-1;
    }

    return npos;
  }

  template <class charT, class traits , class Allocator  >
  _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::find_first_of (const charT* s,
                                                         size_type pos,
                                                         size_type n) const
  {

    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
          "basic_string::find_first_of(char*,size_t,size_t) const").msgstr());

    for (size_type xpos = pos; xpos < length() ; xpos++)
    {
      for (size_type i = 0; i < n ; i++)
        if (traits::eq(_RWdata_.data()[xpos], s[i]))
          return xpos;
    }

    return npos;
  }


  template <class charT, class traits , class Allocator  >
  _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::find_last_of (const charT* s,
                                                        size_type pos,
                                                        size_type n) const
  {
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
          "basic_string::find_last_of(char*,size_t,size_t) const").msgstr());

    if (length())
    {
      size_type slen = length() -1;
      size_type xpos_start = pos < slen ? pos : slen; 
      for (size_type xpos = xpos_start+1; xpos != 0 ; xpos--)
      {
        for(size_type i = 0; i < n ; i++)
          if (traits::eq(_RWdata_.data()[xpos-1], s[i]))
            return xpos-1;
      } 
    }
    return npos;
  }

  template <class charT, class traits , class Allocator  >
  _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::find_first_not_of (const charT* s,
                                                             size_type pos,
                                                             size_type n) const
  {
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
          "basic_string::find_first_not_of(char*,size_t,size_t) const").msgstr());

    for (size_type xpos = pos; xpos < length() ; xpos++)
    {
      bool found = false;
      for (size_type i = 0; i < n ; i++)
      {
        if (traits::eq(_RWdata_.data()[xpos], s[i]))
        {
          found = true;
          break;
        }
      }
      if (!found)
        return xpos;
    }

    return npos;
  }

  template <class charT, class traits , class Allocator  >
  _TYPENAME basic_string<charT, traits, Allocator>::size_type
  basic_string<charT, traits, Allocator>::find_last_not_of(const charT* s,
                                                           size_type pos,
                                                           size_type n) const
  {
    _RWSTD_THROW(s == 0, logic_error,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_UnexpectedNullPtr,
          "basic_string::find_last_not_of(char*,size_t,size_t) const").msgstr());

    if (length())
    {
      size_type slen = length() -1;
      size_type xpos_start = pos < slen ? pos : slen; 
      for (size_type xpos = xpos_start+1; xpos != 0 ; xpos--)
      {
        bool found = false;
        for (size_type i = 0; i < n ; i++)
        {
          if (traits::eq(_RWdata_.data()[xpos-1], s[i]))
          {
            found = true;
            break;
          }
        }
        if (!found)
          return xpos-1;
      }
    }

    return npos;
  }

  template <class charT, class traits, class Allocator >
  basic_string<charT, traits, Allocator>
  basic_string<charT, traits, Allocator>::substr (size_type pos,
                                                  size_type n) const
  {
    _RWSTD_THROW(pos > length(), out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_PosBeyondEndOfString,
          "basic_string::substr(size_t,size_t) const",pos, length()).msgstr());

    size_type slen = length() -pos;
    size_type rlen = n < slen ? n : slen; 
    return basic_string<charT, traits, Allocator>(_RWdata_.data()+pos, rlen);
  }

  template <class charT, class traits, class Allocator >
  int
  basic_string<charT, traits, Allocator>::compare (
      size_type pos1, size_type n1,
      const basic_string<charT, traits, Allocator>& str,
      size_type pos2, size_type n2) const
  {
#if defined(__DECCXX) && !defined(__DECFIXCXXL1298)
#if defined(__DECCXX) && !defined(__DECFIXCXXL1392)
     _RWSTD_THROW(pos1 > length() || pos2 > str.length(),out_of_range,
#else
     _RWSTD_THROW(pos2 > str.length(),out_of_range,
#endif
        __RWSTD::except_msg_string(__RWSTD::_RWrwse_PosBeyondEndOfString,
        "basic_string::compare(size_t,size_t,const basic_string&,size_t,size_t) const", pos2, str.length()).msgstr());

     if(str.length() - pos2 < n2)
        n2 = str.length() - pos2;

     return compare(pos1,n1,str.c_str()+pos2,n2);
#else
    _RWSTD_THROW(pos1+n1 > length() || pos2+n2 > str.length(),out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_PosBeyondEndOfString,
       "basic_string::compare(size_t,size_t,const basic_string&,size_t,size_t) const",
         pos1+n1 > length() ?  (pos1+n1) : (pos2+n2),length() > str.length() ? length(): str.length()).msgstr());

    return compare(pos1,n1,str.c_str()+pos2,n2);
#endif

  }

  template <class charT, class traits, class Allocator >
  int
  basic_string<charT, traits, Allocator>::compare (
      size_type pos1, size_type n1,
      const basic_string<charT, traits, Allocator>& str) const
  {
#if defined(__DECCXX) && !defined(__DECFIXCXXL1298)
#else
    _RWSTD_THROW(pos1+n1 > length() ,out_of_range,
     __RWSTD::except_msg_string(__RWSTD::_RWrwse_PosBeyondEndOfString,
       "basic_string::compare(size_t,size_t,const basic_string&) const",
         pos1+n1 ,length()).msgstr());
#endif

    return compare(pos1,n1,str.c_str(),str.length());
  }

  template <class charT, class traits , class Allocator  >
  int
  basic_string<charT, traits, Allocator>::compare (size_type pos,
                                                   size_type n1,
                                                   const charT* s,
                                                   size_type n2) const
  {
    if(length() - pos < n1)
      n1 = length() - pos;
    size_type rlen = n1 < n2 ? n1 : n2; 
    int result = traits::compare(_RWdata_.data()+pos, s, rlen);

    if (result == 0)
#if defined(__DECCXX) && !defined(__DECFIXCXXL360)
    {
       n1 = (size()-pos < n1) ? (size()-pos) : n1;
#endif
       result = (n1 < n2) ? -1 : (n1 != n2);
#if defined(__DECCXX) && !defined(__DECFIXCXXL360)
    }
#endif

    return result;
  }

  template <class charT, class traits, class Allocator >
  void basic_string<charT, traits, Allocator>::_RWclobber (size_type nc)
  {
    if (_RWpref()->_RWreferences() > 1 || (_RWgetCapac() < nc))
    {
      _RWunLink();
      _RWdata_ = _RWgetRep(nc, 0)->data();
    }
    else
      _RWdata_.data()[_RWpref()->_RWnchars_ = 0] = 0;
  }

  template <class charT, class traits, class Allocator >
  void basic_string<charT, traits, Allocator>::_RWclone()
  {
    _RWstring_ref_type * temp = _RWgetRep(length(), length());
    traits::copy(temp->data(), data(), length());
    _RWunLink();
    _RWdata_ = temp->data();
  }

  template <class charT, class traits, class Allocator >
  void basic_string<charT, traits, Allocator>::_RWclone (size_type nc)
  {
    size_type len = length();
    if (len > nc) len = nc;
    _RWstring_ref_type * temp = _RWgetRep(nc, len);
    traits::copy(temp->data(), data(), length());
    _RWunLink();
    _RWdata_ = temp->data();
  }



#ifdef _RWSTD_MSC22_STATIC_INIT_BUG
#undef npos
#endif

//
// Inserters and Extractors
//

#ifndef _RW_STD_IOSTREAM

  template<class charT, class traits, class Allocator>
  istream & _RWSTDExportTemplate operator>> (istream & is,
                                             basic_string<charT, traits, Allocator > & s)
  {
    int c;

    if (!is.ipfx())
      return is;

    s.erase();

#if defined(__DECCXX) && !defined(__DECFIXCXXL1300)
    c = is.rdbuf()->sgetc();
#else
    c = is.rdbuf()->sbumpc();
#endif

#ifdef _RWSTD_MS40_ISTREAM_BUG
    _TYPENAME size_t i =0;
    _TYPENAME size_t end = s.max_size();
#else
    _TYPENAME Allocator::size_type i   = 0;
    _TYPENAME Allocator::size_type end = s.max_size();
#endif
    if (is.width())
      end =  (int)end < is.width() ? end : is.width(); 

    while (c != EOF && !isspace(c))
    {

#if defined(__DECCXX) && !defined(__DECFIXCXXL1300)
      is.rdbuf()->sbumpc();
#endif

      s.append(1,(charT)c);
      i++;
      if (i == end)
        break;

#if defined(__DECCXX) && !defined(__DECFIXCXXL1300)
      c = is.rdbuf()->sgetc();
#else
      c = is.rdbuf()->sbumpc();
#endif

    }
    if (c == EOF) 
      is.clear(ios::eofbit | is.rdstate());      
    if (!i)
      is.clear(ios::failbit | is.rdstate());      

    is.width(0);
    return is;
  }


  template<class charT, class traits, class Allocator>
  ostream & _RWSTDExportTemplate operator<< (ostream & os,
                                             const basic_string<charT, traits, Allocator > & s)
  {
    os << s.data();
    return os;
  }

  template<class Stream, class charT, class traits, class Allocator>
  Stream& _RWSTDExportTemplate
  getline (Stream& is, basic_string<charT, traits, Allocator>& str, charT delim)
  {
    int c;
    
    if (!is.ipfx(1))
      return is;

    str.erase();
    c = is.rdbuf()->sbumpc();

#ifdef _RWSTD_MS40_ISTREAM_BUG
    size_t i = 0;
    size_t end = str.max_size();
#else
    _TYPENAME Allocator::size_type i = 0;
    _TYPENAME Allocator::size_type end = str.max_size();
#endif

    while (c != EOF)
    {
      i++;
      if ((charT)c == delim)
        break;
      if (i == end)
      {
        is.clear(ios::failbit | is.rdstate());      
        break;
      }
        
      str.append(1,(charT)c);
      c = is.rdbuf()->sbumpc();
    }
    if (c == EOF) 
      is.clear(ios::eofbit | is.rdstate());      
    if (!i)
      is.clear(ios::failbit | is.rdstate());      

    is.isfx();
    return is;
  }  


#endif /* not defined _RW_STD_IOSTREAM */

#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#   pragma __extern_prefix __restore
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __restore
#   endif
#endif
