#ifndef		__TASK_INTERRUPT_HXX
#define		__TASK_INTERRUPT_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	task package
//
//  MODULE:	interrupt
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of Interrupt_handlers and the
//	inline functions which operate on them.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
}

#include <task_object.hxx>
#include <task_sched.hxx>

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//
//
//  EXTERNAL FUNCTIONS
//
//

extern "C" void cxxl_task_handle_signal(int ...);

class Interrupt_handler: public object

//++
//  CLASS DESCRIPTION:
//
//	Interrupt handler
//
//	Changes to this class should also be reflected in
//	Interrupt_handler::print().
//
//--

{
    friend class __cxxl_interrupt_alerter_t;

    friend sched *sched::get_priority_sched();
    friend void sched::cxxl_scheduler();

    friend void cxxl_task_handle_signal(int ...);

private:
    int cxxl_signal_number;		// signal which this object handles
    int cxxl_signal_occurred;		// execute the handler
    Interrupt_handler *cxxl_previous_handler;	// pointer to handler for this
						// signal in effect before this
						// object was created

    virtual void interrupt();

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    static task *__CXXL_XFER_DECL(interrupt_alerter);
						// interrupt alerter task

    static task *__CXXL_XFER_DECL(interrupt_alerter_to_run);	
						// interrupt alerter task
						// (NULL if no handlers need to
						// be executed)

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    static int cxxl_interrupt_handler_exists();

public:
    Interrupt_handler(int);
    ~Interrupt_handler();

    virtual void print(int, int = 0);
    virtual int pending();
    virtual objtype o_type();
};

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// End of task_interrupt.hxx
