#ifndef		__TASK_RAND_HXX
#define		__TASK_RAND_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	task package
//
//  MODULE:	rand
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of random number generators
//	and the inline functions which operate on them.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

//
//
//  LINKAGE
//
//

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif


class randint

//++
//  CLASS DESCRIPTION:
//
//	Uniformly-distributed random number (int in the range 0 to RAND_MAX;
//	float in the range 0 to 1) generator
//--

{

private:
    unsigned long int cxxl_randint_next;	// current value of the seed

public:

    randint(long seed=0);

    int draw();
    void seed(long);

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_FLOAT_FUNC_LINKAGE
#endif

    float fdraw();

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif
};

class urand: public randint

//++
//  CLASS DESCRIPTION:
//
//	Uniformly-distributed random number (within a given range) generator
//--

{

public:
    int low;			// minimum value
    int high;			// maximum value

    urand(int lo, int hi);

    int draw();
};

class erand: public randint

//++
//  CLASS DESCRIPTION:
//
//	Exponentially-distributed random number (with a given mean) generator
//--

{
public:
    int mean;			// mean value

    erand(int m);

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_FLOAT_FUNC_LINKAGE
#endif

    int draw();

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif
};

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// End of task_rand.hxx
