#ifndef		__TASK_SCHED_HXX
#define		__TASK_SCHED_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	task package
//
//  MODULE:	sched
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of scheds and the inline
//	functions which operate on them.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
}

#include <task_object.hxx>

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//
//
//  TYPE DEFINITIONS
//
//

class object;

class sched;

class timer;

class task;

typedef void (*PFV)();



//
//
//  EXTERNAL FUNCTIONS
//
//

#ifdef __CXXL_MODULE_NAME 
extern "C" void cxxl_task_dump();
#endif



//
//
//  EXTERNAL DATA
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

extern const int __CXXL_XFER_DECL(cxxl_task_stack_reserve_area);

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

class sched: public object

//++
//  CLASS DESCRIPTION:
//
//	schedulable object
//
//	Changes to this class should also be reflected in sched::print().
//
//--

{
    friend class object;
    friend class task;
    friend class timer;

    friend class __cxxl_task_thread_t;

#ifdef __CXXL_MODULE_NAME 
    friend void cxxl_task_dump();
#endif

public:
    enum statetype			// state of the schedulable object
    {
	IDLE = 1,
	RUNNING = 2,
	TERMINATED = 4
    };

private:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif


    statetype cxxl_sched_state;		    // Current state of the object
    long cxxl_time;			    // Time at which this schedulable
					    // object should next run
    object *cxxl_who_alerted;		    // Pointer to last object which
					    // alerted this schedulable object
    int cxxl_sched_result;  // set by task::resultis,task::cancel,sched::cancel

    static long __CXXL_XFER_DECL(cxxl_clock); // The current simulated time

    static sched *__CXXL_XFER_DECL(cxxl_run_chain); 
					    // Pointer to the first/next 
					    // runnable schedulable object

    static sched *__CXXL_XFER_DECL(cxxl_last_run_chain);
					    // Pointer to the last runnable
					    // schedulable object

    static int __CXXL_XFER_DECL(cxxl_exit_status);
					    // The current value to be used as
					    // the exit status for the program

    static int __CXXL_XFER_DECL(cxxl_n_waiting_external_events);
					    // Number of waiting external
					    // events which should terminate 
					    // before the main thread can exit 
					    // (from the functions dont_wait, 
					    // keep_waiting)

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    void cxxl_remove_from_run_chain();
    sched *cxxl_remove_from_run_chain_head();
    void cxxl_insert_on_run_chain_head();
    void cxxl_insert_on_run_chain_schedule();
    void cxxl_make_running_next();
    void cxxl_make_running();
    void cxxl_make_idle();
    void cxxl_make_terminated();

    virtual objtype o_type();

protected:

    sched();

public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    static task *__CXXL_XFER_DECL(clock_task);	  // Pointer to task to run 
						  // before updating simulated 
						  // time clock.

    static PFV __CXXL_XFER_DECL(exit_fct);

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    void cancel(int);
    int dont_wait();
    void cxxl_scheduler();
    sched *get_priority_sched();
    int keep_waiting();
    int pending();
    statetype rdstate();
    long rdtime();
    int result();

    virtual void print(int, int = 0);
    virtual void setwho(object *);

    static long get_clock();
    static sched *get_run_chain();
    static int get_exit_status();
    static void set_exit_status(int);
    static void setclock(long);
};



//
//
//  MACRO DEFINITIONS
//
//

// Make function calls look like variable accesses for compatability.
// Parenthesize macro body to obviate lexical problems.

#ifdef CXXL_DEFINE_CLOCK
#define clock		( sched::get_clock() )
#endif
#define run_chain	( sched::get_run_chain() )


#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// End of task_sched.hxx
