#ifndef		__TASK_SYSTEM_SPECIFIC_HXX
#define		__TASK_SYSTEM_SPECIFIC_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	Task package
//
//  MODULE:	task_system_specific
//
//  INFORMATION HIDDEN:
//
//	System-specific information (the CPU architecture & calling standard)
//
//  MODULE DESCRIPTION:
//
//	This module defines the system-independent interface to the functions
//	with system-specific implementations.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

//
//
//  LINKAGE
//
//

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#endif

//
//
//  MACRO DEFINITIONS
//
//

#if defined(__VMS)
// Add '$' to keep names out of user's name space at link time

#define cxxl_callers_argn		__CXXL_PREFIX(callers_argn)
#define cxxl_callers_nargs		__CXXL_PREFIX(callers_nargs)
#define cxxl_callers_nautomatic		__CXXL_PREFIX(callers_nautomatic)
#define cxxl_callers_ret_addr		__CXXL_PREFIX(callers_ret_addr)
#define cxxl_get_sp			__CXXL_PREFIX(get_sp)
#define cxxl_ret_callers_caller		__CXXL_PREFIX(ret_callers_caller)
#define cxxl_save_callers_automatic	__CXXL_PREFIX(save_callers_automatic)
#define cxxl_save_callers_registers	__CXXL_PREFIX(save_callers_registers)
#define cxxl_task_restore_stack_ctx	__CXXL_PREFIX(task_restore_stack_ctx)
#define cxxl_task_save_setup		__CXXL_PREFIX(task_save_setup)
#define cxxl_task_save_stack_ctx	__CXXL_PREFIX(task_save_stack_ctx)
#define cxxl_task_thread_action_jump	__CXXL_PREFIX(task_thread_action_jump)

#if defined(__alpha)
// These symbols are only used on RISC architectures
// for the stack parsing (which we don't want to write in RISC assembly code)

// Pointers to data in the stack frame struct array allocated in C++ code
#define cxxl_p_callers_callers_pc 	__CXXL_PREFIX(_p_callers_callers_pc)
#define cxxl_p_callers_freg	 	__CXXL_PREFIX(_p_callers_freg)
#define cxxl_p_callers_reg	 	__CXXL_PREFIX(_p_callers_reg)
#define cxxl_set_saved_fp		__CXXL_PREFIX(set_saved_fp)

// Data allocated in C++ code
#define cxxl_task_save_double_registers __CXXL_PREFIX(_task_save_double_registers)
#define cxxl_task_save_int_registers 	__CXXL_PREFIX(_task_save_int_registers)

#endif

#endif



//
//
//  EXTERNAL FUNCTIONS
//
//

// See comments in task_system_specific_vms.mar for interface documentation

extern "C"
{
    __cxxl_register_t cxxl_get_sp();
    __cxxl_register_t cxxl_callers_ret_addr();
    int  cxxl_callers_nargs();
    __cxxl_register_t cxxl_callers_argn(int);
    int  cxxl_callers_nautomatic();
    void cxxl_save_callers_automatic(int, __cxxl_register_t *);
    void cxxl_save_callers_registers(__cxxl_register_t *);
    void cxxl_task_save_setup();
#if defined(__VMS)
    void cxxl_ret_callers_caller(__cxxl_register_t);
    void cxxl_task_save_stack_ctx(void **);
    void cxxl_task_restore_stack_ctx(void **);
#endif
    void cxxl_task_thread_action_jump(__cxxl_register_t *, __cxxl_register_t,
	__cxxl_register_t *, int, __cxxl_register_t *, __cxxl_register_t);
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#endif

#endif
// End of task_system_specific.hxx
