#ifndef		__TASK_TASK_HXX
#define		__TASK_TASK_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	task package
//
//  MODULE:	task
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of tasks and the inline
//	functions which operate on them.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
}

#include <task_object.hxx>
#include <task_sched.hxx>

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif



//
//
//  MACRO DEFINITIONS
//
//

// Make function calls look like variable accesses for compatability.
// Parenthesize macro body to obviate lexical problems.
#define thistask	( task::cxxl_get_thistask() )
#define task_chain	( task::get_task_chain() )

// Since the thread system sets up a dedicated stack for each task/thread, this
// parameter is ignored.  (Check it anyway for compatability.)
#ifdef _SHARED_ONLY
#define DEFAULT_MODE SHARED
#else
#define DEFAULT_MODE DEDICATED
#endif

// Number of longwords worth of int & double registers which are saved & restored
// when starting a task (thread fork)
// This is also implicitly known by the associated system-specific module.
// Also, the register number of the first saved (integer) register
#if defined(__VAX)
#   define CXXL_N_SAVED_REGISTERS	11	// task_system_specific_vms.mar
#   define CXXL_FIRST_SAVED_REGISTER	2	// r0 & r1 are not saved
						// ap saved even if not restored
#elif defined(__mips)
#   define CXXL_N_SAVED_REGISTERS	64	// task_system_specific_mips_2.cxx
#   define CXXL_FIRST_SAVED_REGISTER	0	// all registers may be saved
#elif defined(__alpha)
#   define CXXL_N_SAVED_REGISTERS	64	// task_system_specific_alpha_*_2.cxx
#   define CXXL_FIRST_SAVED_REGISTER	0	// all registers may be saved
#else
#   error "Support for this platform is not implemented!"
#endif



//
//
//  TYPE DEFINITIONS
//
//

class object;

class sched;

class timer;

class task;

class __cxxl_task_thread_t;



//
//
//  EXTERNAL FUNCTIONS
//
//

#ifdef __CXXL_MODULE_NAME 
extern "C" void cxxl_task_dump();
#endif



//
//
//  STATIC DATA
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

const int SIZE = 0;	// default stack size in "machine words"
			// 0: use default value

extern int __CXXL_XFER_DECL(_hwm);// non-0: keep track of the stack high water 
				  // mark and have task::print display each 
				  // stack's greatest depth (this is an
				  // implicit input to several functions...)
				  // 0: don't bother with this...
				  // Note: the value need be checked only at
				  // the time of creation of the first task

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif


class task: public sched

//++
//  CLASS DESCRIPTION:
//
//	task (really a coroutine, a thread without real parallelism)
//
//	Changes to this class should also be reflected in task::print().
//
//  DESIGN:
//
//	This is not an abstract base class (achieved by defining a pure
//	virtual member function) because that would require the user's
//	derived class to define this function in order to achieve a
//	non-abstract class.
//--

{
    friend class __cxxl_task_thread_t;

    friend class object;

    friend void sched::cxxl_scheduler();

#ifdef __CXXL_MODULE_NAME 
    friend void cxxl_task_dump();
#endif

public:
    enum modetype			// task stack management mode
    {
	DEDICATED = 1,
	SHARED = 2
    };

private:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    __cxxl_register_t *cxxl_saved_counted_arguments;
					// pointer to counted vector of args
					// ([0]=N, [1]=arg 1, ..., [N]=arg N)
    __cxxl_register_t cxxl_saved_pc;	// program counter
    __cxxl_register_t cxxl_saved_registers[CXXL_N_SAVED_REGISTERS];
					// other registers
    int cxxl_saved_nautomatic;		// number of automatic (stack) data
    __cxxl_register_t *cxxl_saved_automatic;	// automatic (stack) data
    int cxxl_constructor_allocated_name;	// ~task can free name
    __cxxl_register_t cxxl_stack_base;		// addr of stack base
    __cxxl_register_t cxxl_stack_height_max;	// addr of stack top at deepest
    __cxxl_task_thread_t *cxxl_thread_info;	// thread system-specific data

    static task	*__CXXL_XFER_DECL(cxxl_thistask);
					// pointer to currently running task

    static task *__CXXL_XFER_DECL(cxxl_task_chain);
					// pointer to first task on NULL-terminated
					// list of all tasks (NULL if empty list)

    static task *__CXXL_XFER_DECL(cxxl_last_task);
					// pointer to last task on NULL-terminated
					// list of all tasks (NULL if empty list)

    static task *__CXXL_XFER_DECL(cxxl_main_task);
					// pointer to main task (default thread)

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    task(int);
    void cxxl_initialize_cma_main_task();
    void cxxl_remove_from_task_chain();
    void cxxl_initialize_hwm();
    void cxxl_check_hwm();

    static void cxxl_let_main_exit();
    static task *cxxl_get_p_task();

protected:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    task(char *name = (char *)NULL, modetype mode = DEFAULT_MODE,
	int stacksize = SIZE);

public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    task *t_next;			// Pointer to next task in the list of
					// all tasks
    char *t_name;			// Pointer to task name

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    ~task();
    void cancel(int);
    void delay(long);
    long preempt();
    void resultis(int);
    void setwho(object *);
    void sleep(object *object_waiting_for = (object *)NULL);
    void wait(object *);
    int waitlist(object * ...);
    int waitvec(object **);
    object *who_alterted_me();

    virtual void print(int, int = 0);
    virtual objtype o_type();

    inline static task *cxxl_get_thistask();
    inline static task *get_task_chain();
};


task *task::cxxl_get_thistask(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Get a pointer to the currently executing task.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	this
//--

{
    return cxxl_thistask;
}

task *task::get_task_chain(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Get a pointer to the first/next runnable task.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	this
//--

{
    return cxxl_task_chain;
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif


#endif
// End of task_task.hxx
