#ifndef		__TASK_THREAD_HXX
#define		__TASK_THREAD_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	task package
//
//  MODULE:	thread package interface
//
//  INFORMATION HIDDEN:
//
//	The thread system used by the task package.  (This information is shared
//	between task_thread.hxx, task_task.cxx, and task_sched.cxx.)
//
//  MODULE DESCRIPTION:
//
//	This module implements the portion of the representation of tasks which
//	is thread system-specific.  It isolates user compilations from a change
//	in thread systems or thread system interfaces.
//
//--



//
//  INCLUDE FILES
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
#include <cma.h>

#ifndef __ultrix
#include <cma_library.h>
#else
#include <dce/cma_library.h>
#endif

}

#include <task_object.hxx>
#include <task_sched.hxx>


#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//  LINKAGE
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif



//
//
//  TYPE DEFINITIONS
//
//

class object;

class sched;

class timer;

class task;



//
//
//  STATIC DATA
//
//

#if defined(__osf__)
extern cma_t_key cxxl_jmpbuf_key;     // key to get exitting jump buffer
#endif

extern cma_t_key cxxl_task_cma_key;	// key to get task for a given thread

extern cma_t_attr cxxl_thread_attributes;	// thread attributes for a task

extern cma_lib_t_queue cxxl_exit_queue;	// put something in here to let the
					// main task exit when all other tasks
					// have terminated

extern cma_lib_t_attr cxxl_go_queue_attributes;	// thread attributes for an
						// instance of cxxl_go_queue



//
//
//  EXTERNAL FUNCTIONS
//
//

#ifdef __CXXL_MODULE_NAME 
extern "C" void cxxl_task_dump();
#endif

class __cxxl_task_thread_t

//++
//  CLASS DESCRIPTION:
//
//	thread-system-specific information.
//
//	(This was moved from the task class to isolate programs which use the
//	task package from the thread system.)
//
//--

{
    friend class task;

    friend void sched::cxxl_scheduler();

#ifdef __CXXL_MODULE_NAME 
    friend void cxxl_task_dump();
#endif

private:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    cma_t_thread cxxl_task_thread;	// id of thread executing this task
    cma_t_address cxxl_task_thread_result;	// result value for task
    cma_lib_t_queue cxxl_go_queue;	// put something in here to let task run

    static cma_t_address cxxl_task_thread_action(cma_t_address);
};

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// End of task_thread.hxx
