
/***************************************************************************
 *
 * tree.cc - Non-nline tree definitions for the Standard Library
 *
 ***************************************************************************
 *    
 *  Copyright 2000 Compaq Computer Corporation
 *
 *  COMPAQ Registered in U.S. Patent and Trademark Office.
 *
 *  Confidential computer software. Valid license from Compaq required for
 *  possession, use or copying. Consistent with FAR 12.211 and 12.212,
 *  Commercial Computer Software, Computer Software Documentation, and
 *  Technical Data for Commercial Items are licensed to the U.S. Government
 *  under vendor's standard commercial license.
 *
 ****************************************************************************
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Hewlett-Packard Company makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 *
 ***************************************************************************
 *
 * (c) Copyright 1994, 1998 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#include <stdcomp>

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __save
#      pragma __environment __header_defaults
#   endif
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#pragma __extern_prefix __save
#pragma __extern_prefix "CXXL$" 
#endif

#ifndef _RWSTD_NO_NAMESPACE
namespace _RWrwstd {
#endif

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  void _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::_RWdeallocate_buffers ()
  {
    while (_RWbuffer_list.data())
    {
      _RWbuffer_pointer tmp = _RWbuffer_list;
      _RWbuffer_list        = (_RWbuffer_pointer)(_RWbuffer_list.data()->next_buffer);
      _RWnode_alloc_type(_RWbuffer_list).deallocate(tmp->buffer,tmp->size);
      _RWbuffer_alloc_type(_RWbuffer_list).deallocate(tmp,1);
    }
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>& 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::
  operator= (const _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>& x)
  {
    if (!(this == &x))
    {
      //
      // Can't be done as in list because _Key may be a constant type.
      //
      erase(begin(), end());
      _RWroot() = _RWcopy(x._RWroot(), _RWheader);
      if (_RWisNil(_RWroot()))
      {
        _RWleftmost()  = _RWheader; _RWrightmost() = _RWheader;
      }
      else
      {
        _RWleftmost()  = _RWminimum(_RWroot()); _RWrightmost() = _RWmaximum(_RWroot());
      }
      _RWnode_count = x._RWnode_count;
    }
    return *this;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::iterator
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::
  _RWinsert (_RWlink_type x, _RWlink_type y, const _Val& v)
  {
    _RWlink_type z = _RWget_node(v);
    ++_RWnode_count;
    if (y == _RWheader || !_RWisNil(x) || _RWkey_compare(_KeyOf()(v), _RWkey(y)))
    {
      _RWleft(y) = z;  // Also makes _RWleftmost() = z when y == _RWheader.
      if (y == _RWheader)
      {
        _RWroot() = z; _RWrightmost() = z;
      }
      else if (y == _RWleftmost())
        _RWleftmost() = z;   // Maintain _RWleftmost() pointing to minimum node.
    }
    else
    {
      _RWright(y) = z;
      if (y == _RWrightmost())
        _RWrightmost() = z;  // Maintain _RWrightmost() pointing to maximum node.
    }
    _RWparent(z) = y;
    x = z;  // Recolor and rebalance the tree.
    while (x != _RWroot() && _RWcolor(_RWparent(x)) == _RWrb_red) 
    {
      if (_RWparent(x) == _RWleft(_RWparent(_RWparent(x))))
      {
        y = _RWright(_RWparent(_RWparent(x)));
        if (!_RWisNil(y) && _RWcolor(y) == _RWrb_red)
        {
          _RWcolor(_RWparent(x))         = _RWrb_black;
          _RWcolor(y)                 = _RWrb_black;
          _RWcolor(_RWparent(_RWparent(x))) = _RWrb_red;
          x                        = _RWparent(_RWparent(x));
        }
        else
        {
          if (x == _RWright(_RWparent(x)))
          {
            x = _RWparent(x); 
            _RWrotate_left(x);
          }
          _RWcolor(_RWparent(x)) = _RWrb_black;
          _RWcolor(_RWparent(_RWparent(x))) = _RWrb_red;
          _RWrotate_right(_RWparent(_RWparent(x)));
        }
      }
      else
      {
        y = _RWleft(_RWparent(_RWparent(x)));
        if (!_RWisNil(y) && _RWcolor(y) == _RWrb_red)
        {
          _RWcolor(_RWparent(x))         = _RWrb_black;
          _RWcolor(y)                 = _RWrb_black;
          _RWcolor(_RWparent(_RWparent(x))) = _RWrb_red;
          x                        = _RWparent(_RWparent(x));
        }
        else
        {
          if (x == _RWleft(_RWparent(x)))
          {
            x = _RWparent(x); 
            _RWrotate_right(x);
          }
          _RWcolor(_RWparent(x))         = _RWrb_black;
          _RWcolor(_RWparent(_RWparent(x))) = _RWrb_red;
          _RWrotate_left(_RWparent(_RWparent(x)));
        }
      }
    }
    _RWcolor(_RWroot()) = _RWrb_black;
    return iterator(z);
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::pair_iterator_bool
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::insert (const _Val& v)
  {
    _RWlink_type y = _RWheader;
    _RWlink_type x = _RWroot();
    bool _RWSTD_COMP   = true;
    while (!_RWisNil(x))
    {
      y    = x;
      _RWSTD_COMP = _RWkey_compare(_KeyOf()(v), _RWkey(x));
      x    = _RWSTD_COMP ? _RWleft(x) : _RWright(x);
    }
    if (_RWinsert_always)
    {
      pair_iterator_bool tmp(_RWinsert(x, y, v), true); return tmp;
    }
    iterator j = iterator(y);   
    if (_RWSTD_COMP)
    {
      if (j == begin())  
      {
        pair_iterator_bool tmp(_RWinsert(x, y, v), true); return tmp;
      }
      else
        --j;
    }
    if (_RWkey_compare(_RWkey(j.node), _KeyOf()(v)))
    {
      pair_iterator_bool tmp(_RWinsert(x, y, v), true); return tmp;
    }
    pair_iterator_bool tmp(j, false);
    return tmp;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::iterator 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::insert (iterator position,
                                                        const _Val& v)
  {
    if (position == iterator(begin()))
    {
      if (size() > 0 && _RWkey_compare(_KeyOf()(v), _RWkey(position.node)))
        return _RWinsert(position.node, position.node, v);
      //
      // First argument just needs to be non-NIL.
      //
      else
        return insert(v).first;
    }
    else if (position == iterator(end()))
    {
      if (_RWkey_compare(_RWkey(_RWrightmost()), _KeyOf()(v)))
        return _RWinsert(_RWnil(), _RWrightmost(), v);
      else
        return insert(v).first;
    }
    else
    {
      iterator before = --position;
      if (_RWkey_compare(_RWkey(before.node), _KeyOf()(v))
          && _RWkey_compare(_KeyOf()(v), _RWkey(position.node)))
      {
        if (_RWisNil(_RWright(before.node)))
          return _RWinsert(_RWnil(), before.node, v); 
        else
          return _RWinsert(position.node, position.node, v);
        //
        // First argument just needs to be non-NIL.
        //
      }
      else
        return insert(v).first;
    }
  }

#ifndef _RWSTD_NO_MEMBER_TEMPLATES
  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  template<class Iterator>
  void _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::insert (Iterator first, 
                                                             Iterator last)
  {
    while (first != last) insert(*first++);
  }
#else
  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  void _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::insert (const_iterator first, 
                                                             const_iterator last)
  {
    while (first != last) insert(*first++);
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  void _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::insert (const _Val* first, 
                                                             const _Val* last)
  {
    while (first != last) insert(*first++);
  }
#endif
         
  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::iterator 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::erase (iterator position)
  {
    iterator tmp = position; // Retain 'next' position for return
    if (tmp != end())
      tmp++;

    _RWlink_type z;
    _RW_STD::_RWinitialize(z, _RWlink_type(position.node));
    _RWlink_type y = z;
    _RWlink_type x;
    bool _RWdeleted = false;
    if (_RWisNil(_RWleft(y)))
    {
      if (_RWisNil(_RWright(y)))
      {
        x = _RWparent(y);
        _RWerase_leaf(y);
        _RWdeleted = true;
      }
      else
        x = _RWright(y); 
    }
    else
    {
      if (_RWisNil(_RWright(y))) 
        x = _RWleft(y);
      else
      {
        y = _RWright(y);
        while (!_RWisNil(_RWleft(y))) y = _RWleft(y);
        x = _RWright(y);
      }
    }
    if (!_RWdeleted && y != z)
    {
      //
      // Relink y in place of z.
      //
      _RWparent(_RWleft(z)) = y; 
      _RWleft(y) = _RWleft(z);
      if (y != _RWright(z))
      {
        if (!_RWisNil(x))
          _RWparent(x)        = _RWparent(y);
        _RWleft(_RWparent(y))  = x;         // Y must be a left child.
        _RWright(y)         = _RWright(z);
        _RWparent(_RWright(z)) = y;
      }
      else if (!_RWisNil(x))
        _RWparent(x) = y;  

      if (_RWroot() == z)
        _RWroot() = y;
      else if (_RWleft(_RWparent(z)) == z)
        _RWleft(_RWparent(z)) = y;
      else 
        _RWright(_RWparent(z)) = y;

      _RWparent(y) = _RWparent(z);
      if (_RWisNil(x))
        x = y;       // Don't want to start balancing with nil
#ifndef _RWSTD_NO_NAMESPACE
      std::swap(_RWcolor(y), _RWcolor(z));
#else
      ::swap(_RWcolor(y), _RWcolor(z));
#endif
      y = z;        // y points to node to be actually deleted.
    }
    else if (!_RWdeleted)
    {
      //
      // y == z
      //
      _RWparent(x) = _RWparent(y);   
      if (_RWroot() == z)
        _RWroot() = x;
      else 
      {
        if (_RWleft(_RWparent(z)) == z)
          _RWleft(_RWparent(z)) = x;
        else
          _RWright(_RWparent(z)) = x;
      }
      if (_RWleftmost() == z) 
      {
        if (_RWisNil(_RWright(z)))  // _RWleft(z) must be NIL also.
          _RWleftmost() = _RWparent(z);
        else
          _RWleftmost() = _RWminimum(x);
      }
      if (_RWrightmost() == z)  
      {
        if (_RWisNil(_RWleft(z))) // _RWright(z) must be NIL also.
          _RWrightmost() = _RWparent(z);
        else
          _RWrightmost() = _RWmaximum(x);
      }
    }
    if (x != _RWheader && _RWcolor(y) != _RWrb_red)
    { 
      while (x != _RWroot() && _RWcolor(x) == _RWrb_black)
      {
        if (x == _RWleft(_RWparent(x)))
        {
          _RWlink_type w = _RWright(_RWparent(x));
          if (_RWisNil(w))
          {
            _RWcolor(x) = _RWrb_red;
            x = _RWparent(x);
          }
          else
          {
            if (_RWcolor(w) == _RWrb_red)
            {
              _RWcolor(w)         = _RWrb_black;
              _RWcolor(_RWparent(x)) = _RWrb_red;
              _RWrotate_left(_RWparent(x));
              w = _RWright(_RWparent(x));
            }
            if (_RWisNil(w))
            {
              _RWcolor(x) = _RWrb_red;
              x = _RWparent(x);
            }
            else if ((_RWisNil(_RWleft(w)) || _RWcolor(_RWleft(w)) == _RWrb_black) && 
                (_RWisNil(_RWright(w)) || _RWcolor(_RWright(w)) == _RWrb_black))
            {
              _RWcolor(w) = _RWrb_red;
              x = _RWparent(x);
            }
            else
            {
              if (_RWisNil(_RWright(w)) || _RWcolor(_RWright(w)) == _RWrb_black)
              {
                if (!_RWisNil(_RWleft(w)))
                  _RWcolor(_RWleft(w)) = _RWrb_black;
                _RWcolor(w)       = _RWrb_red;
                _RWrotate_right(w);
                w = _RWright(_RWparent(x));
              }
              if (!_RWisNil(w))
              {
                _RWcolor(w) = _RWcolor(_RWparent(x));
                _RWcolor(_RWparent(x)) = _RWrb_black;
                if (!_RWisNil(_RWright(w)))
                  _RWcolor(_RWright(w))  = _RWrb_black;
                _RWrotate_left(_RWparent(x));
              }          
              break;
            }
          }
        }
        else
        {
          //
          // Same as then clause with "right" and "left" exchanged.
          //
          _RWlink_type w = _RWleft(_RWparent(x));
          if (_RWisNil(w))
          {
            _RWcolor(x) = _RWrb_red;
            x = _RWparent(x);
          }
          else
          {
            if (_RWcolor(w) == _RWrb_red)
            {
              _RWcolor(w)         = _RWrb_black;
              _RWcolor(_RWparent(x)) = _RWrb_red;
              _RWrotate_right(_RWparent(x));
              w = _RWleft(_RWparent(x));
            }
            if (_RWisNil(w))
            {
              _RWcolor(x) = _RWrb_red;
              x = _RWparent(x);
            }
            else if ((_RWisNil(_RWright(w)) || _RWcolor(_RWright(w)) == _RWrb_black) &&
              (_RWisNil(_RWleft(w)) || _RWcolor(_RWleft(w)) == _RWrb_black))
            {
              _RWcolor(w) = _RWrb_red; x = _RWparent(x);
            }
            else
            {
              if (_RWisNil(_RWleft(w)) || _RWcolor(_RWleft(w)) == _RWrb_black)
              {
                if (!_RWisNil(_RWright(w)))
                  _RWcolor(_RWright(w)) = _RWrb_black;
                _RWcolor(w)        = _RWrb_red;
                _RWrotate_left(w);
                w = _RWleft(_RWparent(x));
              }
              if (!_RWisNil(w))
              {
                _RWcolor(w) = _RWcolor(_RWparent(x));
                _RWcolor(_RWparent(x)) = _RWrb_black;
                if (!_RWisNil(_RWleft(w)))
                  _RWcolor(_RWleft(w))   = _RWrb_black;
                _RWrotate_right(_RWparent(x));
              }
              break;
            }
          }
        }
      }
      _RWcolor(x) = _RWrb_black;
    }
    _RWput_node(y);
    --_RWnode_count;
    return tmp;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::size_type 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::erase (const _Key& x)
  {
    pair_iterator_iterator p = equal_range(x);
    size_type n;
    _RW_STD::_RWinitialize(n, size_type(0));
    _RW_STD::distance(p.first, p.second, n);
    erase(p.first, p.second);
    return n;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::_RWlink_type 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::_RWcopy (_RWlink_type x, _RWlink_type p)
  {
    //
    // Structural copy.
    // 
    _RWlink_type r = x;
    while (!_RWisNil(x))
    {
      _RWlink_type y = _RWget_node(_RWvalue(x));
      if (r == x) r = y;  // Save for return value.
      _RWleft(p)   = y;
      _RWparent(y) = p;
      _RWcolor(y)  = _RWcolor(x);
      _RWright(y)  = _RWcopy(_RWright(x), y);
      p         = y;
      x         = _RWleft(x);
    }
    _RWleft(p) = _RWnil();
    return r;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  void _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::_RWerase (_RWlink_type x)
  {
    //
    // Erase without rebalancing.
    //
    while (!_RWisNil(x))
    {
      _RWerase(_RWright(x));
      _RWlink_type y = _RWleft(x);
      _RWput_node(x);
      x = y;
    }
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::iterator 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::erase (iterator first, 
                                                       iterator locallast)
  {
    iterator tmp = end();
    if (first == begin() && locallast == end() && _RWnode_count != 0)
    {
      _RWerase(_RWroot());
      _RWleftmost()  = _RWheader;
      _RWroot()      = _RWnil();
      _RWrightmost() = _RWheader;
      _RWnode_count  = 0;
      tmp = end();
    } else
    {
      while (first != locallast) 
        tmp =  erase(first++);
    }
    return tmp;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  void _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::erase (const _Key* first, 
                                                            const _Key* last)
  {
    while (first != last) erase(*first++);
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::iterator 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::find (const _Key& k) const
  {
    _RWlink_type y = _RWheader;  // Last node that is not less than k.
    _RWlink_type x = _RWroot();  // Current node.

    while (!_RWisNil(x))
    {
      if (!_RWkey_compare(_RWkey(x), k))
        y = x, x = _RWleft(x);
      else
        x = _RWright(x);
    }
    iterator j = iterator(y);
    return (j == end() || _RWkey_compare(k, _RWkey(j.node))) ? end() : j;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::size_type 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::count (const _Key& k) const
  {
    _RW_STD::pair<iterator, iterator> p = equal_range(k);
    size_type n;
    _RW_STD::_RWinitialize(n, size_type(0));
    _RW_STD::distance(p.first, p.second, n);
    return n;
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::iterator 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::lower_bound (const _Key& k) const
  {
    _RWlink_type y = _RWheader;  // Last node that is not less than k.
    _RWlink_type x = _RWroot();  // Current node.

    while (!_RWisNil(x))
    {
      if (!_RWkey_compare(_RWkey(x), k))
        y = x, x = _RWleft(x);
      else
        x = _RWright(x);
    }

    return iterator(y);
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::iterator 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::upper_bound (const _Key& k) const
  {
    _RWlink_type y = _RWheader;  // Last node that is greater than k.
    _RWlink_type x = _RWroot();  // Current node.

    while (!_RWisNil(x))
    {
      if (_RWkey_compare(k, _RWkey(x)))
        y = x, x = _RWleft(x);
      else
        x = _RWright(x);
    }

    return iterator(y);
  }

  template <class _Key, class _Val, class _KeyOf, class _Comp, class _Alloc>
  _TYPENAME _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::pair_iterator_iterator 
  _RWrb_tree<_Key, _Val, _KeyOf, _Comp, _Alloc>::equal_range (const _Key& k) const
  {
    pair_iterator_iterator tmp(lower_bound(k), upper_bound(k));
    return tmp;
  }

#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#   pragma __extern_prefix __restore
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __restore
#   endif
#endif
