/***************************************************************************
 *
 * vector.cc - Non-inline definitions for the Standard Library vector class
 *
 ***************************************************************************
 *    
 *  Copyright 2000 Compaq Computer Corporation
 *
 *  COMPAQ Registered in U.S. Patent and Trademark Office.
 *
 *  Confidential computer software. Valid license from Compaq required for
 *  possession, use or copying. Consistent with FAR 12.211 and 12.212,
 *  Commercial Computer Software, Computer Software Documentation, and
 *  Technical Data for Commercial Items are licensed to the U.S. Government
 *  under vendor's standard commercial license.
 *
 ****************************************************************************
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Hewlett-Packard Company makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 *
 ***************************************************************************
 *
 * (c) Copyright 1994, 1998 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/
#include <stdcomp>

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __save
#      pragma __environment __header_defaults
#   endif
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#pragma __extern_prefix __save
#pragma __extern_prefix "CXXL$" 
#endif

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif

//
// This requires that T have a default constructor.
//

  template <class T, class Allocator>
  void vector<T,Allocator>::resize (size_type new_size)
  {
#if defined(__DECCXX) && !defined(__DECFIXCXXL906)
    T value = T();
#else
    T value;
#endif
    if (new_size > size())
      insert(end(), new_size - size(), value);
    else if (new_size < size())
      erase(begin() + new_size, end());
  }

  template <class T, class Allocator>
  void vector<T,Allocator>::resize (size_type new_size, T value)
  {
    if (new_size > size())
      insert(end(), new_size - size(), value);
    else if (new_size < size())
      erase(begin() + new_size, end());
  }

  template <class T, class Allocator>
  vector<T,Allocator>& vector<T,Allocator>::operator= (const vector<T,Allocator>& x)
  {
    if (&x == this) return *this;
    if (x.size() > capacity())
    {
      _RWvalue_alloc_type va(_RWend_of_storage);
      iterator tmp = va.allocate(x.end() - x.begin(),0);
#ifndef _RWSTD_NO_EXCEPTIONS
      try {
        _RWend_of_storage = uninitialized_copy(x.begin(), x.end(), tmp);
      } catch(...) {
        va.deallocate(tmp,x.end()-x.begin());
        throw;
      }
#else
      _RWend_of_storage = uninitialized_copy(x.begin(), x.end(), tmp);
#endif // _RWSTD_NO_EXCEPTIONS
      _RWdestroy(_RWstart, _RWfinish);
      va.deallocate(_RWstart,_RWend_of_storage.data()-_RWstart);
      _RWstart = tmp;
    }
    else if (size() >= x.size())
    {
      iterator i = copy(x.begin(), x.end(), begin());
      _RWdestroy(i, _RWfinish);
    }
    else
    {
      copy(x.begin(), x.begin() + size(), begin());
      uninitialized_copy(x.begin() + size(), x.end(), begin() + size());
    }
    _RWfinish = begin() + x.size();
    return *this;
  }

  template <class T, class Allocator>
  void vector<T,Allocator>::_RWinsert_aux (
      iterator position, const T& x)
  {
    if (_RWfinish != _RWend_of_storage.data())
    {
      _RWvalue_alloc_type(_RWend_of_storage).construct(_RWfinish, *(_RWfinish - 1));
      copy_backward(position, _RWfinish - 1, _RWfinish);
      *position = x;
      ++_RWfinish;
    }
    else
    {
      //
      // We always allocate enough space for a number of additional
      // elements in the vector, unless the size of each element is
      // very large. See definition of _RWrw_allocation_size in
      // memory.
      //
      _RWvalue_alloc_type va(_RWend_of_storage);
      size_type len = __RWSTD::_RWrw_allocation_size((value_type*)0,size(),_RWbuffer_size);
      iterator tmp = va.allocate(len,_RWstart);
      uninitialized_copy(begin(), position, tmp);
      va.construct((tmp + (position - begin())), x);
#ifndef _RWSTD_NO_EXCEPTIONS
      try {
        uninitialized_copy(position, end(), tmp + (position - begin()) + 1); 
      } catch(...) {
        va.deallocate(tmp,len);
        throw;
      }
#else
      uninitialized_copy(position, end(), tmp + (position - begin()) + 1); 
#endif // _RWSTD_NO_EXCEPTIONS
      _RWdestroy(begin(), end());
      va.deallocate(begin(),_RWend_of_storage.data() - begin());
      _RWend_of_storage = tmp + len;
      _RWfinish = tmp + size() + 1;
      _RWstart = tmp;
    }
  }

  template <class T, class Allocator>
  void vector<T,Allocator>::_RWinsert_aux (
      iterator position, size_type n, const T& x)
  {
    if (n == 0) return;
    if ((size_type)(_RWend_of_storage.data() - _RWfinish) >= n)
    {
      iterator old_end = end();
      if ((size_type)(end() - position) > n)
      {
        uninitialized_copy(old_end - n, old_end, old_end);
        _RWfinish += n;
        copy_backward(position, old_end - n, old_end);
        fill(position, position + n, x);
      }
      else
      {
        size_type first_part = n - (old_end - position);
        uninitialized_fill_n(old_end, first_part, x);
        _RWfinish += first_part;
        uninitialized_copy(position, old_end, position + n);
        _RWfinish += n - first_part;
        fill(position, old_end, x);

      }

    }
    else
    {
      _RWvalue_alloc_type va(_RWend_of_storage);
      size_type len = size() + max(size(), n);
      iterator tmp = va.allocate(len,_RWstart);
#ifndef _RWSTD_NO_EXCEPTIONS
      try {
        uninitialized_copy(begin(), position, tmp);
        uninitialized_fill_n(tmp + (position - begin()), n, x);
        uninitialized_copy(position, end(), tmp + (position - begin() + n));
      } catch(...) {
        va.deallocate(tmp,len);
        throw;
      }
#else
      uninitialized_copy(begin(), position, tmp);
      uninitialized_fill_n(tmp + (position - begin()), n, x);
      uninitialized_copy(position, end(), tmp + (position - begin() + n));
#endif // _RWSTD_NO_EXCEPTIONS
      _RWdestroy(begin(), end());
      va.deallocate(begin(),_RWend_of_storage.data() - begin());
      _RWend_of_storage = tmp + len;
      _RWfinish = tmp + size() + n;
      _RWstart = tmp;
    }
  }


#if defined(__DECCXX) && !defined(__DECFIXCXXL1294)
#else
#ifndef _RWSTD_NO_MEMBER_TEMPLATES
  template<class T, class Allocator>
  template<class InputIterator>
  void vector<T,Allocator>::insert (iterator position, 
                                    InputIterator first, 
                                    InputIterator last)
#else
  template<class T, class Allocator>
  void vector<T,Allocator>::insert (iterator position,
                                    const_iterator first,
                                    const_iterator last)
#endif
  {
    if (first == last) return;
    size_type n;
    _RWinitialize(n, size_type(0));
    distance(first, last, n);

    if ((size_type)(_RWend_of_storage.data() - _RWfinish) >= n)
    {
      iterator old_end = end();
      if ((size_type)(old_end - position) > n)
      {
        uninitialized_copy(old_end - n, old_end, old_end);
        _RWfinish += n;
        copy_backward(position, old_end - n, old_end);
        copy(first, last, position);
      }
      else
      {
        size_type first_part = (old_end - position);
        uninitialized_copy(first + first_part, last, old_end);
        _RWfinish += n - first_part;
        uninitialized_copy(position, old_end, position + n);
        _RWfinish += first_part;
        copy(first, first + (old_end - position), position);
      }

    }
    else
    {
      _RWvalue_alloc_type va(_RWend_of_storage);
      size_type len = size() + max(size(), n);
      iterator tmp = va.allocate(len,_RWstart);
#ifndef _RWSTD_NO_EXCEPTIONS
      try {
        uninitialized_copy(begin(), position, tmp);
        uninitialized_copy(first, last, tmp + (position - begin()));
        uninitialized_copy(position, end(), tmp + (position - begin() + n));
      } catch(...) {
        va.deallocate(tmp,len);
        throw;
      }
#else
      uninitialized_copy(begin(), position, tmp);
      uninitialized_copy(first, last, tmp + (position - begin()));
      uninitialized_copy(position, end(), tmp + (position - begin() + n));
#endif // _RWSTD_NO_EXCEPTIONS
      _RWdestroy(begin(), end());
      va.deallocate(begin(),_RWend_of_storage.data() - begin());
      _RWend_of_storage = tmp + len;
      _RWfinish = tmp + size() + n;
      _RWstart = tmp;
    }
  }
#endif // 1294

#if defined(__DECCXX) && !defined(__DECFIXCXXL1294)
#ifndef _RWSTD_NO_MEMBER_TEMPLATES
  template<class T, class Allocator>
  template<class InputIterator>
  void vector<T,Allocator>::_RWinsert_aux2 (iterator position, 
                                    InputIterator first, 
                                    InputIterator last)
#else
  template<class T, class Allocator>
  void vector<T,Allocator>::_RWinsert_aux2 (iterator position,
                                    const_iterator first,
                                    const_iterator last)
#endif
  {
    if (first == last) return;
    size_type n;
    _RWinitialize(n, size_type(0));
    distance(first, last, n);

    if ((size_type)(_RWend_of_storage.data() - _RWfinish) >= n)
    {
      iterator old_end = end();
      if ((size_type)(old_end - position) > n)
      {
        uninitialized_copy(old_end - n, old_end, old_end);
        _RWfinish += n;
        copy_backward(position, old_end - n, old_end);
        copy(first, last, position);
      }
      else
      {
        size_type first_part = (old_end - position);
#ifndef _RWSTD_NO_MEMBER_TEMPLATES
        InputIterator iter(first);
#else
        const_iterator iter(first);
#endif
        advance(iter, first_part);
        uninitialized_copy(iter, last, old_end);
//        uninitialized_copy(first + first_part, last, old_end);
        _RWfinish += n - first_part;
        uninitialized_copy(position, old_end, position + n);
        _RWfinish += first_part;
        copy(first, iter, position);
//        copy(first, first + (old_end - position), position);
      }

    }
    else
    {
      _RWvalue_alloc_type va(_RWend_of_storage);
      size_type len = size() + max(size(), n);
      iterator tmp = va.allocate(len,_RWstart);
#ifndef _RWSTD_NO_EXCEPTIONS
      try {
        uninitialized_copy(begin(), position, tmp);
        uninitialized_copy(first, last, tmp + (position - begin()));
        uninitialized_copy(position, end(), tmp + (position - begin() + n));
      } catch(...) {
        va.deallocate(tmp,len);
        throw;
      }
#else
      uninitialized_copy(begin(), position, tmp);
      uninitialized_copy(first, last, tmp + (position - begin()));
      uninitialized_copy(position, end(), tmp + (position - begin() + n));
#endif // _RWSTD_NO_EXCEPTIONS
      _RWdestroy(begin(), end());
      va.deallocate(begin(),_RWend_of_storage.data() - begin());
      _RWend_of_storage = tmp + len;
      _RWfinish = tmp + size() + n;
      _RWstart = tmp;
    }
  }
#endif // 1294

#ifndef _RWSTD_NO_MEMBER_TEMPLATES
// The body of this function is duplicated in src/vecbool.cpp and
// further down in this file as well.
#ifndef _RWSTD_NO_CLASS_PARTIAL_SPEC
  template <class Allocator>
  template<class InputIterator>
  void vector<bool, Allocator >::insert 
#else
  template<class InputIterator>
  void vector<bool, allocator<bool> >::insert 
#endif // _RWSTD_NO_CLASS_PARTIAL_SPEC
  (iterator position, 
   InputIterator first,
   InputIterator last)

  {
    if (first == last) return;
    size_type n;
    _RWinitialize(n, size_type(0));
    distance(first, last, n);
    if (capacity() - size() >= n)
    {
      copy_backward(position, end(), _RWfinish + n);
      copy(first, last, position);
      _RWfinish += n;
    }
    else
    {
      size_type len = size() + max(size(), n);
      unsigned int* q = _RWbit_alloc(len);
      iterator i = copy(begin(), position, iterator(q, 0));
      i = copy(first, last, i);
      _RWfinish = copy(position, end(), i);
      _RWvalue_alloc_type(_RWend_of_storage).deallocate(_RWstart.p,_RWend_of_storage.data() - _RWstart.p);
      _RWend_of_storage = q + (len + _RWSTD_WORD_BIT - 1)/_RWSTD_WORD_BIT;
      _RWstart = iterator(q, 0);
    }
  }
#endif // _RWSTD_NO_MEMBER_TEMPLATES

#ifndef _RWSTD_NO_CLASS_PARTIAL_SPEC
// Duplicates of the followign functions exist in src/stl/vecbool.cpp.
// Which set is used depends on the availability of partial specialization.

  template <class Allocator>
  void vector<bool,Allocator >::flip ()
  {
    for (iterator i = begin(); !(i == end()); i++)
      *i = !(*i);
  }

  template <class Allocator>
  void vector<bool,Allocator >::swap (reference x, reference y)
  {
    bool tmp = x; x = y; y = tmp;
  }

  template <class Allocator>
  void vector<bool,Allocator >::_RWinsert_aux (iterator position, bool x)
  {
    if (_RWfinish.p != _RWend_of_storage.data())
    {
      _RWcopy_backward(position, _RWfinish - 1, _RWfinish);
      *position = x;
      ++_RWfinish;
    }
    else
    {
      size_type len = size() ? 2 * size() : _RWSTD_WORD_BIT;
      unsigned int* q = _RWbit_alloc(len);
      iterator i = _RWcopy(begin(), position, iterator(q, 0));
      *i++ = x;
      _RWfinish = _RWcopy(position, end(), i);
      _RWvalue_alloc_type(_RWend_of_storage).deallocate(_RWstart.p,_RWend_of_storage.data() - _RWstart.p);
      _RWend_of_storage = q + (len + _RWSTD_WORD_BIT - 1)/_RWSTD_WORD_BIT;
      _RWstart = iterator(q, 0);
    }
  }

  template <class Allocator>
  void vector<bool,Allocator >::insert (iterator position, size_type n, const bool& x)
  {
    if (n == 0) return;
    if (capacity() - size() >= n)
    {
      _RWcopy_backward(position, end(), _RWfinish + n);
      _RWfill(position, position + n, x);
      _RWfinish += n;
    }
    else
    {
      size_type len = size() + max(size(), n);
      unsigned int* q = _RWbit_alloc(len);
      iterator i = _RWcopy(begin(), position, iterator(q, 0));
      _RWfill_n(i, n, x);
      _RWfinish = _RWcopy(position, end(), i + n);
      _RWvalue_alloc_type(_RWend_of_storage).deallocate(_RWstart.p,_RWend_of_storage.data() - _RWstart.p);
      _RWend_of_storage = q + (len + _RWSTD_WORD_BIT - 1)/_RWSTD_WORD_BIT;
      _RWstart = iterator(q, 0);
    }
  }


#ifdef _RWSTD_NO_MEMBER_TEMPLATES
  template <class Allocator>
  void vector<bool,Allocator >::insert (iterator position, const_iterator first,
                                        const_iterator last)
  {
    if (first == last) return;
    size_type n;
    _RWinitialize(n, size_type(0));
    distance(first, last, n);
    if (capacity() - size() >= n)
    {
      _RWcopy_backward(position, end(), _RWfinish + n);
      _RWcopy(first, last, position);
      _RWfinish += n;
    }
    else
    {
      size_type len = size() + max(size(), n);
      unsigned int* q = _RWbit_alloc(len);
      iterator i = _RWcopy(begin(), position, iterator(q, 0));
      i = _RWcopy(first, last, i);
      _RWfinish = _RWcopy(position, end(), i);
      _RWvalue_alloc_type(_RWend_of_storage).deallocate(_RWstart.p,_RWend_of_storage.data() - _RWstart.p);
      _RWend_of_storage = q + (len + _RWSTD_WORD_BIT - 1)/_RWSTD_WORD_BIT;
      _RWstart = iterator(q, 0);
    }
  }
#endif // _RWSTD_NO_MEMBER_TEMPLATES

  template <class Allocator>
  void vector<bool,Allocator >::resize (size_type new_size, bool c)
  {
    if (new_size > size())
      insert(end(), new_size - size(), c);
    else if (new_size < size())
      erase(begin() + new_size, end());
  }

#endif // _RWSTD_NO_CLASS_PARTIAL_SPEC

#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#if defined(__VMS) && defined(__DECCXX) && !defined(__DECFIXCXXL1158)
#   pragma __extern_prefix __restore
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __restore
#   endif
#endif
